@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.10 2004/02/28 10:40:56 ryo_saeba Exp $";
+ *   "$Id: image.c,v 1.11 2005/03/19 09:14:56 mwedel Exp $";
  */
 
 /*
@@ -60,7 +60,8 @@ MapLook blank_look;
 /* Following can just as easily be pointers, but 
  * it is easier to keep them like this.
  */
-New_Face *blank_face, *dark_faces[3], *potion_face, *empty_face;
+New_Face *blank_face, *dark_faces[3], *potion_face, *empty_face,
+    *smooth_face;
 
 
 /* nroffiles is the actual number of bitmaps defined.
@@ -312,6 +313,8 @@ int ReadBmapNames () {
 
     potion_face  = &new_faces[FindFace(POTION_FACE_NAME,0)];
 
+    smooth_face = &new_faces[FindFace(SMOOTH_FACE_NAME,0)];
+
     return nrofpixmaps;
 }
 
