@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.34 2004/02/16 20:03:10 tanner Exp $";
+ *   "$Id: init.c,v 1.41 2005/07/15 13:51:48 ryo_saeba Exp $";
  */
 
 /*
@@ -35,7 +35,7 @@
  * correspond to.
  */
 struct Settings settings = {
-"/var/log/crossfire/logfile",   /* Logfile */
+LOGFILE,   /* Logfile */
 CSPORT,				/* Client/server port */
 
 /* Debug level */
@@ -51,9 +51,10 @@ NULL, /* argv */
 CONFDIR,
 DATADIR, 
 LOCALDIR,
-PLAYERDIR, MAPDIR, ARCHETYPES,TREASURES, 
+PLAYERDIR, MAPDIR, ARCHETYPES,REGIONS,TREASURES, 
 UNIQUE_DIR, TMPDIR,
 STAT_LOSS_ON_DEATH,
+PK_LUCK_PENALTY,
 USE_PERMANENT_EXPERIENCE,
 BALANCED_STAT_LOSS,
 NOT_PERMADETH,
@@ -70,7 +71,11 @@ RECYCLE_TMP_MAPS,
 EXPLORE_MODE,
 SPELLPOINT_LEVEL_DEPEND,
 SET_FRIENDLY_FIRE,
+"", /* Who format specifier */
+"", /* who wiz format specifier */
 MOTD,
+"rules",
+"news",
 "",		/* DM_MAIL */
 0,		/* This and the next 3 values are metaserver values */
 "",
@@ -80,7 +85,13 @@ MOTD,
 0,0,0,0,0,0,0,0,  /* worldmap settings*/
 EMERGENCY_MAPPATH, EMERGENCY_X, EMERGENCY_Y,
 0,
-1.0
+1.0,
+/* Armor enchantment stuff */
+ARMOR_MAX_ENCHANT,
+ARMOR_WEIGHT_REDUCTION,
+ARMOR_WEIGHT_LINEAR,
+ARMOR_SPEED_IMPROVEMENT,
+ARMOR_SPEED_LINEAR
 };
 
 /* perhaps not the best place for this, but needs to be
@@ -165,7 +176,6 @@ static void init_emergency_mappath()
 void init_library() {
     init_environ();
     init_globals();
-    init_function_pointers();
     init_hash_table();
     init_objects();
     init_vars();
