@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.43 2004/04/26 05:07:52 mwedel Exp $";
+ *   "$Id: item.c,v 1.48 2005/07/15 13:51:48 ryo_saeba Exp $";
  */
 
 /*
@@ -244,7 +244,7 @@ char *describe_resistance(object *op, int newline)
 
 char *query_weight(object *op) {
   static char buf[10];
-  int i=op->nrof?op->nrof*op->weight:op->weight+op->carrying;
+  sint32 i=op->nrof?op->nrof*op->weight:op->weight+op->carrying;
 
   if(op->weight<0)
     return "      ";
@@ -707,6 +707,8 @@ static char *describe_monster(object *op) {
 	strcat(retbuf,"(unaggressive)");
     if(QUERY_FLAG(op,FLAG_HITBACK))
 	strcat(retbuf,"(hitback)");
+    if(QUERY_FLAG(op,FLAG_STEALTH))
+	strcat(retbuf,"(stealthy)");
     if(op->randomitems != NULL) {
 	treasure *t;
 	int first = 1;
@@ -739,6 +741,10 @@ static char *describe_monster(object *op) {
 	    sprintf(buf,"(regeneration%+d)",op->contr->gen_hp);
 	    strcat(retbuf,buf);
 	}
+	if(op->stats.luck) {
+	    sprintf(buf,"(luck%+d)",op->stats.luck);
+	    strcat(retbuf,buf);
+	}
     }
 
     /* describe attacktypes */
@@ -1033,7 +1039,7 @@ char *describe_item(object *op, object *owner) {
 	    strcat(retbuf,"(reflect missiles)");
 	if(QUERY_FLAG(op,FLAG_STEALTH))
 	    strcat(retbuf,"(stealth)");
-	if(op->slaying!=NULL) {
+	if(op->slaying!=NULL && op->type != FOOD) {
 	    sprintf(buf,"(slay %s)",op->slaying);
 	    strcat(retbuf,buf);
 	}
@@ -1213,6 +1219,6 @@ void identify(object *op) {
 	    /* A lot of the values can change from an update - might as well send
 	     * it all.
 	     */
-	    esrv_send_item_func(pl, op);
+	    esrv_send_item(pl, op);
     }
 }
