@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c,v 1.63 2004/04/10 07:27:56 mwedel Exp $";
+ *   "$Id: living.c,v 1.71 2005/08/10 13:38:31 ryo_saeba Exp $";
  */
 
 /*
@@ -110,7 +110,7 @@ int max_carry[MAX_STAT + 1]={
  * before, you need to start someplace.
  */
 
-int weight_limit[MAX_STAT+ 1] = {
+uint32 weight_limit[MAX_STAT+ 1] = {
     200000,  /* 0 */
     250000,300000,350000,400000,500000,	    /* 5*/
     600000,700000,800000,900000,1000000,    /* 10 */
@@ -425,32 +425,32 @@ int change_abil(object *op, object *tmp) {
   if(tmp->attacktype & AT_CONFUSION && tmp->type != BOW) {
     success=1;
     if(flag>0)
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"Your hands begin to glow red.");
+      new_draw_info(NDI_UNIQUE, 0, op,"Your hands begin to glow red.");
     else
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"Your hands stop glowing red.");
+      new_draw_info(NDI_UNIQUE, 0, op,"Your hands stop glowing red.");
   }
   if ( QUERY_FLAG(op,FLAG_LIFESAVE) != QUERY_FLAG(&refop,FLAG_LIFESAVE)){
     success=1;
     if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel very protected.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel very protected.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You don't feel protected anymore.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You don't feel protected anymore.");
     }
   }
   if ( QUERY_FLAG(op,FLAG_REFL_MISSILE) != QUERY_FLAG(&refop,FLAG_REFL_MISSILE)){
     success=1;
     if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"A magic force shimmers around you.");
+      new_draw_info(NDI_UNIQUE, 0, op,"A magic force shimmers around you.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"The magic force fades away.");
+      new_draw_info(NDI_UNIQUE, 0, op,"The magic force fades away.");
     }
   }
   if ( QUERY_FLAG(op,FLAG_REFL_SPELL) != QUERY_FLAG(&refop,FLAG_REFL_SPELL)){
     success=1;
     if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel more safe now, somehow.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel more safe now, somehow.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"Suddenly you feel less safe, somehow.");
+      new_draw_info(NDI_UNIQUE, 0, op,"Suddenly you feel less safe, somehow.");
     }
   }
   if(QUERY_FLAG(tmp,FLAG_FLYING)) {
@@ -458,9 +458,9 @@ int change_abil(object *op, object *tmp) {
       success=1;
       /* if were already flying then now flying higher */
       if ( QUERY_FLAG(op,FLAG_FLYING) == QUERY_FLAG(&refop,FLAG_FLYING))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You float a little higher in the air.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You float a little higher in the air.");
       else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You start to float in the air!.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You start to float in the air!.");
         SET_FLAG(op,FLAG_FLYING);
         if(op->speed>1)
           op->speed=1;
@@ -469,9 +469,9 @@ int change_abil(object *op, object *tmp) {
       success=1;
       /* if were already flying then now flying lower */
       if ( QUERY_FLAG(op,FLAG_FLYING) == QUERY_FLAG(&refop,FLAG_FLYING))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You float a little lower in the air.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You float a little lower in the air.");
       else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You float down to the ground.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You float down to the ground.");
 	check_walk_on (op, op);
       }
     }
@@ -485,29 +485,29 @@ int change_abil(object *op, object *tmp) {
       if(flag>0) {
         if(op->race) free_string(op->race); 
 	op->race=add_string("undead");
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your lifeforce drains away!");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce drains away!");
       } else {
         if(op->race) free_string(op->race); 
         if(op->arch->clone.race) 
 	   op->race=add_string(op->arch->clone.race);  
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your lifeforce returns!");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce returns!");
       }
     }
 
   if ( QUERY_FLAG(op,FLAG_STEALTH) != QUERY_FLAG(&refop,FLAG_STEALTH)){
     success=1;
     if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You walk more quietly.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You walk more quietly.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You walk more noisily.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You walk more noisily.");
     }
   }
   if ( QUERY_FLAG(op,FLAG_MAKE_INVIS) != QUERY_FLAG(&refop,FLAG_MAKE_INVIS)){
     success=1;
     if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You become transparent.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You become transparent.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You can see yourself.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You can see yourself.");
     }
   }
   /* blinded you can tell if more blinded since blinded player has minimal
@@ -516,18 +516,18 @@ int change_abil(object *op, object *tmp) {
     success=1;
     if(flag>0) {
       if(QUERY_FLAG(op,FLAG_WIZ))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your mortal self is blinded.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self is blinded.");
       else { 
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You are blinded.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You are blinded.");
         SET_FLAG(op,FLAG_BLIND);
         if(op->type==PLAYER)
           op->contr->do_los=1;
       }  
     } else {
       if(QUERY_FLAG(op,FLAG_WIZ))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your mortal self can now see again.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self can now see again.");
       else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your vision returns.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your vision returns.");
         CLEAR_FLAG(op,FLAG_BLIND);
         if(op->type==PLAYER)
           op->contr->do_los=1;
@@ -538,11 +538,11 @@ int change_abil(object *op, object *tmp) {
   if ( QUERY_FLAG(op,FLAG_SEE_IN_DARK) != QUERY_FLAG(&refop,FLAG_SEE_IN_DARK)){
     success=1;
     if(flag>0) {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your vision is better in the dark.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your vision is better in the dark.");
         if(op->type==PLAYER)
           op->contr->do_los=1;
     } else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"You see less well in the dark.");
+        new_draw_info(NDI_UNIQUE, 0, op,"You see less well in the dark.");
         if(op->type==PLAYER)
           op->contr->do_los=1;
     }  
@@ -552,17 +552,17 @@ int change_abil(object *op, object *tmp) {
     success=1;
     if(flag>0) {
       if(QUERY_FLAG(op,FLAG_WIZ))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your vision becomes a little clearer.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a little clearer.");
       else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Everything becomes transparent.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Everything becomes transparent.");
         if(op->type==PLAYER)
           op->contr->do_los=1;
       }
     } else {
       if(QUERY_FLAG(op,FLAG_WIZ))
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Your vision becomes a bit out of focus.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a bit out of focus.");
       else {
-        (*draw_info_func)(NDI_UNIQUE, 0, op,"Everything suddenly looks very solid.");
+        new_draw_info(NDI_UNIQUE, 0, op,"Everything suddenly looks very solid.");
         if(op->type==PLAYER)
           op->contr->do_los=1;
       }
@@ -570,40 +570,40 @@ int change_abil(object *op, object *tmp) {
   }
   if(tmp->stats.luck) {
     success=1;
-    if(flag>0) {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel more lucky.");
+    if(flag*tmp->stats.luck>0) {
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel more lucky.");
     } else {
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel less lucky.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel less lucky.");
     }
   }
   if(tmp->stats.hp && op->type==PLAYER) {
     success=1;
     if(flag*tmp->stats.hp>0)
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel much more healthy!");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel much more healthy!");
     else
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel much less healthy!");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel much less healthy!");
   }
   if(tmp->stats.sp && op->type==PLAYER && tmp->type!=SKILL) {
     success=1;
     if(flag*tmp->stats.sp>0)
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel one with the powers of magic!");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel one with the powers of magic!");
     else
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You suddenly feel very mundane.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You suddenly feel very mundane.");
   }
   /* for the future when artifacts set this -b.t. */
   if(tmp->stats.grace && op->type==PLAYER) {
     success=1;
      if(flag*tmp->stats.grace>0)
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel closer to your god!");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel closer to your god!");
     else 
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You suddenly feel less holy.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You suddenly feel less holy.");
   } 
   if(tmp->stats.food && op->type==PLAYER) {
     success=1;
     if(tmp->stats.food*flag>0)
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel your digestion slowing down.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel your digestion slowing down.");
     else
-      (*draw_info_func)(NDI_UNIQUE, 0, op,"You feel your digestion speeding up.");
+      new_draw_info(NDI_UNIQUE, 0, op,"You feel your digestion speeding up.");
   }
 
   /* Messages for changed resistance */
@@ -619,7 +619,7 @@ int change_abil(object *op, object *tmp) {
 	sprintf(message, "Your resistance to %s drops to %d%%.",
 	       change_resist_msg[i], op->resist[i]);
       
-      (*draw_info_func)(NDI_UNIQUE|NDI_BLUE, 0, op, message);
+      new_draw_info(NDI_UNIQUE|NDI_BLUE, 0, op, message);
     }
   }
 
@@ -628,9 +628,9 @@ int change_abil(object *op, object *tmp) {
 	    if ((i=get_attr_value(&(tmp->stats),j))!=0) {
 		success=1;
 		if (i * flag > 0)
-		    (*draw_info_func)(NDI_UNIQUE, 0, op, gain_msg[j]);
+		    new_draw_info(NDI_UNIQUE, 0, op, gain_msg[j]);
 		else
-		    (*draw_info_func)(NDI_UNIQUE, 0, op, lose_msg[j]);
+		    new_draw_info(NDI_UNIQUE, 0, op, lose_msg[j]);
 	    }
 	}
     }
@@ -650,7 +650,7 @@ void drain_specific_stat(object *op, int deplete_stats) {
   object *tmp;
   archetype *at;
 
-  at = find_archetype("depletion");
+  at = find_archetype(ARCH_DEPLETION);
   if (!at) {
     LOG(llevError, "Couldn't find archetype depletion.\n");
     return;
@@ -663,7 +663,7 @@ void drain_specific_stat(object *op, int deplete_stats) {
     }
   }
 
-  (*draw_info_func)(NDI_UNIQUE, 0, op, drain_msg[deplete_stats]);
+  new_draw_info(NDI_UNIQUE, 0, op, drain_msg[deplete_stats]);
   change_attr_value(&tmp->stats, deplete_stats, -1);
   fix_player(op);
 }
@@ -676,6 +676,8 @@ void drain_specific_stat(object *op, int deplete_stats) {
 void change_luck(object *op, int value) {
   object *tmp;
   archetype *at;
+  int new_luck;
+
   at = find_archetype("luck");
   if (!at)
     LOG(llevError, "Couldn't find archetype luck.\n");
@@ -689,23 +691,27 @@ void change_luck(object *op, int value) {
       SET_FLAG(tmp,FLAG_APPLIED);
     }
     if (value) {
+      /* Limit the luck value of the bad luck object to +/-100. This
+       * (arbitrary) value prevents overflows (both in the bad luck object and
+       * in op itself).
+       */
+      new_luck = tmp->stats.luck+value;
+      if (new_luck >= -100 && new_luck <= 100) {
       op->stats.luck+=value;
-      tmp->stats.luck+=value;
+	tmp->stats.luck = new_luck;
+      }
     } else {
       if (!tmp->stats.luck) {
-        LOG(llevDebug, "Internal error in change_luck().\n");
         return;
       }
       /* Randomly change the players luck.  Basically, we move it
        * back neutral (if greater>0, subtract, otherwise add)
-       * I believe this is supposed to be > and not >= - this means
-       * if your luck is -1/1, it won't get adjusted - only when your
-       * luck is worse can you hope for improvment.
-       * note that if we adjusted it with it is -1/1, that check above
-       * for 0 luck will happen, resulting in error.
        */
-      if (RANDOM()%(FABS(tmp->stats.luck)) > RANDOM()%30)
-        tmp->stats.luck += tmp->stats.luck>0?-1:1;
+      if (RANDOM()%(FABS(tmp->stats.luck)) >= RANDOM()%30) {
+        int diff = tmp->stats.luck>0?-1:1;
+        op->stats.luck += diff;
+        tmp->stats.luck += diff;
+      }
     }
   }
 }
@@ -1464,7 +1470,7 @@ void dragon_level_gain(object *who) {
 
 	if (abil->resist[abil->stats.exp]>0 && abil->resist[abil->stats.exp]%5 == 0) {
 	    /* time to hand out a new ability-gift */
-	    (*dragon_gain_func)(who, (int)abil->stats.exp,
+	    dragon_ability_gain(who, (int)abil->stats.exp,
 			       (int)((1+abil->resist[abil->stats.exp])/5.));
 	}
     
@@ -1472,7 +1478,7 @@ void dragon_level_gain(object *who) {
 	    /* apply new ability focus */
 	    sprintf(buf, "Your metabolism now focuses on %s!",
 		    change_resist_msg[abil->last_eat]);
-	    (*draw_info_func)(NDI_UNIQUE|NDI_BLUE, 0, who, buf);
+	    new_draw_info(NDI_UNIQUE|NDI_BLUE, 0, who, buf);
       
 	    abil->stats.exp = abil->last_eat;
 	    abil->last_eat = 0;
@@ -1491,7 +1497,7 @@ void dragon_level_gain(object *who) {
  * We return the skill - this makes it easier for calling functions that
  * want to do something with it immediately.
  */
-object *give_skill_by_name(object *op, char *skill_name)
+object *give_skill_by_name(object *op, const char *skill_name)
 {
     object *skill_obj;
 
@@ -1528,13 +1534,14 @@ void player_lvl_adj(object *who, object *op) {
     if(!op)        /* when rolling stats */ 
 	op = who;	
  
-    if(op->level < settings.max_level && op->stats.exp >= level_exp(op->level+1,op->expmul)) {
+    if(op->level < settings.max_level && op->stats.exp >= level_exp(op->level+1,who->expmul)) {
 	op->level++;
 	
 	if (op != NULL && op == who && op->stats.exp > 1 && is_dragon_pl(who))
 	  dragon_level_gain(who);
-	
-	if(who && (who->level < 11) && who->type==PLAYER) { 
+
+        /* Only roll these if it is the player (who) that gained the level */
+	if(who && op==who && (who->level < 11) && who->type==PLAYER) { 
 	    who->contr->levhp[who->level] = die_roll(2, 4, who, PREFER_HIGH)+1;
 	    who->contr->levsp[who->level] = die_roll(2, 3, who, PREFER_HIGH);
 	    who->contr->levgrace[who->level]=die_roll(2, 2, who, PREFER_HIGH)-1;
@@ -1546,15 +1553,15 @@ void player_lvl_adj(object *who, object *op) {
 		sprintf(buf,"You are now level %d in the %s skill.",op->level,op->name);
 	    else
 		sprintf(buf,"You are now level %d.",op->level);
-	    if(who) (*draw_info_func)(NDI_UNIQUE|NDI_RED, 0, who,buf);
+	    if(who) new_draw_info(NDI_UNIQUE|NDI_RED, 0, who,buf);
 	}
 	player_lvl_adj(who,op); /* To increase more levels */
-    } else if (op->level>1 && op->stats.exp<level_exp(op->level,op->expmul)) {
+    } else if (op->level>1 && op->stats.exp<level_exp(op->level,who->expmul)) {
 	op->level--;
 	if(who) fix_player(who);
 	if(op->type!=PLAYER) {
 	    sprintf(buf,"You are now level %d in the %s skill.",op->level,op->name);
-	    if(who) (*draw_info_func)(NDI_UNIQUE|NDI_RED, 0, who,buf);
+	    if(who) new_draw_info(NDI_UNIQUE|NDI_RED, 0, who,buf);
 	}
 	player_lvl_adj(who,op); /* To decrease more levels */
     }
@@ -1606,7 +1613,7 @@ void calc_perm_exp(object *op)
  * flag is what to do if the player doesn't have the skill:
  */
 
-static void add_player_exp(object *op, sint64 exp, char *skill_name, int flag)
+static void add_player_exp(object *op, sint64 exp, const char *skill_name, int flag)
 {
     object *skill_obj=NULL;
     sint64 limit, exp_to_add;
@@ -1710,7 +1717,7 @@ sint64 check_exp_adjust(object *op, sint64 exp)
  * exp is the amount of exp to subtract - thus, it should be
  * a postive number.
  */
-static void subtract_player_exp(object *op, sint64 exp, char *skill, int flag)
+static void subtract_player_exp(object *op, sint64 exp, const char *skill, int flag)
 {
     float fraction = (float) exp/(float) op->stats.exp;
     object *tmp;
@@ -1750,7 +1757,7 @@ static void subtract_player_exp(object *op, sint64 exp, char *skill, int flag)
  * these last two values are only used for players.
  */
  
-void change_exp(object *op, sint64 exp, char *skill_name, int flag) {
+void change_exp(object *op, sint64 exp, const char *skill_name, int flag) {
 
 #ifdef EXP_DEBUG
 #ifndef WIN32
