@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_loger_c =
- *   "$Id: logger.c,v 1.8 2003/09/13 05:01:29 mwedel Exp $ ";
+ *   "$Id: logger.c,v 1.10 2005/07/15 13:51:49 ryo_saeba Exp $ ";
  */
 
 /*
@@ -40,7 +40,7 @@
  * and llevError are always printed, regardless of debug mode.
  */
 
-void LOG (LogLevel logLevel, char *format, ...)
+void LOG (LogLevel logLevel, const char *format, ...)
 {
   char buf[20480];  /* This needs to be really really big - larger
 		     * than any other buffer, since that buffer may
@@ -69,7 +69,7 @@ void LOG (LogLevel logLevel, char *format, ...)
       logLevel == llevError && ++nroferrors > MAX_ERRORS) {
     exiting = 1;
     if (!trying_emergency_save)
-      (*emergency_save_func) (0);
+      emergency_save(0);
   }
   va_end(ap);
 }
