@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.64 2004/04/16 06:23:41 mwedel Exp $";
+ *   "$Id: map.c,v 1.76 2005/07/17 11:00:03 ryo_saeba Exp $";
  */
 
 /*
@@ -43,7 +43,7 @@ extern int nrofallocobjects,nroffreeobjects;
  * return NULL if no match is found.
  */
 
-mapstruct *has_been_loaded (char *name) {
+mapstruct *has_been_loaded (const char *name) {
     mapstruct *map;
 
     if (!name || !*name) 
@@ -61,7 +61,7 @@ mapstruct *has_been_loaded (char *name) {
  * it really should be called create_mapname
  */
 
-char *create_pathname (char *name) {
+const char *create_pathname (const char *name) {
     static char buf[MAX_BUF];
 
     /* Why?  having extra / doesn't confuse unix anyplace?  Dependancies
@@ -78,7 +78,7 @@ char *create_pathname (char *name) {
  * same as create_pathname, but for the overlay maps.
  */
 
-char *create_overlay_pathname (char *name) {
+const char *create_overlay_pathname (const char *name) {
     static char buf[MAX_BUF];
 
     /* Why?  having extra / doesn't confuse unix anyplace?  Dependancies
@@ -97,7 +97,7 @@ char *create_overlay_pathname (char *name) {
  * files than full directory structure, but if this is problem it can 
  * be changed.
  */
-static char *create_items_path (char *s) {
+static const char *create_items_path (const char *s) {
     static char buf[MAX_BUF];
     char *t;
     
@@ -132,7 +132,7 @@ static char *create_items_path (char *s) {
  * that seems to be missing the prepend_dir processing
  */
 
-int check_path (char *name, int prepend_dir)
+int check_path (const char *name, int prepend_dir)
 {
     char buf[MAX_BUF];
 #ifndef WIN32
@@ -316,9 +316,18 @@ int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
 		    continue;
 	    }
 	} /* if check_inv */
-	else if (QUERY_FLAG(tmp,FLAG_NO_PASS) || (QUERY_FLAG(tmp,FLAG_ALIVE) &&
-		     tmp->head != ob && tmp != ob && tmp->type != DOOR))
+	else {
+	    /* Broke apart a big nasty if into several here to make
+	     * this more readable.  first check - if the space blocks
+	     * movement, can't move here.
+	     * second - if a monster, can't move there, unles it is a 
+	     * hidden dm
+	     */
+	    if (QUERY_FLAG(tmp,FLAG_NO_PASS)) return 1;
+	    if (QUERY_FLAG(tmp,FLAG_ALIVE) && tmp->head != ob && tmp != ob && 
+		tmp->type != DOOR && !(QUERY_FLAG(tmp,FLAG_WIZ) && tmp->contr->hidden))
 			return 1;
+	}
 
     }
     return 0;
@@ -463,12 +472,11 @@ void load_objects (mapstruct *m, FILE *fp, int mapflags) {
 	bufstate = LO_REPEAT;
 
 	/* if the archetype for the object is null, means that we
-	 * got an invalid object.  Don't do anythign with it - the game
+	 * got an invalid object.  Don't do anything with it - the game
 	 * or editor will not be able to do anything with it either.
 	 */
 	if (op->arch==NULL) {
-	    if (op->name!=NULL)
-		LOG(llevDebug,"Discarded object %s - invalid archetype.\n",op->name);
+	    LOG(llevDebug,"Discarding object without arch: %s\n", op->name?op->name:"(null)");
 	    continue;
 	}
 
@@ -492,7 +500,10 @@ void load_objects (mapstruct *m, FILE *fp, int mapflags) {
 		insert_ob_in_map(op,m,op,INS_NO_MERGE | INS_NO_WALK_ON | INS_ABOVE_FLOOR_ONLY);
 	    else
 		insert_ob_in_map(op,m,op,INS_NO_MERGE | INS_NO_WALK_ON | INS_ON_TOP);
-	    if (op->inv) sum_weight(op);
+
+	    if (op->inv)
+		sum_weight(op);
+
 	    prev=op,last_more=op;
 	    break;
 
@@ -598,7 +609,7 @@ void allocate_map(mapstruct *m) {
      * that is their poor assumption.
      */
     if (m->spaces) {
-	LOG(llevError,"allocate_map callled with already allocated map (%s)\n", m->path);
+	LOG(llevError,"allocate_map called with already allocated map (%s)\n", m->path);
 	free(m->spaces);
     }
 
@@ -635,7 +646,7 @@ mapstruct *get_empty_map(int sizex, int sizey) {
 
 static int load_map_header(FILE *fp, mapstruct *m)
 {
-    char buf[HUGE_BUF], msgbuf[HUGE_BUF], *key, *value, *end;
+    char buf[HUGE_BUF], msgbuf[HUGE_BUF], *key=NULL, *value, *end;
     int msgpos=0;
 
     while (fgets(buf, HUGE_BUF-1, fp)!=NULL) {
@@ -653,6 +664,12 @@ static int load_map_header(FILE *fp, mapstruct *m)
 	    while (isspace(*value)) value++;
 	    end = strchr(value, '\n');
 	}
+	if (!end) {
+	    LOG(llevError, "Error loading map header - did not find a newline - perhaps file is truncated?  Buf=%s\n",
+		buf);
+	    return 1;
+	}
+	    
 
 	/* key is the field name, value is what it should be set
 	 * to.  We've already done the work to null terminate key,
@@ -689,7 +706,7 @@ static int load_map_header(FILE *fp, mapstruct *m)
 	     */
 	    if (msgpos != 0)
 		m->msg = strdup_local(msgbuf);
-	} 
+	}
 	/* first strcmp value on these are old names supported
 	 * for compatibility reasons.  The new values (second) are
 	 * what really should be used.
@@ -714,6 +731,8 @@ static int load_map_header(FILE *fp, mapstruct *m)
 	    m->fixed_resettime = atoi(value);
 	} else if (!strcmp(key,"unique")) {
 	    m->unique = atoi(value);
+	} else if (!strcmp(key,"region")) {
+	    m->region = get_region_by_name(value);
 	} else if (!strcmp(key,"outdoor")) {
 	    m->outdoor = atoi(value);
 	} else if (!strcmp(key, "temp")) {
@@ -787,7 +806,7 @@ static int load_map_header(FILE *fp, mapstruct *m)
 	    LOG(llevError,"Got unknown value in map header: %s %s\n", key, value);
 	}
     }
-    if (strcmp(key,"end")) {
+    if (!key || strcmp(key,"end")) {
 	LOG(llevError,"Got premature eof on map header!\n");
 	return 1;
     }
@@ -806,7 +825,7 @@ static int load_map_header(FILE *fp, mapstruct *m)
  *		managed map list.
  */
 
-mapstruct *load_original_map(char *filename, int flags) {
+mapstruct *load_original_map(const char *filename, int flags) {
     FILE *fp;
     mapstruct *m;
     int comp;
@@ -866,7 +885,7 @@ static mapstruct *load_temporary_map(mapstruct *m) {
 	delete_map(m);
         m = load_original_map(buf, 0);
 	if(m==NULL) return NULL;
-	(*fix_auto_apply_func)(m); /* Chests which open as default */
+	fix_auto_apply(m); /* Chests which open as default */
 	return m;
     }
 
@@ -877,7 +896,7 @@ static mapstruct *load_temporary_map(mapstruct *m) {
 	delete_map(m);
         m = load_original_map(buf, 0);
 	if(m==NULL) return NULL;
-	(*fix_auto_apply_func)(m); /* Chests which open as default */
+	fix_auto_apply(m); /* Chests which open as default */
 	return m;
     }
     
@@ -905,7 +924,7 @@ static mapstruct *load_temporary_map(mapstruct *m) {
  * option if we can't find the original map)
  */
 
-mapstruct *load_overlay_map(char *filename, mapstruct *m) {
+mapstruct *load_overlay_map(const char *filename, mapstruct *m) {
     FILE *fp;
     int comp;
     char pathname[MAX_BUF];
@@ -1067,6 +1086,7 @@ int new_save_map(mapstruct *m, int flag) {
      * or a difficulty value we generated when the map was first loaded
      */
     if (m->difficulty) fprintf(fp,"difficulty %d\n", m->difficulty);
+    if (m->region) fprintf(fp,"region %s\n", m->region->name);
     if (m->darkness) fprintf(fp,"darkness %d\n", m->darkness);
     if (m->width) fprintf(fp,"width %d\n", m->width);
     if (m->height) fprintf(fp,"height %d\n", m->height);
@@ -1185,7 +1205,7 @@ void free_all_objects(mapstruct *m) {
      */
     for (op=objects; op!=NULL; op=op->next) {
 	if (!QUERY_FLAG(op, FLAG_REMOVED) && op->map == m) {
-	    LOG(llevDebug,"free_all_objects: object %s still on map after it should have been freed", op->name);
+	    LOG(llevDebug,"free_all_objects: object %s still on map after it should have been freed\n", op->name);
 	    abort();
 	}
     }
@@ -1290,7 +1310,7 @@ void delete_map(mapstruct *m) {
  * Returns a pointer to the given map.
  */
 
-mapstruct *ready_map_name(char *name, int flags) {
+mapstruct *ready_map_name(const char *name, int flags) {
     mapstruct *m;
 
     if (!name)
@@ -1329,7 +1349,7 @@ mapstruct *ready_map_name(char *name, int flags) {
 	if (!(m = load_original_map(name, (flags & MAP_PLAYER_UNIQUE))))
 	    return (NULL);
 
-	(*fix_auto_apply_func)(m); /* Chests which open as default */
+	fix_auto_apply(m); /* Chests which open as default */
 
 	/* If a player unique map, no extra unique object file to load.
 	 * if from the editor, likewise.
@@ -1371,9 +1391,9 @@ mapstruct *ready_map_name(char *name, int flags) {
     /* In case other objects press some buttons down */
     update_buttons(m);
     if (m->outdoor)
-	(*set_darkness_map_func)(m);
+	    set_darkness_map(m);
     /* run the weather over this map */
-    (*weather_effect_func)(name);
+    weather_effect(name);
     return m;
 }
 
@@ -1473,9 +1493,9 @@ int change_map_light(mapstruct *m, int change) {
 
     /* inform all players on the map */
     if (change>0) 
-	(info_map_func)(NDI_BLACK, m,"It becomes darker.");
+	    new_info_map(NDI_BLACK, m,"It becomes darker.");
     else
-	(info_map_func)(NDI_BLACK, m,"It becomes brighter.");
+	    new_info_map(NDI_BLACK, m,"It becomes brighter.");
 
     /* Do extra checking.  since m->darkness is a unsigned value,
      * we need to be extra careful about negative values.
