@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_c =
- *   "$Id: object.c,v 1.86 2004/04/14 07:24:30 mwedel Exp $";
+ *   "$Id: object.c,v 1.97 2005/07/15 13:51:49 ryo_saeba Exp $";
  */
 
 /*
@@ -85,7 +85,7 @@ int freedir[SIZEOFFREE]= {
  * check weight
  */
 
-inline int CAN_MERGE(object *ob1, object *ob2) {
+int CAN_MERGE(object *ob1, object *ob2) {
 
     /* A couple quicksanity checks */
     if ((ob1 == ob2) || (ob1->type != ob2->type)) return 0;
@@ -617,6 +617,7 @@ void copy_object(object *op2, object *op) {
 	if (evt_new->hook)  add_refcount(evt_new->hook);
 	if (evt_new->plugin)  add_refcount(evt_new->plugin);
 	if (evt_new->options)  add_refcount(evt_new->options);
+	evt_new->next = NULL;
 
 	/* Try to be a little clever here, and store away the
 	 * last event we copied, so that its simpler to update the
@@ -625,7 +626,7 @@ void copy_object(object *op2, object *op) {
 	if (evt2)
 	    evt2->next = evt_new;
 	else
-	    op->events = evt2;
+	    op->events = evt_new;
 
 	evt2 = evt_new;
     }
@@ -861,6 +862,9 @@ void update_object(object *op, int action) {
     if (op->x < 0 || op->x >= MAP_WIDTH(op->map) ||
 	op->y < 0 || op->y >= MAP_HEIGHT(op->map)) {
         LOG(llevError,"update_object() called for object out of map!\n");
+#ifdef MANY_CORES
+	abort();
+#endif
 	return;
     }
     
@@ -1219,7 +1223,7 @@ void remove_ob(object *op) {
 	if (check_walk_off && (QUERY_FLAG (op, FLAG_FLYING) ?
 	    QUERY_FLAG (tmp, FLAG_FLY_OFF) : QUERY_FLAG (tmp, FLAG_WALK_OFF))) {
 	    
-	    move_apply_func (tmp, op, NULL);
+	    move_apply(tmp, op, NULL);
 	    if (was_destroyed (op, tag)) {
 		LOG (llevError, "BUG: remove_ob(): name %s, archname %s destroyed "
 		     "leaving object\n", tmp->name, tmp->arch->name);
@@ -1245,7 +1249,7 @@ void remove_ob(object *op) {
     else
 	update_object(last, UP_OBJ_REMOVE);
 
-    if(QUERY_FLAG(op,FLAG_BLOCKSVIEW)|| (op->glow_radius>0)) 
+    if(QUERY_FLAG(op,FLAG_BLOCKSVIEW)|| (op->glow_radius != 0)) 
 	update_all_los(op->map, op->x, op->y);
 
 }
@@ -1357,20 +1361,24 @@ object *insert_ob_in_map (object *op, mapstruct *m, object *originator, int flag
     }
     CLEAR_FLAG(op,FLAG_REMOVED);
 
-    if(op->nrof && !(flag & INS_NO_MERGE)) {
-	for(tmp=GET_MAP_OB(m,op->x,op->y);tmp!=NULL;tmp=tmp->above)
-	    if (CAN_MERGE(op,tmp)) {
-		op->nrof+=tmp->nrof;
-		remove_ob(tmp);
-		free_object(tmp);
-	    }
-    }
     /* Debugging information so you can see the last coordinates this object had */
     op->ox=op->x;
     op->oy=op->y;
     x = op->x;
     y = op->y;
     op->map=get_map_from_coord(m, &x, &y);
+
+    /* this has to be done after we translate the coordinates.
+     */
+    if(op->nrof && !(flag & INS_NO_MERGE)) {
+	for(tmp=GET_MAP_OB(op->map,x,y);tmp!=NULL;tmp=tmp->above)
+	    if (CAN_MERGE(op,tmp)) {
+		op->nrof+=tmp->nrof;
+		remove_ob(tmp);
+		free_object(tmp);
+	    }
+    }
+
     /* Ideally, the caller figures this out.  However, it complicates a lot
      * of areas of callers (eg, anything that uses find_free_spot would now
      * need extra work
@@ -1434,6 +1442,7 @@ object *insert_ob_in_map (object *op, mapstruct *m, object *originator, int flag
 	     */
 
 	    /* Have object 'fall below' other objects that block view.
+ 	     * Unless those objects are exits, type 66
 	     * If INS_ON_TOP is used, don't do this processing
 	     * Need to find the object that in fact blocks view, otherwise
 	     * stacking is a bit odd.
@@ -1442,7 +1451,7 @@ object *insert_ob_in_map (object *op, mapstruct *m, object *originator, int flag
 		(get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL) & P_BLOCKSVIEW) && 
 		(op->face && !op->face->visibility)) {
 		for (last=top; last != floor; last=last->below)
-		    if QUERY_FLAG(last, FLAG_BLOCKSVIEW) break;
+ 		    if (QUERY_FLAG(last, FLAG_BLOCKSVIEW)&&(last->type != EXIT)) break;
 		/* Check to see i we found the object that blocks view,
 		 * and make sure we have a below pointer for it so that
 		 * we can get inserted below this one, which requires we
@@ -1491,7 +1500,7 @@ object *insert_ob_in_map (object *op, mapstruct *m, object *originator, int flag
      * or just updating the P_NEED_UPDATE for spaces within this area
      * of effect may be sufficient.
      */
-    if(MAP_DARKNESS(op->map) && op->glow_radius>0) 
+    if(MAP_DARKNESS(op->map) && (op->glow_radius != 0)) 
 	update_all_los(op->map, op->x, op->y);
 
 
@@ -1556,7 +1565,7 @@ void replace_insert_ob_in_map(char *arch_string, object *op) {
  * global static errmsg array.
  */
 
-object *get_split_ob(object *orig_ob,int nr) {
+object *get_split_ob(object *orig_ob, uint32 nr) {
     object *newob;
     int is_removed = (QUERY_FLAG (orig_ob, FLAG_REMOVED) != 0);
 
@@ -1642,15 +1651,13 @@ object *decrease_ob_nr (object *op, int i)
             sub_weight (op->env, op->weight * i);
             op->nrof -= i;
             if (tmp) {
-                (*esrv_send_item_func) (tmp, op);
-                (*esrv_update_item_func) (UPD_WEIGHT, tmp, tmp);
+                esrv_send_item(tmp, op);
             }
         } else {
             remove_ob (op);
             op->nrof = 0;
             if (tmp) {
-                (*esrv_del_item_func) (tmp->contr, op->count);
-                (*esrv_update_item_func) (UPD_WEIGHT, tmp, tmp);
+                esrv_del_item(tmp->contr, op->count);
             }
         }
     }
@@ -1668,9 +1675,9 @@ object *decrease_ob_nr (object *op, int i)
         for (tmp = above; tmp != NULL; tmp = tmp->above)
             if (tmp->type == PLAYER) {
                 if (op->nrof)
-                    (*esrv_send_item_func) (tmp, op);
+                    esrv_send_item(tmp, op);
                 else
-                    (*esrv_del_item_func) (tmp->contr, op->count);
+                    esrv_del_item(tmp->contr, op->count);
             }
     }
 
@@ -1778,7 +1785,7 @@ object *insert_ob_in_ob(object *op,object *where) {
   op->ox=0,op->oy=0;
 
   /* reset the light list and los of the players on the map */
-  if(op->glow_radius>0&&where->map)
+  if((op->glow_radius!=0)&&where->map)
   {
 #ifdef DEBUG_LIGHTS
       LOG(llevDebug, " insert_ob_in_ob(): got %s to insert in map/op\n",
@@ -1852,8 +1859,8 @@ int check_walk_on (object *op, object *originator)
 
 	    diff=(SLOW_PENALTY(tmp)*FABS(op->speed));
 	    if (op->type==PLAYER) {
-		if ((QUERY_FLAG(tmp,FLAG_IS_HILLY) && (*find_skill_by_number_func)(op,SK_CLIMBING)) ||
-		    (QUERY_FLAG(tmp,FLAG_IS_WOODED) && (*find_skill_by_number_func)(op,SK_WOODSMAN)))  {
+		if ((QUERY_FLAG(tmp,FLAG_IS_HILLY) && find_skill_by_number(op,SK_CLIMBING)) ||
+		    (QUERY_FLAG(tmp,FLAG_IS_WOODED) && find_skill_by_number(op,SK_WOODSMAN)))  {
 			diff=diff/4.0;
 		}
 	    }
@@ -1861,7 +1868,7 @@ int check_walk_on (object *op, object *originator)
 	}
 	if(QUERY_FLAG(op,FLAG_FLYING)?QUERY_FLAG(tmp,FLAG_FLY_ON):
 	   QUERY_FLAG(tmp,FLAG_WALK_ON)) {
-	    move_apply_func (tmp, op, originator);
+	    move_apply(tmp, op, originator);
             if (was_destroyed (op, tag))
               return 1;
 	    /* what the person/creature stepped onto has moved the object
@@ -2053,6 +2060,25 @@ int find_first_free_spot(archetype *at, mapstruct *m,int x,int y) {
     return -1;
 }
 
+/*
+ * The function permute(arr, begin, end) randomly reorders the array
+ * arr[begin..end-1].
+ */
+static void permute(int *arr, int begin, int end)
+{
+    int i, j, tmp, len;
+
+    len = end-begin;
+    for(i = begin; i < end; i++)
+    {
+	j = begin+RANDOM()%len;
+
+	tmp = arr[i];
+	arr[i] = arr[j];
+	arr[j] = tmp;
+    }
+}
+
 /* new function to make monster searching more efficient, and effective! 
  * This basically returns a randomized array (in the passed pointer) of
  * the spaces to find monsters.  In this way, it won't always look for
@@ -2062,96 +2088,17 @@ int find_first_free_spot(archetype *at, mapstruct *m,int x,int y) {
  */
 void get_search_arr(int *search_arr)
 {
-    int i,j,tmp;
-    int search_nums[SIZEOFFREE]={
-         0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,
-	25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48};
-    
-    /* it is assumed that search_arr is of size SIZEOFFREE = 49 */
-    for (i=0;i<SIZEOFFREE;i++) {
-        if (i>36)
-	{
-	    tmp=(RANDOM()%(49-i))+37;
-	    for(j=37;j<(49-i+37);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>28)
-	{
-	    tmp=(RANDOM()%(37-i))+29;
-	    for(j=29;j<(37-i+29);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>20)
-	{
-	    tmp=(RANDOM()%(29-i))+21;
-	    for(j=21;j<(29-i+21);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>12)
-	{
-	    tmp=(RANDOM()%(21-i))+13;
-	    for(j=13;j<(21-i+13);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>8)
-	{
-	    tmp=(RANDOM()%(13-i))+9;
-	    for(j=9;j<(13-i+9);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>4)
-	{
-	    tmp=(RANDOM()%(9-i))+5;
-	    for(j=5;j<(9-i+5);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else if (i>0)
-	{
-	    tmp=(RANDOM()%(5-i))+1;
-	    for(j=1;j<(5-i+1);j++)
-	    {
-	        if(j<=tmp && search_nums[j]==0)
-		    tmp++;
-	    }
-	    *(search_arr+i)=search_nums[tmp];
-	    search_nums[tmp]=0;
-	}
-	else
-	    *search_arr=0;
-	
-    }
-}
+    int i;
 
+    for(i = 0; i < SIZEOFFREE; i++)
+    {
+	search_arr[i] = i;
+    }
+
+    permute(search_arr, 1, SIZEOFFREE1+1);
+    permute(search_arr, SIZEOFFREE1+1, SIZEOFFREE2+1);
+    permute(search_arr, SIZEOFFREE2+1, SIZEOFFREE);
+}
 
 /*
  * find_dir(map, x, y, exclude) will search some close squares in the
