@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_porting_c =
- *   "$Id: porting.c,v 1.23 2004/03/26 22:41:43 ryo_saeba Exp $";
+ *   "$Id: porting.c,v 1.25 2005/04/27 06:39:50 mwedel Exp $";
  */
 
 /*
@@ -78,6 +78,11 @@ char *tempnam_local(char *dir, char *pfx)
     char *f, *name;
     pid_t pid=getpid();
 
+/* HURD does not have a hard limit, but we do */
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 4096
+#endif
+
     if (!(name = (char *) malloc(MAXPATHLEN)))
 	return(NULL);
 
@@ -491,7 +496,7 @@ char *uncomp[NROF_COMPRESS_METHODS][3] = {
  * be changed in the config file.)
  */
 
-FILE *open_and_uncompress(char *name,int flag, int *compressed) {
+FILE *open_and_uncompress(const char *name,int flag, int *compressed) {
   FILE *fp;
   char buf[MAX_BUF],buf2[MAX_BUF], *bufend;
   int try_once = 0;
