@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_c =
- *   "$Id: re-cmp.c,v 1.9 2003/09/13 05:01:30 mwedel Exp $";
+ *   "$Id: re-cmp.c,v 1.11 2005/01/12 08:03:59 mwedel Exp $";
  */
 
 
@@ -134,8 +134,25 @@ re_cmp(char *str, char *regexp) {
 		    return str;
 		break;
 	}
+	/* The logic here is that re_match_token only sees
+	 * if the one letter matches.  Thus, if the
+	 * regex is like '@match eureca', and the
+	 * the user enters anything with an e, re_match_token
+	 * returns true, but they really need to match the
+	 * entire regexp, which re_cmp_step will do.
+	 * However, what happens is that there can be a case
+	 * where the string being match is something like
+	 * 'where is eureca'.  In this case, the re_match_token
+	 * matches that first e, but the re_cmp_step below,
+	 * fails because the next character (r) doesn't match
+	 * the u.  So we call re_cmp with the string
+	 * after the first r, so that it should hopefully match
+	 * up properly.
+	 */
 	if (re_cmp_step(str+1, next_regexp, 1, 0))
 	    return str;
+	else if (*(str+1) != 0)
+	    return re_cmp(str+1, regexp);
     }
     return NULL;
 }
@@ -185,7 +202,7 @@ re_cmp_step(char *str, char *regexp, int slot, int matches) {
 	++matches;
 
     if (*str == 0)
-	return (*next_regexp == 0 || re_token[slot]->type == sel_end);
+	return (*next_regexp == 0 || re_token[slot]->type == sel_end) && matched;
 
     switch (re_token[slot]->repeat) {
 	case rep_once:
