@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_readable_c =
- *   "$Id: readable.c,v 1.13 2003/09/13 05:01:30 mwedel Exp $";
+ *   "$Id: readable.c,v 1.19 2005/07/16 21:10:37 akirschbaum Exp $";
  */
 
 /*
@@ -322,7 +322,10 @@ static char *formula_book_name[] =
     "formulary",
     "lab book",
     "lab notes",
-    "recipe book"
+    "recipe book",
+    "experiment record",
+    "work plan",
+    "design notes"
 };
 
 /* this isn't used except for empty books */
@@ -448,7 +451,77 @@ static char *book_descrpt[] =
     "transcendental",
     "weathered"
 };
+/* Each line of this array is a readable subtype
+ * Be careful to keep the order. If you add readable subtype, add them
+ * at the bottom of the list. Never delete a subtype because index is used as
+ * subtype paramater in arch files!
+ */
+static readable_message_type readable_message_types[] = 
+{
+	/*subtype 0  */ {0,0},
+                    /* book messages subtypes */
+	/*subtype 1  */ {MSG_TYPE_BOOK, MSG_TYPE_BOOK_CLASP_1},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_CLASP_2},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_ELEGANT_1},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_ELEGANT_2},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_QUARTO_1},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_QUARTO_2},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_SPELL_EVOKER},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_SPELL_PRAYER},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_SPELL_PYRO},
+	/*subtype 10 */ {MSG_TYPE_BOOK, MSG_TYPE_BOOK_SPELL_SORCERER},
+                    {MSG_TYPE_BOOK, MSG_TYPE_BOOK_SPELL_SUMMONER},
+                    /* card messages subtypes*/
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_SIMPLE_1},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_SIMPLE_2},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_SIMPLE_3},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_ELEGANT_1},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_ELEGANT_2},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_ELEGANT_3},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_STRANGE_1},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_STRANGE_2},
+	/*subtype 20 */ {MSG_TYPE_CARD, MSG_TYPE_CARD_STRANGE_3},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_MONEY_1},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_MONEY_2},
+                    {MSG_TYPE_CARD, MSG_TYPE_CARD_MONEY_3},
+                    
+                    /* Paper messages subtypes */
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_NOTE_1},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_NOTE_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_NOTE_3},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_LETTER_OLD_1},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_LETTER_OLD_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_LETTER_NEW_1},
+	/*subtype 30 */ {MSG_TYPE_PAPER, MSG_TYPE_PAPER_LETTER_NEW_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_ENVELOPE_1},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_ENVELOPE_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_SCROLL_OLD_1},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_SCROLL_OLD_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_SCROLL_NEW_1},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_SCROLL_NEW_2},
+                    {MSG_TYPE_PAPER, MSG_TYPE_PAPER_SCROLL_MAGIC},
 
+                    /* road signs messages subtypes */
+                    {MSG_TYPE_SIGN, MSG_TYPE_SIGN_BASIC},
+                    {MSG_TYPE_SIGN, MSG_TYPE_SIGN_DIR_LEFT},
+	/*subtype 40 */ {MSG_TYPE_SIGN, MSG_TYPE_SIGN_DIR_RIGHT},
+                    {MSG_TYPE_SIGN, MSG_TYPE_SIGN_DIR_BOTH},
+
+                    /* stones and monument messages */
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STONE_1},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STONE_2},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STONE_3},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STATUE_1},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STATUE_2},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_STATUE_3},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_GRAVESTONE_1},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_GRAVESTONE_2},
+	/*subtype 50 */ {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_GRAVESTONE_3},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_WALL_1},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_WALL_2},
+                    {MSG_TYPE_MONUMENT, MSG_TYPE_MONUMENT_WALL_3}
+};
+int last_readable_subtype = sizeof(readable_message_types)/sizeof(readable_message_type);
 
 static int max_titles[6] =
 {
@@ -1062,11 +1135,11 @@ change_book (object *book, int msgtype)
 	      else
 		{
 		    int     numb, maxnames = max_titles[msgtype];
-		    char    old_title[MAX_BUF], old_name[MAX_BUF];
+		    char *old_title;
+		    char *old_name;
 
-		    if (book->title)
-			strcpy (old_title, book->title);
-		    strcpy(old_name, book->name);
+                    old_title = book->title ? add_string(book->title) : NULL;
+                    old_name = add_string(book->name);
 
 		    /* some pre-generated books have title already set (from
 		     * maps), also don't bother looking for unique title if
@@ -1074,7 +1147,7 @@ change_book (object *book, int msgtype)
 
 		    if (!tl)
 		      {
-			  LOG (llevError, "change_book_name(): can't find title list");
+			  LOG (llevError, "change_book_name(): can't find title list\n");
 			  numb = 0;
 		      }
 		    else
@@ -1086,6 +1159,9 @@ change_book (object *book, int msgtype)
 			  LOG (llevDebug, "titles for list %d full (%d possible).\n",
 			       msgtype, maxnames);
 #endif
+			  if (old_title != NULL)
+			      free_string(old_title);
+			  free_string(old_name);
 			  break;
 		      }
 		    /* shouldnt change map-maker books */
@@ -1119,13 +1195,24 @@ change_book (object *book, int msgtype)
 			  free_string (book->title);
 			  if (old_title!=NULL)
 			      book->title = add_string (old_title);
+
+			  if (RANDOM () % 4) {
 			  /* Lets give the book a description to individualize it some */
-			  if (RANDOM () % 4)
-			      sprintf (old_name, "%s %s", book_descrpt[RANDOM () % nbr], old_name);
-			  book->name = add_string (old_name);
+				char new_name[MAX_BUF];
+				snprintf (new_name, MAX_BUF, "%s %s", book_descrpt[RANDOM () % nbr], old_name);
+ 
+	    			book->name = add_string (new_name);
+			  } else {
+				book->name = add_string (old_name);
+			  }
 		      }
-		    else if (book->title && strlen (book->msg) > 5)	/* archive if long msg texts */
+		    else if (book->title && strlen (book->msg) > 5) {	/* archive if long msg texts */
 			add_book_to_list (book, msgtype);
+		    }
+
+		    if (old_title != NULL)
+		        free_string(old_title);
+		    free_string(old_name);
 
 		}
 	      break;
@@ -1201,7 +1288,7 @@ get_random_mon (int level)
 
     if (i == 0)
       {
-	  LOG (llevError, "get_random_mon() couldnt return monster for level %d\n",
+	  LOG (llevError, "get_random_mon() couldn't return monster for level %d\n",
 	       level);
 	  return NULL;
       }
@@ -1521,7 +1608,7 @@ void make_formula_book(object *book, int level) {
 
     if (fl->total_chance == 0)
       {
-	book->msg = add_string(" <indescipherable text>\n");
+	book->msg = add_string(" <indecipherable text>\n");
 	new_text_name(book, 4);
 	add_author(book,4);
 	return;
@@ -1535,11 +1622,8 @@ void make_formula_book(object *book, int level) {
 	    break;
     }
 
-    /* preamble */
-    strcpy (retbuf, "Herein is described an alchemical proceedure: \n");
-
     if (!formula) {
-	book->msg = add_string(" <indescipherable text>\n");
+	book->msg = add_string(" <indecipherable text>\n");
 	new_text_name(book, 4);
 	add_author(book,4);
 	
@@ -1552,6 +1636,10 @@ void make_formula_book(object *book, int level) {
 	archetype *at;
 	int     nindex = nstrtok (formula->arch_name, ",");
 
+	/* preamble */
+	sprintf(retbuf, "Herein is described a project using %s: \n", 
+		 formula->skill?formula->skill:"an unknown skill");
+
 	/* construct name of object to be made */
 	if (nindex > 1)
 	{
@@ -1569,7 +1657,7 @@ void make_formula_book(object *book, int level) {
 	if ((at = find_archetype (op_name)) != (archetype *) NULL)
 	    op_name = at->clone.name;
 	else
-	    LOG (llevError, "formula_msg() can't find arch %s for formula.",
+	    LOG (llevError, "formula_msg() can't find arch %s for formula.\n",
 		   op_name);
 
 	/* item name */
@@ -1610,7 +1698,14 @@ void make_formula_book(object *book, int level) {
 	if (formula->ingred != NULL)
 	{
 	    linked_char *next;
-	    strcat (retbuf, " may be made using the \nfollowing ingredients:\n");
+	    archetype *at;
+
+	    at = find_archetype(formula->cauldron);
+
+	    sprintf(retbuf + strlen(retbuf),
+		    " may be made at %s using the following ingredients:\n",
+		    at?query_name(&at->clone):"an unknown place");
+
 	    for (next = formula->ingred; next != NULL; next = next->next)
 	    {
 		strcat (retbuf, next->name);
@@ -1618,7 +1713,7 @@ void make_formula_book(object *book, int level) {
 	    }
 	}
 	else
-	    LOG (llevError, "formula_msg() no ingredient list for object %s of %s",
+	    LOG (llevError, "formula_msg() no ingredient list for object %s of %s\n",
 		   op_name, formula->title);
 	if (retbuf[strlen(retbuf)-1]!= '\n') strcat(retbuf, "\n");
 	if (book->msg) free_string(book->msg);
@@ -2037,4 +2132,9 @@ write_book_archive (void)
 	  chmod (fname, SAVE_MODE);
       }
 }
-
+readable_message_type* get_readable_message_type(object* readable){
+	uint8 subtype = readable->subtype;
+	if ((subtype<0) || (subtype>last_readable_subtype))
+	    return &(readable_message_types[0]);
+	return &(readable_message_types[subtype]);
+}
