@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c,v 1.48 2004/04/08 06:48:51 mwedel Exp $";
+ *   "$Id: treasure.c,v 1.56 2005/07/16 21:10:37 akirschbaum Exp $";
  */
 
 /*
@@ -46,6 +46,7 @@
 
 
 static void change_treasure(treasure *t, object *op); /* overrule default values */
+extern char *spell_mapping[];
 
 /*
  * Initialize global archtype pointers:
@@ -161,7 +162,7 @@ static void check_treasurelist(treasure *t, treasurelist *tl)
     if (t->item==NULL && t->name==NULL)
 	LOG(llevError,"Treasurelist %s has element with no name or archetype\n", tl->name);
     if (t->chance>=100 && t->next_yes && (t->next || t->next_no))
-	LOG(llevError,"Treasurelist %s has element that has 100% generation, next_yes field as well as next or next_no\n",
+	LOG(llevError,"Treasurelist %s has element that has 100%% generation, next_yes field as well as next or next_no\n",
 		tl->name);
     /* find_treasurelist will print out its own error message */
     if (t->name && strcmp(t->name,"NONE"))
@@ -214,7 +215,7 @@ void load_treasures() {
 		    if (t->next_yes || t->next_no) {
 			LOG(llevError,"Treasure %s is one item, but on treasure %s\n",
 			    tl->name, t->item ? t->item->name : t->name);
-			LOG(llevError,"  the next_yes or next_no field is set");
+			LOG(llevError,"  the next_yes or next_no field is set\n");
 		    }
 #endif
 		    tl->total_chance += t->chance;
@@ -241,7 +242,7 @@ void load_treasures() {
  * of treasurelists which has been built by load_treasures().
  */
 
-treasurelist *find_treasurelist(char *name) {
+treasurelist *find_treasurelist(const char *name) {
   char *tmp=find_string(name);
   treasurelist *tl;
 
@@ -290,9 +291,9 @@ static void put_treasure (object *op, object *creator, int flags)
     } else {
         op = insert_ob_in_ob (op, creator);
         if ((flags & GT_APPLY) && QUERY_FLAG (creator, FLAG_MONSTER))
-            (void) (*monster_check_apply_func) (creator, op);
+            monster_check_apply(creator, op);
         if ((flags & GT_UPDATE_INV) && (tmp = is_player_inv (creator)) != NULL)
-            (*esrv_send_item_func) (tmp, op);
+            esrv_send_item(tmp, op);
     }
 }
 
@@ -748,9 +749,12 @@ int get_magic(int diff) {
  * ! (flags & GT_ENVIRONMENT):
  *     Automatically calls fix_flesh_item().
  *
- * flags & FLAG_STARTEQUIP:
- *     Sets FLAG_STARTEQIUP on item if appropriate, or clears the item's
- *     value.
+ * flags:
+ *    GT_STARTEQUIP: Sets FLAG_STARTEQIUP on item if appropriate, or clears the item's
+ *      value.
+ *    GT_MINIMAL: Does minimal processing on the object - just enough to make it
+ *	a working object - don't change magic, value, etc, but set it material
+ *      type as appropriate, for objects that need spell objects, set those, etc
  */
 void fix_generated_item (object *op, object *creator, int difficulty,
                          int max_magic, int flags)
@@ -768,50 +772,66 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 	create_treasure(op->randomitems, op,flags ,difficulty, 0);
 	if (!op->inv) LOG(llevDebug,"fix_generated_item: Unable to generate treasure for %s\n",
 			  op->name);
+	/* So the treasure doesn't get created again */
+	op->randomitems = NULL;
     }
 
     if (difficulty<1) difficulty=1;
-    if (op->arch == crown_arch) {
-	set_magic(difficulty>25?30:difficulty+5, op, max_magic, flags);
-	num_enchantments = calc_item_power(op, 1);
-	generate_artifact(op,difficulty);
-    } else {
-	if(!op->magic && max_magic)
-	    set_magic(difficulty,op,max_magic, flags);
-	num_enchantments = calc_item_power(op, 1);
-	if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT)) || op->type == HORN ||
-	    difficulty >= 999 )
-	generate_artifact(op, difficulty);
-    }
-    /* Object was made an artifact.  Calculate its item_power rating.
-     * the item_power in the object is what the artfiact adds.
-     */
-    if (op->title) {
-	/* if save_item_power is set, then most likely we started with an
-	 * artifact and have added new abilities to it - this is rare, but
-	 * but I have seen things like 'strange rings of fire'.  So just figure
-	 * out the power from the base power plus what this one adds.  Note
-	 * that since item_power is not quite linear, this actually ends up
-	 * being somewhat of a bonus
-	 */
-	if (save_item_power) {
-	    op->item_power = save_item_power + get_power_from_ench(op->item_power);
+    if (!(flags & GT_MINIMAL)) {
+	if (op->arch == crown_arch) {
+	    set_magic(difficulty>25?30:difficulty+5, op, max_magic, flags);
+	    num_enchantments = calc_item_power(op, 1);
+	    generate_artifact(op,difficulty);
 	} else {
-	    op->item_power = get_power_from_ench(op->item_power + num_enchantments);
+	    if(!op->magic && max_magic)
+		set_magic(difficulty,op,max_magic, flags);
+	    num_enchantments = calc_item_power(op, 1);
+	    if ((!was_magic && !(RANDOM()%CHANCE_FOR_ARTIFACT)) || op->type == HORN ||
+		difficulty >= 999 )
+	    generate_artifact(op, difficulty);
 	}
-    } else if (save_item_power) {
-	/* restore the item_power field to the object if we haven't changed it.
-	 * we don't care about num_enchantments - that will basically just
-	 * have calculated some value from the base attributes of the archetype.
+
+	/* Object was made an artifact.  Calculate its item_power rating.
+	 * the item_power in the object is what the artfiact adds.
 	 */
-	op->item_power = save_item_power;
+	if (op->title) {
+	    /* if save_item_power is set, then most likely we started with an
+	     * artifact and have added new abilities to it - this is rare, but
+	     * but I have seen things like 'strange rings of fire'.  So just figure
+	     * out the power from the base power plus what this one adds.  Note
+	     * that since item_power is not quite linear, this actually ends up
+	     * being somewhat of a bonus
+	     */
+	    if (save_item_power) {
+		op->item_power = save_item_power + get_power_from_ench(op->item_power);
+	    } else {
+		op->item_power = get_power_from_ench(op->item_power + num_enchantments);
+	    }
+	} else if (save_item_power) {
+	    /* restore the item_power field to the object if we haven't changed it.
+	     * we don't care about num_enchantments - that will basically just
+	     * have calculated some value from the base attributes of the archetype.
+	     */
+	    op->item_power = save_item_power;
+	}
     }
 
     /* materialtype modifications.  Note we allow this on artifacts. */
 
     set_materialname(op, difficulty, NULL);
 
-    if (!op->title) /* Only modify object if not special */
+    if (flags & GT_MINIMAL) {
+	if (op->type == POTION)
+	    /* Handle healing and magic power potions */
+	    if (op->stats.sp && !op->randomitems) {
+		object *tmp;
+
+		tmp = get_archetype(spell_mapping[op->stats.sp]);
+		insert_ob_in_ob(tmp, op);
+		op->stats.sp=0;
+	    }
+    }
+    else if (!op->title) /* Only modify object if not special */
 	switch(op->type) {
 	    case WEAPON:
 	    case ARMOUR:
@@ -823,7 +843,7 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 		break;
 
 	    case BRACERS:
-		if(!(RANDOM()%(QUERY_FLAG(op, FLAG_CURSED)?5:20))) {
+		if (!(RANDOM()%(QUERY_FLAG(op, FLAG_CURSED)?5:20))) {
 		    set_ring_bonus(op,QUERY_FLAG(op, FLAG_CURSED)?-DICE2:DICE2);
 		    if (!QUERY_FLAG(op, FLAG_CURSED))
 			op->value*=3;
@@ -832,7 +852,6 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 
 	    case POTION: {
 		int too_many_tries=0,is_special=0;
-		extern char *spell_mapping[];
 
 		/* Handle healing and magic power potions */
 		if (op->stats.sp && !op->randomitems) {
@@ -989,11 +1008,11 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 		break;
 
 	    case RUNE:
-		(*trap_adjust_func)(op,difficulty);
+		trap_adjust(op,difficulty);
 		break;
 		
 		case TRAP:
-		(*trap_adjust_func)(op,difficulty);
+		trap_adjust(op,difficulty);
 		break;
 	} /* switch type */
 
@@ -1220,7 +1239,7 @@ void init_artifacts() {
 	else if (sscanf(cp, "difficulty %d", &value))
 	    art->difficulty = (uint8) value;
 	else if (!strncmp(cp, "Object",6)) {
-	    art->item = (object *) malloc(sizeof(object));
+	    art->item = (object *) calloc(1, sizeof(object));
 	    reset_object(art->item);
 	    if (!load_object(fp, art->item,LO_LINEMODE,0))
 		LOG(llevError,"Init_Artifacts: Could not load object.\n");
@@ -1237,7 +1256,7 @@ void init_artifacts() {
 		art = NULL;
 	}
 	else
-	    LOG(llevError,"Unkown input in artifact file: %s\n", buf);
+	    LOG(llevError,"Unknown input in artifact file: %s\n", buf);
     }
 
     close_and_delete(fp, comp);
