@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_utils_c =
- *   "$Id: utils.c,v 1.12 2003/11/10 05:39:50 mwedel Exp $";
+ *   "$Id: utils.c,v 1.18 2005/07/17 11:00:03 ryo_saeba Exp $";
  */
 
 /*
@@ -50,7 +50,7 @@
  */
 
 int random_roll(int min, int max, object *op, int goodbad) {
-    int omin, diff, luck, base;
+    int omin, diff, luck, base, ran;
 
     omin = min;
     diff = max - min + 1;
@@ -61,8 +61,10 @@ int random_roll(int min, int max, object *op, int goodbad) {
       return(min); /* avoids a float exception */
     }
 
+    ran = RANDOM();
+
     if (op->type != PLAYER)
-	return((RANDOM()%diff)+min);
+	return((ran%diff)+min);
 
     luck = op->stats.luck;
     if (RANDOM()%base < MIN(10, abs(luck))) {
@@ -73,9 +75,9 @@ int random_roll(int min, int max, object *op, int goodbad) {
 	    return(omin); /*check again*/
 	((goodbad) ? (min += luck) : (diff));
 
-	return(MAX(omin, MIN(max, (RANDOM()%diff)+min)));
+	return(MAX(omin, MIN(max, (ran%diff)+min)));
     }
-    return((RANDOM()%diff)+min);
+    return((ran%diff)+min);
 }
 
 /*
@@ -92,7 +94,11 @@ sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad) {
     ((diff > 2) ? (base = 20) : (base = 50)); /* d2 and d3 are corner cases */
 
     if (max < 1 || diff < 1) {
-      LOG(llevError, "Calling random_roll with min=%d max=%d\n", min, max);
+#ifndef WIN32
+      LOG(llevError, "Calling random_roll with min=%lld max=%lld\n", min, max);
+#else
+      LOG(llevError, "Calling random_roll with min=%I64d max=%I64d\n", min, max);
+#endif
       return(min); /* avoids a float exception */
     }
 
@@ -129,7 +135,7 @@ sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad) {
  */
 
 int die_roll(int num, int size, object *op, int goodbad) {
-    int min, diff, luck, total, i, gotlucky, base;
+    int min, diff, luck, total, i, gotlucky, base, ran;
 
     diff = size;
     min = 1;
@@ -152,7 +158,8 @@ int die_roll(int num, int size, object *op, int goodbad) {
 	    if (diff < 1)
 		return(num); /*check again*/
 	    ((goodbad) ? (min += luck) : (diff));
-	    total += MAX(1, MIN(size, (RANDOM()%diff)+min));
+	    ran = RANDOM();
+	    total += MAX(1, MIN(size, (ran%diff)+min));
 	} else {
 	    total += RANDOM()%size+1;
 	}
@@ -243,7 +250,7 @@ void decay_objects(mapstruct *m)
 
 /* convert materialname to materialtype_t */
 
-materialtype_t *name_to_material(char *name)
+materialtype_t *name_to_material(const char *name)
 {
     materialtype_t *mt, *nmt;
 
@@ -359,3 +366,45 @@ void set_materialname(object *op, int difficulty, materialtype_t *nmt)
 	}
     }
 }
+
+/*
+ * Strip out the media tags from a String.
+ * Warning the input string will contain the result string
+ */
+void strip_media_tag(char *message){
+    int in_tag=0;
+    char* dest;
+    char* src;
+    src=dest=message;
+    while (*src!='\0'){
+        if (*src=='['){
+            in_tag=1;
+        } else if (in_tag && (*src==']'))
+            in_tag=0;
+        else if (!in_tag){
+            *dest=*src;
+            dest++;
+        }
+        src++;
+    }
+    *dest='\0';
+}
+
+const char* strrstr(const char* haystack, const char* needle){
+    const char* lastneedle;
+    lastneedle=NULL;
+    while(haystack=strstr(haystack,needle)){
+        lastneedle=haystack;
+        haystack++;
+    }
+    return lastneedle;
+        
+}
+#define EOL_SIZE (sizeof("\n")-1)
+void strip_endline(char* buf){
+    if (strlen(buf)<sizeof("\n")){
+        return;  
+    }
+    if (!strcmp(buf+strlen(buf)-EOL_SIZE,"\n"))
+        buf[strlen(buf)-EOL_SIZE]='\0';    
+}
