@@ -80,7 +80,8 @@ mapstruct *MapMoveScrollResize(mapstruct *source,
     if (!height) height = sy;
     target = get_empty_map (width, height);
 
-    strncpy (target->path, source->path, BIG_NAME);
+    strncpy (target->path, source->path, sizeof(target->path)-1);
+    target->path[sizeof(target->path)-1] = '\0';
 
     MAP_WIDTH(target) = width;
     MAP_HEIGHT(target) = height;  
@@ -488,7 +489,8 @@ static Boolean Load(Edit self, char *name)
 	 * can check for Null map data pretty easy, but can't really check
 	 * for unknown map data.
 	 */
-	XtVaSetValues(self->w, XtNmap, NULL, NULL);
+	if (self->w) /* only when already a window, open otherwhise coredump */
+	    XtVaSetValues(self->w, XtNmap, NULL, NULL);
 	sprintf(buf,"Cannot load map %s",path);
 	CnvNotify(buf,"Continue",NULL);
 	if(!(tmp = ready_map_name(save,MAP_FLUSH))) return False;
@@ -936,6 +938,7 @@ static char * MapNameCreate (String path)
     int i,j;
 
     strncpy(name,path,PATH_MAX);
+    name[PATH_MAX] = '\0';
     for(i = strlen(name)-1; i && name[i] != '/';i--);
     i++;
     for(j = i; name[j] && name[j] != '.';j++);
