@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_png_c =
- *   "$Id: png.c,v 1.7 2002/07/15 04:25:39 mwedel Exp $";
+ *   "$Id: png.c,v 1.8 2005/07/16 21:10:37 akirschbaum Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -121,7 +121,7 @@ again:
 	}
 	else {
 #if 0
-	    fprintf(stderr,"Unable to allocate color %d %d %d, %d colors alloced, will use closenss value %d\n",
+	    fprintf(stderr,"Unable to allocate color %d %d %d, %d colors alloced, will use closeness value %d\n",
 		    red, green, blue, colors_alloced, closeness);
 #endif
 	    colors_alloced = colormap_size;	/* Colormap is exhausted */
