@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_config_h =
- *   "$Id: config.h,v 1.58 2004/02/17 00:35:53 mwedel Exp $";
+ *   "$Id: config.h,v 1.65 2005/08/08 06:27:29 mwedel Exp $";
  */
 
 /*
@@ -131,8 +131,14 @@
 #define NOT_PERMADETH TRUE
 #define EXPLORE_MODE FALSE
 #define STAT_LOSS_ON_DEATH FALSE
+#define PK_LUCK_PENALTY 1
 #define CASTING_TIME FALSE
 #define SET_FRIENDLY_FIRE 5
+#define ARMOR_MAX_ENCHANT   5
+#define ARMOR_WEIGHT_REDUCTION  10
+#define ARMOR_WEIGHT_LINEAR TRUE
+#define ARMOR_SPEED_IMPROVEMENT 10
+#define ARMOR_SPEED_LINEAR  TRUE
 
 /* you can edit the ones below */
 
@@ -377,9 +383,9 @@
 
 #ifndef LOGFILE
 #ifdef WIN32 /* change define path */
-#define LOGFILE "tmp\\cross.log"
+#define LOGFILE "var\\crossfire.log"
 #else 
-#define LOGFILE "/tmp/cross.log"
+#define LOGFILE "/var/log/crossfire/logfile"
 #endif
 #endif
 
@@ -589,6 +595,7 @@
 
 #define MAPDIR		"maps"
 #define ARCHETYPES	"archetypes"
+#define REGIONS	"regions"
 #define HIGHSCORE	"highscore"
 #define TREASURES	"treasures"
 #define	BANISHFILE  "banish_file"
