@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h,v 1.72 2004/02/17 03:38:58 mwedel Exp $";
+ *   "$Id: define.h,v 1.86 2005/05/29 17:35:53 tchize Exp $";
  */
 
 /*
@@ -57,7 +57,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define FONTNAME ""
 
 /* Decstations have trouble with fabs()... */
-#define FABS(x) (x<0?-x:x)
+#define FABS(x) ((x)<0?-(x):(x))
 
 #ifdef __NetBSD__
 #include <sys/param.h>
@@ -69,6 +69,11 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define MAX(x,y) ((x)>(y)?(x):(y))
 #endif
 
+/*NAME_MAX used by random maps may not be defined on pure ansi systems*/
+#ifndef NAME_MAX
+#define NAME_MAX 255
+#endif
+
 /* MAX3 is basically like MAX, but instead does 3 values. */
 #ifndef MAX3
 #define MAX3(x,y, z) (MAX(x, MAX(y,z)))
@@ -91,7 +96,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 
 #define MAX_ANIMATIONS		256
 
-#define MAX_NAME 16
+#define MAX_NAME 48
 #define BIG_NAME 32
 #define MAX_EXT_TITLE 98
 
@@ -281,6 +286,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 
 #define BUILDER            160 /* Generic item builder, see subtypes */
 #define MATERIAL            161 /* Material for building */
+#define GPS                 162 /* Ground positionning system */
+#define ITEM_TRANSFORMER    163 /* Transforming one item with another */
 /* END TYPE DEFINE */
 
 /* Subtypes for BUILDER */
@@ -342,6 +349,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define PU_SPELLBOOK		0x00100000
 #define PU_SKILLSCROLL		0x00200000
 #define PU_READABLES		0x00400000
+#define PU_MAGIC_DEVICE		0x00800000
 
 
 /* Instead of using arbitrary constants for indexing the
@@ -557,7 +565,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define FLAG_CONTENT_ON_GEN      108
 #define FLAG_IS_A_TEMPLATE       109 /* Object has no ingame life until instanciated*/
 #define FLAG_IS_BUILDABLE        110 /* Can build on item */
-#define NUM_FLAGS                110 /* Should always be equal to the last
+#define FLAG_AFK                 111 /* Player is AFK */
+#define NUM_FLAGS                111 /* Should always be equal to the last
                                       * defined flag.  If you change this,
                                       * make sure you update the flag_links
                                       * in common/loader.l
@@ -603,6 +612,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define EXIT_LEVEL(xyz)		(xyz)->stats.food
 #define EXIT_X(xyz)		(xyz)->stats.hp
 #define EXIT_Y(xyz)		(xyz)->stats.sp
+#define EXIT_ALT_X(xyz)		(xyz)->stats.maxhp
+#define EXIT_ALT_Y(xyz)		(xyz)->stats.maxsp
 
 /* for use by the lighting code */
 #define MAX_LIGHT_RADII		4	/* max radii for 'light' object, really
@@ -617,6 +628,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define F_SELL		1
 #define F_TRUE		2   /* True value of item, unadjusted */
 #define F_NO_BARGAIN	4   /* combine with F_BUY or F_SELL to disable bargaining calc */
+#define F_IDENTIFIED	8   /* flag to calculate value of identified item */
+#define F_NOT_CURSED	16  /* flag to calculate value of uncursed item */
 
 #define DIRX(xyz)	freearr_x[(xyz)->direction]
 #define DIRY(xyz)	freearr_y[(xyz)->direction]
@@ -718,6 +731,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define DARK_FACE1_NAME		"dark1.111"
 #define DARK_FACE2_NAME		"dark2.111"
 #define DARK_FACE3_NAME		"dark3.111"
+#define SMOOTH_FACE_NAME	"default_smoothed.111"
 
 /*
  * Defines for the luck/random functions to make things more readable
@@ -891,7 +905,11 @@ enum apply_flag {
 #  endif
 #endif
 
-#endif /* DEFINE_H */
+/* Returns the weight of the given object. Note: it does not take the number of
+ * items (nrof) into account.
+ */
+#define WEIGHT(op) (op->nrof?op->weight:op->weight+op->carrying)
+
 
 /* Code fastening defines
  * FAST_STRCAT & FAST_STRNCAT will add buf2__ at position pointed by
@@ -914,3 +932,105 @@ enum apply_flag {
   */
 /*#define FAST_STRNCAT(buf__,buf2__,size__) {memcpy (buf__,buf2__,size__);buf__+=size__;\
  if (size__!=strlen(buf2__)) printf ("Error, bad length for %s\n",buf2__);}*/
+
+ 
+ 
+ 
+ 
+ 
+ 
+/*
+ * Warning!
+ * If you add message types here, don't forget
+ * to keep the client up to date too!
+ */
+ 
+  
+/* message types */
+#define MSG_TYPE_BOOK            1
+#define MSG_TYPE_CARD            2
+#define MSG_TYPE_PAPER           3
+#define MSG_TYPE_SIGN            4
+#define MSG_TYPE_MONUMENT        5
+#define MSG_TYPE_SCRIPTED_DIALOG 6
+#define MSG_TYPE_MOTD            7
+#define MSG_TYPE_ADMIN           8
+#define MSG_TYPE_LAST            9
+
+#define MSG_SUBTYPE_NONE         0
+
+/* book messages subtypes */
+#define MSG_TYPE_BOOK_CLASP_1    1
+#define MSG_TYPE_BOOK_CLASP_2    2
+#define MSG_TYPE_BOOK_ELEGANT_1  3
+#define MSG_TYPE_BOOK_ELEGANT_2  4
+#define MSG_TYPE_BOOK_QUARTO_1   5
+#define MSG_TYPE_BOOK_QUARTO_2   6
+#define MSG_TYPE_BOOK_SPELL_EVOKER    7
+#define MSG_TYPE_BOOK_SPELL_PRAYER    8
+#define MSG_TYPE_BOOK_SPELL_PYRO      9
+#define MSG_TYPE_BOOK_SPELL_SORCERER  10
+#define MSG_TYPE_BOOK_SPELL_SUMMONER  11
+
+/* card messages subtypes*/
+#define MSG_TYPE_CARD_SIMPLE_1    1
+#define MSG_TYPE_CARD_SIMPLE_2    2
+#define MSG_TYPE_CARD_SIMPLE_3    3
+#define MSG_TYPE_CARD_ELEGANT_1   4
+#define MSG_TYPE_CARD_ELEGANT_2   5
+#define MSG_TYPE_CARD_ELEGANT_3   6
+#define MSG_TYPE_CARD_STRANGE_1   7
+#define MSG_TYPE_CARD_STRANGE_2   8
+#define MSG_TYPE_CARD_STRANGE_3   9
+#define MSG_TYPE_CARD_MONEY_1     10
+#define MSG_TYPE_CARD_MONEY_2     11
+#define MSG_TYPE_CARD_MONEY_3     12
+
+/* Paper messages subtypes */
+#define MSG_TYPE_PAPER_NOTE_1       1
+#define MSG_TYPE_PAPER_NOTE_2       2
+#define MSG_TYPE_PAPER_NOTE_3       3
+#define MSG_TYPE_PAPER_LETTER_OLD_1 4
+#define MSG_TYPE_PAPER_LETTER_OLD_2 5
+#define MSG_TYPE_PAPER_LETTER_NEW_1 6
+#define MSG_TYPE_PAPER_LETTER_NEW_2 7
+#define MSG_TYPE_PAPER_ENVELOPE_1   8
+#define MSG_TYPE_PAPER_ENVELOPE_2   9
+#define MSG_TYPE_PAPER_SCROLL_OLD_1 10
+#define MSG_TYPE_PAPER_SCROLL_OLD_2 11
+#define MSG_TYPE_PAPER_SCROLL_NEW_1 12
+#define MSG_TYPE_PAPER_SCROLL_NEW_2 13
+#define MSG_TYPE_PAPER_SCROLL_MAGIC 14
+
+/* road signs messages subtypes */
+#define MSG_TYPE_SIGN_BASIC         1
+#define MSG_TYPE_SIGN_DIR_LEFT      2
+#define MSG_TYPE_SIGN_DIR_RIGHT     3
+#define MSG_TYPE_SIGN_DIR_BOTH      4
+
+/* stones and monument messages */
+#define MSG_TYPE_MONUMENT_STONE_1      1
+#define MSG_TYPE_MONUMENT_STONE_2      2
+#define MSG_TYPE_MONUMENT_STONE_3      3
+#define MSG_TYPE_MONUMENT_STATUE_1     4
+#define MSG_TYPE_MONUMENT_STATUE_2     5
+#define MSG_TYPE_MONUMENT_STATUE_3     6
+#define MSG_TYPE_MONUMENT_GRAVESTONE_1 7
+#define MSG_TYPE_MONUMENT_GRAVESTONE_2 8
+#define MSG_TYPE_MONUMENT_GRAVESTONE_3 9
+#define MSG_TYPE_MONUMENT_WALL_1       10
+#define MSG_TYPE_MONUMENT_WALL_2       11
+#define MSG_TYPE_MONUMENT_WALL_3       12
+
+/*some readable flags*/
+
+/* dialog messsage */
+#define MSG_TYPE_DIALOG_NPC            1 /*A message from the npc*/
+#define MSG_TYPE_DIALOG_ANSWER         2 /*One of possible answers*/
+#define MSG_TYPE_DIALOG_ANSWER_COUNT   3 /*Number of possible answers*/
+
+/* admin messages */
+#define MSG_TYPE_ADMIN_RULES           1
+#define MSG_TYPE_ADMIN_NEWS            2
+
+#endif /* DEFINE_H */
