@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_funcpoint_h =
- *   "$Id: funcpoint.h,v 1.11 2003/10/27 03:44:33 temitchell Exp $";
+ *   "$Id: funcpoint.h,v 1.13 2005/07/17 11:00:04 ryo_saeba Exp $";
  */
 
 /*
@@ -33,6 +33,7 @@
 /*
  * Some function types
  */
+
 typedef void (*type_move_apply_func) (object *, object *, object *);
 typedef void (*type_func_int)(int);
 typedef void (*type_func_int_int)(int,int);
@@ -57,29 +58,35 @@ typedef object* (*type_ob_func_ob_int)(object *, int);
 /*
  * These function-pointers are defined in common/glue.c
  * The functions used to set and initialise them are also there.
+ *
+ * Massive change. Those functions are just defined, no callback, and they should be implemented.
+ * This means glue.c code & such can go away almost entirely.
+ * Ryo 2005-07-15
  */
 
-extern void	(*move_apply_func)		(object *, object *, object *);
-extern void	(*draw_info_func)		(int, int, object *, const char *);
-extern void	(*emergency_save_func)		(int);
-extern void	(*fix_auto_apply_func)		(mapstruct *);
-extern void	(*init_blocksview_players_func)	();
-extern void	(*monster_check_apply_func)	(object *, object *);
-extern void	(*process_active_maps_func)	();
-extern void	(*remove_friendly_object_func)	(object *);
-extern void	(*update_buttons_func)		(mapstruct *);
-extern void	(*info_map_func)		(int, mapstruct *, char *);
-extern void	(*move_teleporter_func)		(object *);
-extern void	(*move_firewall_func)		(object *);
-extern void	(*move_creator_func)		(object *);
-extern void  (*move_trigger_marker_func)	(object *);
-extern void	(*move_duplicator_func)		(object *);
-extern void     (*trap_adjust_func)		(object *, int);
-extern void	(*esrv_send_item_func)		(object *, object *);
-extern void	(*esrv_del_item_func)		(player *, int);
-extern void	(*esrv_update_item_func)	(int, object *, object *);
-extern void	(*set_darkness_map_func)	(mapstruct *m);
-extern void     (*dragon_gain_func)             (object *, int, int);
-extern void	(*weather_effect_func)		(char *);
-extern object *	(*find_skill_by_number_func)	(object *, int);
+extern void	move_apply(object *, object *, object *);
+extern void	draw_info(int, int, object *, const char *);
+extern void	emergency_save(int);
+extern void	clean_tmp_files();
+extern void	fix_auto_apply(mapstruct *);
+extern void	init_blocksview_players();
+extern void	monster_check_apply(object *, object *);
+extern void	process_active_maps();
+extern void	remove_friendly_object(object *);
+extern void	update_buttons(mapstruct *);
+extern void	info_map(int, mapstruct *, char *);
+extern void	move_teleporter(object *);
+extern void	move_firewall(object *);
+extern void	move_creator(object *);
+extern void move_marker(object *);
+extern void	move_duplicator(object *);
+extern void trap_adjust(object *, int);
+extern void	esrv_send_item(object *, object *);
+extern void	esrv_del_item(player *, int);
+extern void	esrv_update_item(int, object *, object *);
+extern void	set_darkness_map(mapstruct *m);
+extern void dragon_ability_gain(object *, int, int);
+extern void	weather_effect(const char *);
+extern object *	find_skill_by_number(object *, int);
+
 #endif
