@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.47 2004/04/25 07:17:38 mwedel Exp $";
+ *   "$Id: global.h,v 1.55 2005/05/29 15:37:53 tchize Exp $";
  */
 
 /*
@@ -53,15 +53,27 @@
 
  
 typedef unsigned int	uint32;
+
+#ifndef UINT32_MAX
 #define UINT32_MAX	4294967295U
+#endif
+
 typedef signed int	sint32;
 #define SINT32_MAX	2147483647
+
 typedef unsigned short	uint16;
+#ifndef UINT16_MAX
 #define UINT16_MAX	65535
+#endif
+
 typedef signed short	sint16;
 #define SINT16_MAX	32767
+
 typedef unsigned char	uint8;
+#ifndef UINT8_MAX
 #define UINT8_MAX	255
+#endif
+
 typedef signed char	sint8;
 #define SINT8_MAX	127
 
@@ -80,7 +92,7 @@ typedef signed __int64		sint64;
 #pragma warning( disable: 4244 ) /* conversion from 'xxx' to 'yyy', possible loss of data */
 #pragma warning( disable: 4305 ) /* initializing float f = 0.05; instead of f = 0.05f; */
 
-#else // WIN32
+#else /* WIN32 */
 
 #if SIZEOF_LONG == 8
 typedef unsigned long	    uint64;
@@ -118,7 +130,8 @@ error correct and send mail to crossfire-devel on how to do this
 
 #include "commands.h"
 
-
+/* pull in book structures */
+#include "book.h"
 /*
  * So far only used when dealing with artifacts.
  * (now used by alchemy and other code too. Nov 95 b.t).
@@ -162,6 +175,7 @@ extern New_Face *new_faces;
  */
 EXTERN player *first_player;
 EXTERN mapstruct *first_map;
+EXTERN region *first_region;
 EXTERN treasurelist *first_treasurelist;
 EXTERN artifactlist *first_artifactlist;
 EXTERN archetype *first_archetype;
@@ -235,7 +249,7 @@ extern int leftof_x[9], leftof_y[9];
 
 extern New_Face *blank_face, *empty_face;
 extern New_Face *dark_faces[];
-extern New_Face *potion_face;
+extern New_Face *potion_face, *smooth_face;
 extern MapLook blank_look;
 
 
@@ -313,10 +327,12 @@ typedef struct Settings {
     char    *playerdir;	    /* Where the player files are */
     char    *mapdir;	    /* Where the map files are */
     char    *archetypes;    /* name of the archetypes file - libdir is prepended */
+    char    *regions;       /* name of the regions file - libdir is prepended */
     char    *treasures;	    /* location of the treasures file. */
     char    *uniquedir;	    /* directory for the unique items */
     char    *tmpdir;	    /* Directory to use for temporary files */
     uint8   stat_loss_on_death;	/* If true, chars lose a random stat when they die */
+    sint16  pk_luck_penalty;    /* Amount by which player luck is reduced if they PK */
     uint8   use_permanent_experience; /* If true, players can gain perm exp */
     uint8   balanced_stat_loss; /* If true, Death stat depletion based on level etc */
     uint8   not_permadeth;  /* if true, death is non-permament */
@@ -333,7 +349,11 @@ typedef struct Settings {
     uint8   recycle_tmp_maps; /* re-use tmp maps. */
     uint8   explore_mode;     /* explore mode allowed? */
     uint8   spellpoint_level_depend; /* spell costs go up with level */
+    char    who_format[MAX_BUF]; /* The format that the who command should use */
+    char    who_wiz_format[MAX_BUF]; /* The format that the who command should use when called by a dm*/
     char    motd[MAX_BUF];    /* name of the motd file */
+    char    *rules;           /* name of rules file*/
+    char    *news;            /* name of news file*/
     char    dm_mail[MAX_BUF]; /* DM's Email address */
 
     /* The meta_ is information for the metaserver.  These are set in 
@@ -359,6 +379,12 @@ typedef struct Settings {
 
     sint16  max_level;		    /* This is read out of exp_table */
     float   item_power_factor;	    /* See note in setings file */
+
+    int     armor_max_enchant;  /* Maximum number of times an armor can be enchanted */
+    int     armor_weight_reduction; /* Weight reduction per enchantment */
+    uint8   armor_weight_linear;  /* If 1, weight reduction is linear, else exponantiel */
+    int     armor_speed_improvement;    /* Speed improvement */
+    uint8   armor_speed_linear;         /* If 1, speed improvement is linear, else exponantiel */
 } Settings;
 
 extern Settings settings;
