@@ -4,12 +4,12 @@ extern void init_anim(void);
 extern int find_animation(char *name);
 extern void animate_object(object *op, int dir);
 /* arch.c */
-extern archetype *find_archetype_by_object_name(char *name);
-extern object *get_archetype_by_skill_name(char *skill, int type);
+extern archetype *find_archetype_by_object_name(const char *name);
+extern object *get_archetype_by_skill_name(const char *skill, int type);
 extern archetype *get_archetype_by_type_subtype(int type, int subtype);
-extern object *get_archetype_by_object_name(char *name);
-extern object *find_best_weapon_used_match(object *pl, char *params);
-extern int item_matched_string(object *pl, object *op, char *name);
+extern object *get_archetype_by_object_name(const char *name);
+extern object *find_best_weapon_used_match(object *pl, const char *params);
+extern int item_matched_string(object *pl, object *op, const char *name);
 extern void init_archetypes(void);
 extern void arch_info(object *op);
 extern void clear_archetable(void);
@@ -23,10 +23,10 @@ extern void second_arch_pass(FILE *fp);
 extern void check_generators(void);
 extern void load_archetypes(void);
 extern object *arch_to_object(archetype *at);
-extern object *create_singularity(char *name);
-extern object *get_archetype(char *name);
-extern unsigned long hasharch(char *str, int tablesize);
-extern archetype *find_archetype(char *name);
+extern object *create_singularity(const char *name);
+extern object *get_archetype(const char *name);
+extern unsigned long hasharch(const char *str, int tablesize);
+extern archetype *find_archetype(const char *name);
 extern void add_arch(archetype *at);
 extern archetype *type_to_archetype(int type);
 extern object *clone_arch(int type);
@@ -60,49 +60,7 @@ extern void remove_friendly_object(object *op);
 extern void dump_friendly_objects(void);
 extern void clean_friendly_list(void);
 /* glue.c */
-extern void init_function_pointers(void);
-extern void set_emergency_save(type_func_int addr);
-extern void set_clean_tmp_files(type_func_void addr);
-extern void set_fix_auto_apply(type_func_map addr);
-extern void set_remove_friendly_object(type_func_ob addr);
-extern void set_process_active_maps(type_func_void addr);
-extern void set_update_buttons(type_func_map addr);
-extern void set_draw_info(type_func_int_int_ob_cchar addr);
-extern void set_move_apply(type_move_apply_func addr);
-extern void set_monster_check_apply(type_func_ob_ob addr);
-extern void set_init_blocksview_players(type_func_void addr);
-extern void set_info_map(type_func_int_map_char addr);
-extern void set_move_teleporter(type_func_ob addr);
-extern void set_move_firewall(type_func_ob addr);
-extern void set_trap_adjust(type_func_ob_int addr);
-extern void set_move_creator(type_func_ob addr);
-extern void set_move_trigger_marker(type_func_ob addr);
-extern void set_move_duplicator(type_func_ob addr);
-extern void set_esrv_send_item(type_func_ob_ob addr);
-extern void set_esrv_update_item(type_func_int_ob_ob addr);
-extern void set_esrv_del_item(type_func_player_int addr);
-extern void set_set_darkness_map(type_func_map addr);
-extern void set_dragon_gain_func(type_func_dragon_gain addr);
-extern void set_weather_effect_func(type_func_char addr);
-extern void set_find_skill_by_number_func(type_ob_func_ob_int addr);
 extern void fatal(int err);
-extern void dummy_function_int(int i);
-extern void dummy_function_int_int(int i, int j);
-extern void dummy_function_player_int(player *p, int j);
-extern void dummy_function(void);
-extern void dummy_function_map(mapstruct *m);
-extern void dummy_function_ob(object *ob);
-extern void dummy_function_ob2(object *ob, object *ob2);
-extern int dummy_function_ob2int(object *ob, object *ob2);
-extern void dummy_function_ob_int(object *ob, int i);
-extern object *dummy_ob_function_ob_int(object *ob, int i);
-extern void dummy_function_txtnr(char *txt, int nr);
-extern void dummy_draw_info(int a, int b, object *ob, const char *txt);
-extern void dummy_function_mapstr(int a, mapstruct *map, char *str);
-extern void dummy_function_int_ob_ob(int n, object *ob, object *ob2);
-extern void dummy_move_apply_func(object *ob, object *ob2, object *ob3);
-extern void dummy_function_dragongain(object *ob, int a1, int a2);
-extern void dummy_function_char(char *filename);
 /* holy.c */
 extern void init_gods(void);
 extern void add_god_to_list(archetype *god_arch);
@@ -168,17 +126,17 @@ extern void fix_player(object *op);
 extern int allowed_class(object *op);
 extern void set_dragon_name(object *pl, object *abil, object *skin);
 extern void dragon_level_gain(object *who);
-extern object *give_skill_by_name(object *op, char *skill_name);
+extern object *give_skill_by_name(object *op, const char *skill_name);
 extern void player_lvl_adj(object *who, object *op);
 extern sint64 level_exp(int level, double expmul);
 extern void calc_perm_exp(object *op);
 extern sint64 check_exp_loss(object *op, sint64 exp);
 extern sint64 check_exp_adjust(object *op, sint64 exp);
-extern void change_exp(object *op, sint64 exp, char *skill_name, int flag);
+extern void change_exp(object *op, sint64 exp, const char *skill_name, int flag);
 extern void apply_death_exp_penalty(object *op);
 extern int did_make_save(object *op, int level, int bonus);
 /* logger.c */
-extern void LOG(LogLevel logLevel, char *format, ...);
+extern void LOG(LogLevel logLevel, const char *format, ...);
 /* los.c */
 extern void set_block(int x, int y, int bx, int by);
 extern void init_block(void);
@@ -194,14 +152,14 @@ extern void print_los(object *op);
 extern void make_sure_seen(object *op);
 extern void make_sure_not_seen(object *op);
 /* map.c */
-extern mapstruct *has_been_loaded(char *name);
-extern char *create_pathname(char *name);
-extern char *create_overlay_pathname(char *name);
-extern int check_path(char *name, int prepend_dir);
+extern mapstruct *has_been_loaded(const char *name);
+extern const char *create_pathname(const char *name);
+extern const char *create_overlay_pathname(const char *name);
+extern int check_path(const char *name, int prepend_dir);
 extern void dump_map(mapstruct *m);
 extern void dump_all_maps(void);
 extern int get_map_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sint16 *nx, sint16 *ny);
-extern int blocked_link(object *ob, mapstruct *m, int x, int y);
+extern int blocked_link(object *ob, mapstruct *m, int sx, int sy);
 extern int arch_blocked(archetype *at, mapstruct *m, sint16 x, sint16 y);
 extern void fix_container(object *container);
 extern void load_objects(mapstruct *m, FILE *fp, int mapflags);
@@ -209,14 +167,14 @@ extern void save_objects(mapstruct *m, FILE *fp, FILE *fp2, int flag);
 extern mapstruct *get_linked_map(void);
 extern void allocate_map(mapstruct *m);
 extern mapstruct *get_empty_map(int sizex, int sizey);
-extern mapstruct *load_original_map(char *filename, int flags);
-extern mapstruct *load_overlay_map(char *filename, mapstruct *m);
+extern mapstruct *load_original_map(const char *filename, int flags);
+extern mapstruct *load_overlay_map(const char *filename, mapstruct *m);
 extern int new_save_map(mapstruct *m, int flag);
 extern void clean_object(object *op);
 extern void free_all_objects(mapstruct *m);
 extern void free_map(mapstruct *m, int flag);
 extern void delete_map(mapstruct *m);
-extern mapstruct *ready_map_name(char *name, int flags);
+extern mapstruct *ready_map_name(const char *name, int flags);
 extern int calculate_difficulty(mapstruct *m);
 extern void clean_tmp_map(mapstruct *m);
 extern void free_all_maps(void);
@@ -229,6 +187,7 @@ extern void get_rangevector(object *op1, object *op2, rv_vector *retval, int fla
 extern void get_rangevector_from_mapcoord(mapstruct *m, int x, int y, object *op2, rv_vector *retval, int flags);
 extern int on_same_map(object *op1, object *op2);
 /* object.c */
+extern int CAN_MERGE(object *ob1, object *ob2);
 extern signed long sum_weight(object *op);
 extern object *is_player_inv(object *op);
 extern void dump_object2(object *op);
@@ -262,7 +221,7 @@ extern object *merge_ob(object *op, object *top);
 extern object *insert_ob_in_map_at(object *op, mapstruct *m, object *originator, int flag, int x, int y);
 extern object *insert_ob_in_map(object *op, mapstruct *m, object *originator, int flag);
 extern void replace_insert_ob_in_map(char *arch_string, object *op);
-extern object *get_split_ob(object *orig_ob, int nr);
+extern object *get_split_ob(object *orig_ob, uint32 nr);
 extern object *decrease_ob_nr(object *op, int i);
 extern void add_weight(object *op, signed long weight);
 extern object *insert_ob_in_ob(object *op, object *where);
@@ -301,13 +260,22 @@ extern char *ltostr10(signed long n);
 extern char *doubletostr10(double v);
 extern void save_long(char *buf, char *name, long n);
 extern void save_long_long(char *buf, char *name, sint64 n);
-extern FILE *open_and_uncompress(char *name, int flag, int *compressed);
+extern FILE *open_and_uncompress(const char *name, int flag, int *compressed);
 extern void close_and_delete(FILE *fp, int compressed);
 extern void make_path_to_file(char *filename);
 /* player.c */
 extern void free_player(player *pl);
 extern int atnr_is_dragon_enabled(int attacknr);
 extern int is_dragon_pl(object *op);
+/* quest.c */
+extern int quest_is_same_quest( const char* slaying1, const char* slaying2 );
+extern int quest_marker_compatible( object* marker, object* player );
+extern void quest_clear_markers( object* marker, object* player );
+extern char* quest_message_check( char* message, object* pl );
+extern const char* quest_get_name( object* marker );
+extern int quest_is_quest_marker( object* marker );
+extern int quest_is_start( const char* slaying );
+extern int quest_is_end( const char* slaying );
 /* re-cmp.c */
 extern char *re_cmp(char *str, char *regexp);
 /* readable.c */
@@ -328,12 +296,13 @@ extern char *god_info_msg(int level, int booksize);
 extern void tailor_readable_ob(object *book, int msg_type);
 extern void free_all_readable(void);
 extern void write_book_archive(void);
+extern readable_message_type* get_readable_message_type(object* readable);
 /* recipe.c */
 extern recipelist *get_formulalist(int i);
 extern void init_formulae(void);
 extern void check_formulae(void);
 extern void dump_alchemy(void);
-extern archetype *find_treasure_by_name(treasure *t, char *name, int depth);
+extern archetype *find_treasure_by_name(treasure *t, const char *name, int depth);
 extern long find_ingred_cost(char *name);
 extern void dump_alchemy_costs(void);
 extern char *ingred_name(char *name);
@@ -343,6 +312,18 @@ extern int numb_ingred(char *buf);
 extern recipelist *get_random_recipelist(void);
 extern recipe *get_random_recipe(recipelist *rpl);
 extern void free_all_recipes(void);
+/* region.c */
+extern region *get_region_by_name(const char *region_name);
+extern region *get_region_by_map(mapstruct *m);
+extern const char *get_name_of_region_for_map(mapstruct *m);
+extern region *get_region_from_string(const char *name);
+extern int region_is_child_of_region(region *child, region *r);
+extern const char *get_region_longname(region *r);
+extern const char *get_region_msg(region *r);
+extern void init_regions(void);
+extern region *get_region_struct(void);
+extern void parse_regions(FILE *fp);
+extern void assign_region_parents(void);
 /* shstr.c */
 extern void init_hash_table(void);
 extern char *add_string(const char *str);
@@ -366,7 +347,7 @@ extern long seconds(void);
 /* treasure.c */
 extern void init_archetype_pointers(void);
 extern void load_treasures(void);
-extern treasurelist *find_treasurelist(char *name);
+extern treasurelist *find_treasurelist(const char *name);
 extern void create_all_treasures(treasure *t, object *op, int flag, int difficulty, int tries);
 extern void create_one_treasure(treasurelist *tl, object *op, int flag, int difficulty, int tries);
 extern void create_treasure(treasurelist *t, object *op, int flag, int difficulty, int tries);
@@ -398,9 +379,12 @@ extern sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad);
 extern int die_roll(int num, int size, object *op, int goodbad);
 extern int rndm(int min, int max);
 extern void decay_objects(mapstruct *m);
-extern materialtype_t *name_to_material(char *name);
+extern materialtype_t *name_to_material(const char *name);
 extern void transmute_materialname(object *op, object *change);
 extern void set_materialname(object *op, int difficulty, materialtype_t *nmt);
+extern void strip_media_tag(char *message);
+extern const char* strrstr(const char* haystack, const char* needle);
+extern void strip_endline(char* buf);
 /* loader.c */
 extern int lex_load(object *op, int map_flags);
 extern void yyrestart(FILE *input_file);
