@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_h =
- *   "$Id: living.h,v 1.9 2003/09/13 05:01:33 mwedel Exp $";
+ *   "$Id: living.h,v 1.11 2005/06/01 05:58:03 akirschbaum Exp $";
  */
 
 /*
@@ -53,16 +53,13 @@ extern int thaco_bonus[MAX_STAT + 1];
 extern int turn_bonus[MAX_STAT + 1];
 extern int max_carry[MAX_STAT + 1];
 extern int dam_bonus[MAX_STAT + 1];
-extern int savethrow[111];
-extern int turn_bonus[MAX_STAT + 1];
-extern int learn_prayer_chance[MAX_STAT + 1];
 extern int learn_spell[];
 extern char *restore_msg[NUM_STATS];
 extern char *statname[NUM_STATS];
 extern char *short_stat_name[NUM_STATS];
 extern char *lose_msg[NUM_STATS];
 extern float speed_bonus[MAX_STAT + 1];
-extern int weight_limit[MAX_STAT + 1];
+extern uint32 weight_limit[MAX_STAT + 1];
 extern int cleric_chance[MAX_STAT + 1];
 extern int fear_bonus[MAX_STAT + 1];
 
