@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newserver_h =
- *   "$Id: newserver.h,v 1.29 2003/09/13 05:01:33 mwedel Exp $";
+ *   "$Id: newserver.h,v 1.31 2005/05/06 21:10:16 tchize Exp $";
  */
 
 /*
@@ -103,7 +103,7 @@ typedef struct NewSocket {
     enum Sock_Status status;
     int fd;
     struct Map lastmap;
-    uint8 faces_sent[MAXFACENUM];
+    uint8 faces_sent[MAXFACENUM];   /* This is a bitmap on sent face status */
     uint8 anims_sent[MAXANIMNUM];
     struct statsinfo stats;
     /* If we get an incomplete packet, this is used to hold the data. */
@@ -121,6 +121,9 @@ typedef struct NewSocket {
     uint32  image2:1;	    /* Client wants image2/face2 commands */
     uint32  update_look:1;  /* If true, we need to send the look window */
     uint32  can_write:1;    /* Can we write to this socket? */
+    uint32  has_readable_type:1; /* If true client accept additional text information
+                                    used to arrange text in books, scrolls, or scripted dialogs */
+    uint32  supported_readables; /* each bit is a readable supported by client */                    
     uint32  cs_version, sc_version; /* versions of the client */
     enum MapMode mapmode;   /* Type of map commands the client wants. */
     uint16  look_position;  /* start of drawing of look window */
@@ -130,14 +133,27 @@ typedef struct NewSocket {
     uint32  ext_mapinfos:1;  /* If true client accept additionnal info on maps*/
     /* Below are flags for extedend infos to pass to client 
      * with S->C mapextended command */
-    uint32  EMI_smooth:1;   /* Send smooth in extendmapinfos*/
+    uint32  EMI_smooth:1;   /* Send smooth in extendmapinfos*/    
 
     /* Below here is information only relevant for old sockets */
     char    *comment;	    /* name or listen comment */
     enum Old_Mode old_mode;
 } NewSocket;
 
-/*Constants in the form EMI_ is for extended map infos.
+
+#define CLIENT_SUPPORT_READABLES(__sockPtr,__type)\
+	( ((__type)>0) &&\
+	  ((__sockPtr)->has_readable_type) && \
+	  ((__sockPtr)->supported_readables & (1<<(__type))) )
+
+
+/* Bitmask for the faces_sent[] array - what
+ * portion of the face have we sent?
+ */
+#define NS_FACESENT_FACE	0x1
+#define NS_FACESENT_SMOOTH	0x2
+
+/* Constants in the form EMI_ is for extended map infos.
  * Even if the client select the additionnal infos it wants
  * on the map, there may exist cases where this whole info
  * is not given in one buch but in separate bunches. This 
@@ -154,7 +170,8 @@ typedef struct NewSocket {
  */ 
 #define EMI_NOREDRAW        0x01  
 #define EMI_SMOOTH          0x02
-/*this last one says the bitfield continue un next byte
+
+/* this last one says the bitfield continue un next byte
  * There may be several on contiguous bytes. So there is 7
  * actual bits used per byte, and the number of bytes
  * is not fixed in protocol
