@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: object.h,v 1.34 2004/02/16 18:05:32 ryo_saeba Exp $";
+ *   "$Id: object.h,v 1.36 2005/08/10 13:38:31 ryo_saeba Exp $";
  */
 
 /*
@@ -305,4 +305,9 @@ extern int nroffreeobjects;
 #define INS_ON_TOP		0x0008
 #define INS_BELOW_ORIGINATOR	0x0010
 
+#define ARCH_SINGULARITY        "singularity"
+#define ARCH_SINGULARITY_LEN    11
+#define ARCH_DETECT_MAGIC       "detect_magic"
+#define ARCH_DEPLETION          "depletion"
+
 #endif
