@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h,v 1.35 2004/02/16 18:05:32 ryo_saeba Exp $";
+ *   "$Id: player.h,v 1.38 2005/07/16 19:03:02 qal21 Exp $";
  */
 
 /*
@@ -67,7 +67,8 @@ typedef enum _bowtype {
 typedef enum _petmode {
     pet_normal = 0,
     pet_sad = 1,
-    pet_defend = 2
+    pet_defend = 2,
+    pet_arena = 3
 } petmode_t;
 
 typedef enum usekeytype {
@@ -137,6 +138,7 @@ typedef struct pl {
     float	weapon_sp;	    /* Penalties to speed when fighting w speed >ws/10*/
     float	last_weapon_sp;	    /* if diff than weapon_sp, update client */
     uint16	last_flags;	    /* fire/run on flags for last tick */
+    uint32	last_weight;	    /* Last weight as sent to client; (uint32)-1 means do not send weight */
     uint32	last_weight_limit;  /* Last weight limit transmitted to client */
     living	orig_stats;	    /* Permanent real stats of player */
     living	last_stats;	    /* Last stats as sent to client */
@@ -183,6 +185,9 @@ typedef struct pl {
     uint16	outputs_count;	    /* Print if this count is exceeded */
     object	*mark;		    /* marked object */
     uint32	mark_count;	    /* count of mark object */
+    /* Special DM fields */
+    tag_t*  stack_items;    /* Item stack for patch/dump/... commands */
+    int     stack_position; /* Current stack position, 0 for no item */
 } player;
 
 
