@@ -80,9 +80,11 @@
 #define EVENT_MAPRESET 22 /* A map is resetting.                             */
 #define EVENT_REMOVE   23 /* A Player character has been removed.            */
 #define EVENT_SHOUT    24 /* A player 'shout' something.                     */
-#define EVENT_TELL     25 /* A player 'tell' something.                      */
+#define EVENT_TELL     25 /* A player 'tell' something.						 */
+#define EVENT_MUZZLE   26 /* A player was Muzzled (no_shout set).			 */
+#define EVENT_KICK     27 /* A player was Kicked by a DM                     */
 
-#define NR_EVENTS 26
+#define NR_EVENTS 28
 
 /*****************************************************************************/
 /* Hook codes. A hook is a function pointer passed from the server to the    */
@@ -157,7 +159,21 @@
 #define HOOK_DECREASEOBJECTNR    61
 #define HOOK_FREEMEMORY          62
 #define HOOK_FIXPLAYER           63
-#define NR_OF_HOOKS             64
+#define HOOK_CHECKTRIGGER        64
+#define HOOK_OUTOFMAP            65
+#define HOOK_QUERYNAME           66
+#define HOOK_QUERYBASENAME       67
+#define HOOK_INSERTOBINOB        68
+#define HOOK_GETMAPFLAGS         69
+#define HOOK_GETSETTINGS         70
+#define HOOK_PRESENTARCHBYNAME   71
+#define HOOK_STRDUPLOCAL         72
+#define HOOK_RECMP               73
+#define HOOK_CREATEPATHNAME      74
+#define HOOK_UPDATEOBSPEED       75
+#define HOOK_PRESENTARCHNAMEINOB 76
+#define HOOK_SETDIRECTION        77
+#define NR_OF_HOOKS             78
 
 /*****************************************************************************/
 /* CFParm is the data type used to pass informations between the server and  */
