@@ -12,9 +12,9 @@ void remove_contents(object *first_ob, object *save_item);
 int calc_alch_danger(object *caster, object *cauldron, recipe *rp);
 /* apply.c */
 int apply_potion(object *op, object *tmp);
-int check_item(object *op, char *item);
+int check_item(object *op, const char *item);
 int check_weapon_power(object *who, int improvs);
-int improve_weapon_stat(object *op, object *improver, object *weapon, signed char *stat, int sacrifice_count, char *statname);
+int improve_weapon_stat(object *op, object *improver, object *weapon, signed char *stat, int sacrifice_count, const char *statname);
 int prepare_weapon(object *op, object *improver, object *weapon);
 int improve_weapon(object *op, object *improver, object *weapon);
 int check_improve_weapon(object *op, object *tmp);
@@ -24,7 +24,7 @@ int apply_container(object *op, object *sack);
 int esrv_apply_container(object *op, object *sack);
 void move_apply(object *trap, object *victim, object *originator);
 void do_learn_spell(object *op, object *spell, int special_prayer);
-void do_forget_spell(object *op, char *spell);
+void do_forget_spell(object *op, const char *spell);
 void apply_scroll(object *op, object *tmp, int dir);
 int dragon_eat_flesh(object *op, object *meal);
 void apply_poison(object *op, object *tmp);
@@ -43,6 +43,8 @@ void eat_special_food(object *who, object *food);
 void apply_lighter(object *who, object *lighter);
 void scroll_failure(object *op, int failure, int power);
 void apply_changes_to_player(object *pl, object *change);
+void apply_positioning_system(object *pl, object *gps);
+void apply_item_transformer(object *pl, object *transformer);
 /* attack.c */
 void cancellation(object *op);
 int did_make_save_item(object *op, int type, object *originator);
@@ -66,18 +68,25 @@ void deathstrike_player(object *op, object *hitter, int *dam);
 int adj_attackroll(object *hitter, object *target);
 int is_aimed_missile(object *op);
 /* ban.c */
-int checkbanned(char *login, char *host);
+int checkbanned(const char *login, const char *host);
 /* build_map.c */
 void remove_marking_runes(struct mapdef *map, short x, short y);
 int find_unused_connected_value(struct mapdef *map);
+int find_or_create_connection_for_map(object *pl, short x, short y);
 object *get_wall(struct mapdef *map, int x, int y);
 void fix_walls(struct mapdef *map, int x, int y);
+void apply_builder_floor(object *pl, object *material, short x, short y);
+void apply_builder_wall(object *pl, object *material, short x, short y);
+void apply_builder_item(object *pl, object *item, short x, short y);
+void apply_builder_remove(object *pl, int dir);
+void apply_map_builder(object *pl, int dir);
 /* c_chat.c */
 int command_say(object *op, char *params);
 int command_me(object *op, char *params);
 int command_cointoss(object *op, char *params);
 int command_orcknuckle(object *op, char *params);
 int command_shout(object *op, char *params);
+int command_chat(object *op, char *params);
 int command_tell(object *op, char *params);
 int command_reply(object *op, char *params);
 int command_nod(object *op, char *params);
@@ -140,10 +149,16 @@ int command_body(object *op, char *params);
 int command_motd(object *op, char *params);
 int command_bug(object *op, char *params);
 void malloc_info(object *op);
+void current_region_info(object *op);
 void current_map_info(object *op);
+int command_whereabouts(object *op, char *params);
 int command_who(object *op, char *params);
+void display_who_entry(object *op, player *pl, char *format);
+void get_who_escape_code_value(char *return_val, const char letter, player *pl);
+int command_afk(object *op, char *params);
 int command_malloc(object *op, char *params);
 int command_mapinfo(object *op, char *params);
+int command_whereami(object *op, char *params);
 int command_maps(object *op, char *params);
 int command_strings(object *op, char *params);
 int command_sstable(object *op, char *params);
@@ -190,6 +205,7 @@ int command_wimpy(object *op, char *params);
 int command_brace(object *op, char *params);
 int command_style_map_info(object *op, char *params);
 int command_kill_pets(object *op, char *params);
+int command_quests(object *pl, char *params);
 /* c_move.c */
 int command_east(object *op, char *params);
 int command_north(object *op, char *params);
@@ -220,7 +236,7 @@ int sack_can_hold(object *pl, object *sack, object *op, int nrof);
 void pick_up(object *op, object *alt);
 int command_take(object *op, char *params);
 void put_object_in_sack(object *op, object *sack, object *tmp, long nrof);
-void drop_object(object *op, object *tmp, long nrof);
+void drop_object(object *op, object *tmp, uint32 nrof);
 void drop(object *op, object *tmp);
 int command_dropall(object *op, char *params);
 int command_drop(object *op, char *params);
@@ -254,6 +270,7 @@ void change_spell(object *op, char k);
 int command_rotateshoottype(object *op, char *params);
 /* c_wiz.c */
 int command_loadtest(object *op, char *params);
+void do_wizard_hide(object *op, int silent_dm);
 int command_hide(object *op, char *params);
 int command_setgod(object *op, char *params);
 int command_banish(object *op, char *params);
@@ -281,14 +298,26 @@ int command_stats(object *op, char *params);
 int command_abil(object *op, char *params);
 int command_reset(object *op, char *params);
 int command_nowiz(object *op, char *params);
+int do_wizard_dm(object *op, char *params, int silent);
 int command_dm(object *op, char *params);
 int command_invisible(object *op, char *params);
+object *get_spell_by_name(char *spell_name);
 int command_learn_spell(object *op, char *params);
 int command_learn_special_prayer(object *op, char *params);
 int command_forget_spell(object *op, char *params);
 int command_listplugins(object *op, char *params);
 int command_loadplugin(object *op, char *params);
 int command_unloadplugin(object *op, char *params);
+int command_dmhide(object *op, char *params);
+void dm_stack_pop(player *pl);
+object *dm_stack_peek(player *pl);
+void dm_stack_push(player *pl, tag_t item);
+object *get_dm_object(player *pl, char **params, int *from);
+int command_stack_pop(object *op, char *params);
+int command_stack_push(object *op, char *params);
+int command_stack_list(object *op, char *params);
+int command_stack_clear(object *op, char *params);
+int command_diff(object *op, char *params);
 /* commands.c */
 void init_commands(void);
 CommFunc find_oldsocket_command(char *cmd);
@@ -311,7 +340,7 @@ object *find_disease(object *victim);
 int cure_disease(object *sufferer, object *caster);
 int reduce_symptoms(object *sufferer, int reduction);
 /* egoitem.c */
-object *create_artifact(object *op, char *artifactname);
+int create_artifact(object *op, char *artifactname);
 int apply_power_crystal(object *op, object *crystal);
 /* hiscore.c */
 char *spool(char *bp, char *error);
@@ -355,6 +384,7 @@ void set_confdir(char *path);
 void set_localdir(char *path);
 void set_mapdir(char *path);
 void set_archetypes(char *path);
+void set_regions(char *path);
 void set_treasures(char *path);
 void set_uniquedir(char *path);
 void set_playerdir(char *path);
@@ -376,7 +406,6 @@ void rec_sigbus(int i);
 void rec_sigterm(int i);
 void fatal_signal(int make_core, int close_sockets);
 void init_signals(void);
-void setup_library(void);
 void init_races(void);
 void dump_races(void);
 void add_to_racelist(char *race_name, object *op);
@@ -402,7 +431,7 @@ void enter_player_savebed(object *op);
 void leave_map(object *op);
 void set_map_timeout(mapstruct *oldmap);
 char *clean_path(char *file);
-char *unclean_path(char *src);
+char *unclean_path(const char *src);
 void enter_exit(object *op, object *exit_ob);
 void process_active_maps(void);
 void process_players1(mapstruct *map);
@@ -452,7 +481,7 @@ void check_earthwalls(object *op, mapstruct *m, int x, int y);
 void check_doors(object *op, mapstruct *m, int x, int y);
 void communicate(object *op, char *txt);
 int talk_to_npc(object *op, object *npc, char *txt);
-int talk_to_wall(object *npc, char *txt);
+int talk_to_wall(object *pl, object *npc, char *txt);
 object *find_mon_throw_ob(object *op);
 int can_detect_enemy(object *op, object *enemy, rv_vector *rv);
 int stand_in_light(object *op);
@@ -478,10 +507,14 @@ void control_golem(object *op, int dir);
 int summon_golem(object *op, object *caster, int dir, object *spob);
 object *choose_cult_monster(object *pl, object *god, int summon_level);
 int summon_object(object *op, object *caster, object *spell_ob, int dir);
+object *get_real_owner(object *ob);
+int should_arena_attack(object *pet, object *owner, object *target);
 /* player.c */
 player *find_player(char *plname);
 player *find_player_partial_name(char *plname);
 void display_motd(object *op);
+void send_rules(object *op);
+void send_news(object *op);
 int playername_ok(char *cp);
 int add_player(NewSocket *ns);
 archetype *get_player_archetype(archetype *at);
@@ -506,7 +539,7 @@ int check_pick(object *op);
 object *find_arrow(object *op, char *type);
 object *find_better_arrow(object *op, object *target, char *type, int *better);
 object *pick_arrow_target(object *op, char *type, int dir);
-int fire_bow(object *op, object *part, object *arrow, int dir, int wc_mod, int sx, int sy);
+int fire_bow(object *op, object *part, object *arrow, int dir, int wc_mod, sint16 sx, sint16 sy);
 int player_fire_bow(object *op, int dir);
 void fire_misc_object(object *op, int dir);
 void fire(object *op, int dir);
@@ -536,9 +569,9 @@ void dragon_ability_gain(object *who, int atnr, int level);
 CommArray_s *find_plugin_command(char *cmd, object *op);
 void displayPluginsList(object *op);
 int findPlugin(char *id);
-void initPlugins(void);
-void removeOnePlugin(char *id);
-void initOnePlugin(char *pluginfile);
+int initPlugins(void);
+int initOnePlugin(char *pluginfile);
+int removeOnePlugin(char *id);
 CFParm *CFWLog(CFParm *PParm);
 CFParm *CFWNewInfoMap(CFParm *PParm);
 CFParm *CFWSpringTrap(CFParm *PParm);
@@ -600,6 +633,20 @@ CFParm *CFWObjectCreateClone(CFParm *PParm);
 CFParm *CFWSetVariable(CFParm *PParm);
 CFParm *CFWDecreaseObjectNR(CFParm *PParm);
 CFParm *CFWTeleportObject(CFParm *PParm);
+CFParm *CFWCheckTrigger(CFParm *PParm);
+CFParm *CFWOutOfMap(CFParm *PParm);
+CFParm *CFWQueryName(CFParm *PParm);
+CFParm *CFWQueryBaseName(CFParm *PParm);
+CFParm *CFWInsertObInOb(CFParm *PParm);
+CFParm *CFWGetSettings(CFParm *PParm);
+CFParm *CFWGetMapFlags(CFParm *PParm);
+CFParm *CFWReCmp(CFParm *PParm);
+CFParm *CFWStrdupLocal(CFParm *PParm);
+CFParm *CFWCreatePathname(CFParm *PParm);
+CFParm *CFWUpdateObSpeed(CFParm *PParm);
+CFParm *CFWPresentArchByName(CFParm *PParm);
+CFParm *CFWPresentArchNameInOb(CFParm *PParm);
+CFParm *CFWSetDirection(CFParm *PParm);
 CFParm *RegisterGlobalEvent(CFParm *PParm);
 CFParm *UnregisterGlobalEvent(CFParm *PParm);
 void GlobalEvent(CFParm *PParm);
@@ -624,9 +671,8 @@ uint64 query_cost(object *tmp, object *who, int flag);
 char *cost_string_from_value(uint64 cost);
 char *query_cost_string(object *tmp, object *who, int flag);
 uint64 query_money(object *op);
-int pay_for_amount(int to_pay, object *pl);
+int pay_for_amount(uint64 to_pay, object *pl);
 int pay_for_item(object *op, object *pl);
-uint64 pay_from_container(object *op, object *pouch, int to_pay);
 int get_payment(object *pl, object *op);
 void sell_item(object *op, object *pl);
 void shop_listing(object *op);
@@ -736,8 +782,8 @@ int SP_level_spellpoint_cost(object *caster, object *spell, int flags);
 int SP_level_dam_adjust(object *caster, object *spob);
 int SP_level_duration_adjust(object *caster, object *spob);
 int SP_level_range_adjust(object *caster, object *spob);
-object *check_spell_known(object *op, char *name);
-object *lookup_spell_by_name(object *op, char *spname);
+object *check_spell_known(object *op, const char *name);
+object *lookup_spell_by_name(object *op, const char *spname);
 int reflwall(mapstruct *m, int x, int y, object *sp_op);
 int cast_create_obj(object *op, object *caster, object *new_op, int dir);
 int ok_to_put_more(mapstruct *m, sint16 x, sint16 y, object *op, int immune_stop);
@@ -751,6 +797,7 @@ int summon_hostile_monsters(object *op, int n, char *monstername);
 void shuffle_attack(object *op, int change_face);
 void prayer_failure(object *op, int failure, int power);
 void spell_failure(object *op, int failure, int power, object *skill);
+int cast_party_spell(object *op, object *caster, int dir, object *spell_ob, char *stringarg);
 int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stringarg);
 void move_spell_effect(object *op);
 void check_spell_effect(object *op);
@@ -831,16 +878,16 @@ void init_rainfall(void);
 void init_weatheravoid(weather_avoids_t wa[]);
 void init_weather(void);
 void perform_weather(void);
-void weather_effect(char *filename);
+void weather_effect(const char *filename);
 object *avoid_weather(int *av, mapstruct *m, int x, int y, int *gs, int grow);
 void calculate_temperature(mapstruct *m, int wx, int wy);
-void let_it_snow(mapstruct *m, int wx, int wy, char *filename);
-void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename);
-void plant_a_garden(mapstruct *m, int wx, int wy, char *filename);
-void change_the_world(mapstruct *m, int wx, int wy, char *filename);
-void feather_map(mapstruct *m, int wx, int wy, char *filename);
+void let_it_snow(mapstruct *m, int wx, int wy);
+void singing_in_the_rain(mapstruct *m, int wx, int wy);
+void plant_a_garden(mapstruct *m, int wx, int wy);
+void change_the_world(mapstruct *m, int wx, int wy);
+void feather_map(mapstruct *m, int wx, int wy);
 int worldmap_to_weathermap(int x, int y, int *wx, int *wy, mapstruct *m);
-char *weathermap_to_worldmap_corner(int wx, int wy, int *x, int *y, int dir);
+const char *weathermap_to_worldmap_corner(int wx, int wy, int *x, int *y, int dir);
 int polar_distance(int x, int y, int equator);
 void update_humid(void);
 int humid_tile(int x, int y);
