@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_random_map_c =
- *   "$Id: random_map.c,v 1.35 2003/09/26 05:44:43 mwedel Exp $";
+ *   "$Id: random_map.c,v 1.38 2005/04/14 06:57:01 mwedel Exp $";
  */
 
 /*
@@ -61,8 +61,13 @@ mapstruct *generate_random_map(char *OutFileName, RMParms *RP) {
 
     write_map_parameters_to_string(buf, RP);
 
-    if(RP->difficulty==0)
+    if(RP->difficulty==0) {
 	RP->difficulty = RP->dungeon_level; /* use this instead of a map difficulty  */
+	if (RP->difficulty_increase > 0.001) {
+	    RP->difficulty = (int)((float)RP->dungeon_level * RP->difficulty_increase);
+	    if (RP->difficulty < 1) RP->difficulty=1;
+	}
+    }
     else
 	RP->difficulty_given=1;
 
@@ -115,8 +120,7 @@ mapstruct *generate_random_map(char *OutFileName, RMParms *RP) {
     put_decor(theMap,layout,RP->decorstyle,RP->decoroptions,RP);
 
     /* generate treasures, etc. */
-    if(RP->generate_treasure_now)
-	fix_auto_apply(theMap);
+    fix_auto_apply(theMap);
 
     unblock_exits(theMap,layout,RP);
 
@@ -564,7 +568,12 @@ void write_map_parameters_to_string(char *buf,RMParms *RP) {
     sprintf(small_buf,"final_map %s\n",RP->final_map);
     strcat(buf,small_buf);
   }
-
+  
+  if(RP->exit_on_final_map[0]) { 
+    sprintf(small_buf,"exit_on_final_map %s\n",RP->exit_on_final_map);
+    strcat(buf,small_buf);
+  }
+  
   if(RP->this_map[0]) {
     sprintf(small_buf,"origin_map %s\n",RP->this_map);
     strcat(buf,small_buf);
@@ -603,6 +612,11 @@ void write_map_parameters_to_string(char *buf,RMParms *RP) {
     strcat(buf,small_buf);
   }
 
+  if(RP->difficulty_increase != 1.0 ) {
+    sprintf(small_buf,"difficulty_increase %f\n",RP->difficulty_increase);
+    strcat(buf,small_buf);
+  }
+
   sprintf(small_buf,"dungeon_level %d\n",RP->dungeon_level);
   strcat(buf,small_buf);
 
@@ -656,6 +670,7 @@ void write_parameters_to_string(char *buf,
                                 char *doorstyle_n,
                                 char *exitstyle_n,
                                 char *final_map_n,
+                                char *exit_on_final_map_n,
                                 char *this_map_n,
                                 int layoutoptions1_n,
                                 int layoutoptions2_n,
@@ -670,7 +685,8 @@ void write_parameters_to_string(char *buf,
                                 int origin_x_n,
                                 int origin_y_n,
                                 int random_seed_n,
-                                int treasureoptions_n ) 
+                                int treasureoptions_n,
+				float difficulty_increase) 
 {
 
   char small_buf[256];
@@ -721,6 +737,11 @@ void write_parameters_to_string(char *buf,
     strcat(buf,small_buf);
   }
 
+  if(exit_on_final_map_n && exit_on_final_map_n[0]) {
+    sprintf(small_buf,"exit_on_final_map %s\n",exit_on_final_map_n);
+    strcat(buf,small_buf);
+  }
+  
   if(this_map_n && this_map_n[0]) {
     sprintf(small_buf,"origin_map %s\n",this_map_n);
     strcat(buf,small_buf);
@@ -754,6 +775,11 @@ void write_parameters_to_string(char *buf,
     strcat(buf,small_buf);
   }
 
+  if(difficulty_increase > 0.001 ) {
+    sprintf(small_buf,"difficulty_increase %f\n",difficulty_increase);
+    strcat(buf,small_buf);
+  }
+
   sprintf(small_buf,"dungeon_level %d\n",dungeon_level_n);
   strcat(buf,small_buf);
 
