@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: random_map.h,v 1.10 2003/03/07 07:33:13 mwedel Exp $";
+ *   "$Id: random_map.h,v 1.14 2005/07/20 08:38:11 akirschbaum Exp $";
  */
 
 /*
@@ -42,7 +42,8 @@ typedef struct {
   char final_map[512];
   char exitstyle[512];
   char this_map[512];
-
+  char exit_on_final_map[512];
+	
   int Xsize;
   int Ysize;
   int expand2x;
@@ -52,6 +53,7 @@ typedef struct {
   int symmetry;
   int difficulty;
   int difficulty_given;
+  float difficulty_increase;
   int dungeon_level;
   int dungeon_depth;
   int decoroptions;
@@ -63,7 +65,6 @@ typedef struct {
   int map_layout_style;
   int treasureoptions;
   int symmetry_used;
-  int generate_treasure_now;
 } RMParms;
 
 int load_parameters(FILE *fp, int bufstate,RMParms *RP);
@@ -118,4 +119,6 @@ the numerical values matter so don't change them. */
     from 0 to x, centered at x/2 */
 #define BC_RANDOM(x) ((int) ((RANDOM() % (x)+RANDOM()%(x)+RANDOM()%(x))/3.))
 
+int set_random_map_variable(RMParms *rp,char *buf);
+
 #endif
