@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_standalone_c =
- *   "$Id: standalone.c,v 1.11 2003/09/03 12:25:06 tchize Exp $";
+ *   "$Id: standalone.c,v 1.14 2005/07/17 11:00:05 ryo_saeba Exp $";
  */
 
 /*
@@ -60,7 +60,6 @@ int main(int argc, char *argv[]) {
 
   init_gods();
   memset(&rp, 0, sizeof(RMParms));
-  rp.generate_treasure_now=1;
   rp.Xsize=-1;
   rp.Ysize=-1;
   if ((fp=fopen(InFileName, "r"))==NULL) {
@@ -165,3 +164,83 @@ void fix_auto_apply(mapstruct *m) {
             && (tmp->type == TRIGGER_BUTTON || tmp->type == TRIGGER_PEDESTAL))
           check_trigger(tmp,tmp->above);
 }
+
+/**
+ * Those are dummy functions defined to resolve all symboles.
+ * Added as part of glue cleaning.
+ * Ryo 2005-07-15
+ **/
+ 
+void new_draw_info(int a, int b, object *ob, const char *txt)
+    {
+    fprintf(logfile, "%s\n", txt);
+    }
+void new_info_map(int color, mapstruct *map, const char *str)
+    {
+    fprintf(logfile, "new_info_map: %s\n", str);
+    }
+ 
+void move_teleporter( object* ob)
+    {
+    }
+ 
+void move_firewall( object* ob)
+    {
+    }
+ 
+void move_duplicator( object* ob)
+    {
+    }
+ 
+void move_marker( object* ob)
+    {
+    }
+ 
+void move_creator( object* ob)
+    {
+    }
+ 
+void emergency_save( int x )
+    {
+    }
+ 
+void clean_tmp_files( )
+    {
+    }
+ 
+void esrv_send_item( object* ob, object* obx )
+    {
+    }
+ 
+void dragon_ability_gain( object* ob, int x, int y )
+    {
+    }
+ 
+void weather_effect( const char* c )
+    {
+    }
+ 
+void set_darkness_map( mapstruct* m)
+    {
+    }
+ 
+void move_apply( object* ob, object* obt, object* obx )
+    {
+    }
+ 
+object* find_skill_by_number( object* ob, int x )
+    {
+    return NULL;
+    }
+ 
+void esrv_del_item(player *pl, int tag)
+    {
+    }
+ 
+void monster_check_apply( object* ob, object* obt )
+    {
+    }
+ 
+void trap_adjust( object* ob, int x )
+    {
+    }
