@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c,v 1.20 2003/09/13 05:02:07 mwedel Exp $";
+ *   "$Id: alchemy.c,v 1.23 2005/07/16 21:10:39 akirschbaum Exp $";
  */
 
 /*
@@ -36,12 +36,13 @@
 #include <skills.h>
 #include <spells.h>
 
-/* define this for some helpful debuging information */
+/** define this for some helpful debuging information */
 #define ALCHEMY_DEBUG
 
-/* define this for loads of (marginal) debuging information */
+/** define this for loads of (marginal) debuging information */
 #define EXTREME_ALCHEMY_DEBUG
 
+/** Random cauldrons effects */
 static char *cauldron_effect [] = { 
  "vibrates briefly",
  "produces a cloud of steam",
@@ -60,15 +61,18 @@ static char *cauldron_effect [] = {
 }; 
 
 
-/* cauldron_sound() - returns a random selection from cauldron_effect[] */
+static int is_defined_recipe(const recipe *rp, const object *cauldron, object *caster);
 
+
+/** Returns a random selection from cauldron_effect[] */
 char * cauldron_sound ( void ) {
   int size=sizeof(cauldron_effect)/sizeof(char *);
 
   return cauldron_effect[rndm(0, size-1)]; 
 }
 
-/* attempt_do_alchemy() - Main part of the ALCHEMY code. From this we call fctns
+/**
+ * Main part of the ALCHEMY code. From this we call fctns
  * that take a look at the contents of the 'cauldron' and, using these ingredients, 
  * we construct an integer formula value which is referenced (randomly) against a 
  * formula list (the formula list chosen is based on the # contents of the cauldron).
@@ -127,7 +131,7 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 			rp->arch_name, formula/rp->index);
 #endif
 		attempt_recipe(caster, cauldron, ability, rp, formula/rp->index);
-	    } else LOG(llevDebug, "WIZ couldnt find formula for ingredients.\n"); 
+	    } else LOG(llevDebug, "WIZ couldn't find formula for ingredients.\n"); 
 	    return;
 	} /* End of WIZ alchemy */
 
@@ -136,6 +140,11 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 	    ;
 
 	if (rp) { /* if we found a recipe */
+	    uint64 value_ingredients;
+	    uint64 value_item;
+	    object *tmp;
+	    int attempt_shadow_alchemy;
+
 	    ave_chance = fl->total_chance/(float)fl->number;
 	    /* the caster gets an increase in ability based on thier skill lvl */
 	    if (rp->skill != NULL) {
@@ -155,6 +164,13 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 		return;
 	    }
 
+	    /* determine value of ingredients */
+	    value_ingredients = 0;
+	    for(tmp = cauldron->inv; tmp != NULL; tmp = tmp->below)
+		value_ingredients += query_cost(tmp, NULL, F_TRUE);
+
+	    attempt_shadow_alchemy = !is_defined_recipe(rp, cauldron, caster);
+
 	    /* create the object **FIRST**, then decide whether to keep it.	*/
 	    if ((item=attempt_recipe(caster, cauldron, ability, rp, formula/rp->index)) != NULL) {
 		/*  compute base chance of recipe success */
@@ -168,8 +184,18 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 		    success_chance, ability, rp->diff, item->level);
 #endif
 
+		value_item = query_cost(item, NULL, F_TRUE|F_IDENTIFIED|F_NOT_CURSED);
+		if(attempt_shadow_alchemy && value_item > value_ingredients) {
+#ifdef ALCHEMY_DEBUG
+#ifndef WIN32
+		    LOG(llevDebug, "Forcing failure for shadow alchemy recipe because price of ingredients (%llu) is less than price of result (%llu).\n", value_ingredients, value_item);
+#else
+		    LOG(llevDebug, "Forcing failure for shadow alchemy recipe because price of ingredients (%I64d) is less than price of result (%I64d).\n", value_ingredients, value_item);
+#endif
+#endif
+		}
 		/* roll the dice */
-		if ((float)(random_roll(0, 101, caster, PREFER_LOW)) <= 100.0 * success_chance) {
+		else if ((float)(random_roll(0, 101, caster, PREFER_LOW)) <= 100.0 * success_chance) {
 		    change_exp(caster, rp->exp, rp->skill, SK_EXP_NONE);
 		    return;
 		}
@@ -181,7 +207,8 @@ void attempt_do_alchemy(object *caster, object *cauldron) {
 	calc_alch_danger(caster, cauldron, rp));
 }
 
-/* content_recipe_value()- recipe value of the entire contents of a container.
+/**
+ * Recipe value of the entire contents of a container.
  * This appears to just generate a hash value, which I guess for now works
  * ok, but the possibility of duplicate hashes is certainly possible - msw
   */
@@ -210,7 +237,9 @@ int content_recipe_value (object *op) {
     return formula;
 }
 
-/* numb_ob_inside() */
+/**
+ * Returns total number of items in op
+ */
 
 int numb_ob_inside (object *op) {
   object *tmp=op->inv;
@@ -228,7 +257,8 @@ int numb_ob_inside (object *op) {
     return o_number;
 }
  
-/* attempt_recipe() - essentially a wrapper for make_item_from_recipe() and 
+/**
+ * Essentially a wrapper for make_item_from_recipe() and 
  * insert_ob_in_ob. If the caster has some alchemy skill, then they might
  * gain some exp from (successfull) fabrication of the product. 
  * If nbatches==-1, don't give exp for this creation (random generation/
@@ -293,8 +323,10 @@ object * attempt_recipe(object *caster, object *cauldron, int ability, recipe *r
 
 
 
-/* adjust_product() - we adjust the nrof, exp and level of the final product, based
- * on the item's default parameters, and the relevant caster skill level. */
+/**
+ * We adjust the nrof, exp and level of the final product, based
+ * on the item's default parameters, and the relevant caster skill level.
+ */
 
 void adjust_product(object *item, int lvl, int yield) {
     int nrof=1;
@@ -313,8 +345,10 @@ void adjust_product(object *item, int lvl, int yield) {
 }
 
 
-/* make_item_from_recipe()- using a list of items and a recipe to make an artifact. */
- 
+/**
+ * Using a list of items and a recipe to make an artifact.
+ */
+
 object * make_item_from_recipe(object *cauldron, recipe *rp) {
   artifact *art=NULL;
   object *item=NULL;
@@ -345,7 +379,8 @@ object * make_item_from_recipe(object *cauldron, recipe *rp) {
 }
 
 
-/* find_transmution_ob() - looks through the ingredient list, if we find a
+/**
+ * Looks through the ingredient list. If we find a
  * suitable object in it - we will use that to make the requested artifact.
  * Otherwise the code returns a 'generic' item. -b.t.
  */
@@ -374,7 +409,8 @@ object * find_transmution_ob ( object *first_ingred, recipe *rp) {
 }
 
 
-/* alchemy_failure_effect - Ouch. We didnt get the formula we wanted.
+/**
+ * Ouch. We didnt get the formula we wanted.
  * This fctn simulates the backfire effects--worse effects as the level
  * increases. If SPELL_FAILURE_EFFECTS is defined some really evil things
  * can happen to the would be alchemist. This table probably needs some
@@ -569,7 +605,8 @@ void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int danger) {
 }
 
 
-/* remove_contents() - all but object "save_item" are elimentated from
+/**
+ * All but object "save_item" are elimentated from
  * the container list. Note we have to becareful to remove the inventories
  * of objects in the cauldron inventory (ex icecube has stuff in it).  
  */
@@ -590,7 +627,8 @@ void remove_contents (object *first_ob, object *save_item) {
     }
 }
 
-/* calc_alch_danger() - "Danger" level will determine how bad the backfire
+/**
+ *"Danger" level, will determine how bad the backfire
  * could be if the user fails to concoct a recipe properly. Factors include
  * the number of ingredients, the length of the name of each ingredient,
  * the user's effective level, the user's Int and the enchantment on the
@@ -640,4 +678,85 @@ int calc_alch_danger(object *caster,object *cauldron, recipe *rp) {
    return danger;
 }
 
+/**
+ * Determines if ingredients in a container match the
+ * proper ingredients for a recipe.
+ *
+ * rp is the recipe to check
+ * cauldron is the container that holds the ingredients
+ * returns 1 if the ingredients match the recipe, 0 if not
+ *
+ * This functions tries to find each defined ingredient in the container. It is
+ * the defined recipe iff
+ *  - the number of ingredients of the recipe and in the container is equal
+ *  - all ingredients of the recipe are found in the container
+ *  - the number of batches is the same for all ingredients
+ */
+static int is_defined_recipe(const recipe *rp, const object *cauldron, object *caster)
+{
+    uint32 batches_in_cauldron;
+    const linked_char *ingredient;
+    int number;
+    const object *ob;
 
+    /* check for matching number of ingredients */
+    number = 0;
+    for(ingredient = rp->ingred; ingredient != NULL; ingredient = ingredient->next)
+	number++;
+    for(ob = cauldron->inv; ob != NULL; ob = ob->below)
+	number--;
+    if(number != 0)
+	return 0;
+
+    /* check for matching ingredients */
+    batches_in_cauldron = 0;
+    for(ingredient = rp->ingred; ingredient != NULL; ingredient = ingredient->next) {
+	uint32 nrof;
+	const char *name;
+	int ok;
+
+	/* determine and remove nrof from name */
+	name = ingredient->name;
+	nrof = 0;
+	while(isdigit(*name)) {
+	    nrof = 10*nrof+(*name-'0');
+	    name++;
+	}
+	if(nrof == 0)
+	    nrof = 1;
+	while(*name == ' ')
+	    name++;
+
+	/* find the current ingredient in the cauldron */
+	ok = 0;
+	for(ob = cauldron->inv; ob != NULL; ob = ob->below) {
+	    char name_ob[MAX_BUF];
+	    const char *name2;
+
+	    if(ob->title == NULL)
+		name2 = ob->name;
+	    else {
+		snprintf(name_ob, sizeof(name_ob), "%s %s", ob->name, ob->title);
+		name2 = name_ob;
+	    }
+
+	    if(strcmp(name2, name) == 0) {
+		if(ob->nrof%nrof == 0) {
+		    uint32 batches;
+
+		    batches = ob->nrof/nrof;
+		    if(batches_in_cauldron == 0) {
+			batches_in_cauldron = batches;
+			ok = 1;
+		    } else if(batches_in_cauldron == batches)
+			ok = 1;
+		}
+		break;
+	    }
+	}
+	if(!ok)
+	    return(0);
+    }
+
+    return(1);
+}
