@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_attack_c =
- *   "$Id: attack.c,v 1.97 2004/03/24 08:15:17 mwedel Exp $";
+ *   "$Id: attack.c,v 1.108 2005/07/26 00:23:00 akirschbaum Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -48,6 +48,9 @@ typedef struct att_msg_str {
 void cancellation(object *op)
 {
     object *tmp;
+
+    if (op->invisible)
+	return;
    
     if (QUERY_FLAG (op, FLAG_ALIVE) || op->type == CONTAINER  || op->type == THROWN_OBJ) {
 	/* Recur through the inventory */
@@ -192,7 +195,6 @@ void save_throw_object (object *op, int type, object *originator)
 
 		if (tmp) {
 		    esrv_del_item(tmp->contr, op->count);
-		    esrv_update_item(UPD_WEIGHT, tmp, tmp);
 		}
 	    }
 	    if ( ! QUERY_FLAG (op, FLAG_REMOVED))
@@ -259,7 +261,7 @@ int hit_map(object *op,int dir,int type) {
     }
 
     if ( ! op->map) {
-	LOG (llevError,"BUG: hit_map(): %s has no map", op->name);
+	LOG (llevError,"BUG: hit_map(): %s has no map\n", op->name);
 	return 0;
     }
 
@@ -646,16 +648,17 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
     if ((evt = find_event(op, EVENT_ATTACK)) != NULL)
     {
         CFParm CFP;
-        int k, l, m;
+        int k, l;
+        uint32 n;
         k = EVENT_ATTACK;
         l = SCRIPT_FIX_ALL;
-        m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = hitter;
         CFP.Value[2] = hitter;
         CFP.Value[3] = op;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &base_dam;
         CFP.Value[7] = &base_wc;
         CFP.Value[8] = &l;
@@ -673,7 +676,8 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
             if ((evt = find_event(hitter->current_weapon, EVENT_ATTACK)) != NULL)
             {
                 CFParm CFP;
-                int k, l, n;
+                int k, l;
+                uint32 n;
                 n = 0;
                 k = EVENT_ATTACK;
                 l = SCRIPT_FIX_ALL;
@@ -897,16 +901,17 @@ object *hit_with_arrow (object *op, object *victim)
     {
         CFParm CFP;
         CFParm* CFR;
-        int k, l, m;
+        int k, l;
+        uint32 n;
         k = EVENT_ATTACK;
         l = SCRIPT_FIX_ALL;
-        m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = hitter;
         CFP.Value[2] = op;
         CFP.Value[3] = victim;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &(op->stats.dam);
         CFP.Value[7] = &(op->stats.wc);
         CFP.Value[8] = &l;
@@ -919,7 +924,6 @@ object *hit_with_arrow (object *op, object *victim)
         }
     }
     else
-
         hit_something = attack_ob_simple (victim, hitter, op->stats.dam,
                                         op->stats.wc);
 
@@ -927,7 +931,8 @@ object *hit_with_arrow (object *op, object *victim)
      * arrow, move_apply() calls this function, arrow sticks in demon,
      * attack_ob_simple() returns, and we've got an arrow that still exists
      * but is no longer on the map. Ugh. (Beware: Such things can happen at
-     * other places as well!) */
+     * other places as well!)
+     */
     if (was_destroyed (hitter, hitter_tag) || hitter->env != NULL) {
         if (container) {
             remove_ob (container);
@@ -938,7 +943,8 @@ object *hit_with_arrow (object *op, object *victim)
 
     /* Missile hit victim */
     /* if the speed is > 10, then this is a fast moving arrow, we go straight
-       through the target */
+     * through the target 
+     */
     if (hit_something && op->speed <= 10.0)
     {
         /* Stop arrow */
@@ -963,7 +969,7 @@ object *hit_with_arrow (object *op, object *victim)
             remove_ob (hitter);
             hitter->x = victim_x;
             hitter->y = victim_y;
-            insert_ob_in_map (hitter, hitter->map, hitter,0);
+            insert_ob_in_map (hitter, victim->map, hitter,0);
         } else {
             /* Else leave arrow where it is */
             merge_ob (hitter, NULL);
@@ -1156,6 +1162,8 @@ int hit_player_attacktype(object *op, object *hitter, int dam,
 	  {
 	    object *tmp;
 	    for(tmp=op->inv; tmp!=NULL; tmp=tmp->below) {
+		if(tmp->invisible)
+		  continue;
 		if(!QUERY_FLAG(tmp, FLAG_APPLIED) ||
 		   (tmp->resist[ATNR_ACID] >= 10))
 		  /* >= 10% acid res. on itmes will protect these */
@@ -1355,6 +1363,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
     char buf[MAX_BUF], *skill;
     int maxdam=0;
     int battleg=0;    /* true if op standing on battleground */
+    int pk=0;         /* true if op and what controls hitter are both players*/
     int killed_script_rtn = 0;
     object *owner=NULL;
     int evtid;
@@ -1370,15 +1379,17 @@ int kill_object(object *op,int dam, object *hitter, int type)
     {
         CFParm* CFR;
         int k, l, m;
+        uint32 n;
         k = EVENT_DEATH;
         l = SCRIPT_FIX_ALL;
         m = 0;
+        n = type;
         CFP.Value[0] = &k;
         CFP.Value[1] = hitter;
         CFP.Value[2] = op;
         CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &type;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
@@ -1436,6 +1447,9 @@ int kill_object(object *op,int dam, object *hitter, int type)
 
     /* is the victim (op) standing on battleground? */
     if (op_on_battleground(op, NULL, NULL)) battleg=1;
+    
+    /* is this player killing?*/
+    if (op->type == PLAYER && owner->type == PLAYER) pk=1;
 
     /* Player killed something */
     if(owner->type==PLAYER) {
@@ -1479,15 +1493,16 @@ int kill_object(object *op,int dam, object *hitter, int type)
 		play_sound_map(owner->map, owner->x, owner->y, SOUND_PLAYER_KILLS);
 	}
 
-	/* If a player kills another player with melee, not on
+	/* If a player kills another player, not on
 	 * battleground, the "killer" looses 1 luck. Since this is
 	 * not reversible, it's actually quite a pain IMHO. -AV 
 	 * Fix bug in that we were changing the luck of the hitter, not
 	 * player that the object belonged to - so if you killed another player
 	 * with spells, pets, whatever, there was no penalty.
+	 * Changed to make luck penalty configurable in settings.
 	 */
-	if(op->type == PLAYER && hitter != op && !battleg)
-	    change_luck(owner, -1);
+	if(op->type == PLAYER && owner != op && !battleg)
+	    change_luck(owner, -settings.pk_luck_penalty);
 
 	/* This code below deals with finding the appropriate skill
 	 * to credit exp to.  This is a bit problematic - we should
@@ -1521,12 +1536,12 @@ int kill_object(object *op,int dam, object *hitter, int type)
 
     /* Pet (or spell) killed something. */
     if(owner != hitter ) {
-	(void) sprintf(buf,"%s killed %s with %s%s.",owner->name,
-	       query_name(op),query_name(hitter), battleg? " (duel)":"");
+	(void) sprintf(buf,"%s killed %s with %s%s%s.",owner->name,
+	       query_name(op),query_name(hitter), battleg? " (duel)":"", pk? " (pk)":"");
     }
     else {
-	(void) sprintf(buf,"%s killed %s%s.",hitter->name,op->name,
-	       battleg? " (duel)":"");
+	(void) sprintf(buf,"%s killed %s in hand to hand combat %s%s.",hitter->name,op->name,
+	       battleg? " (duel)":"", pk? " (pk)":"");
     }
     /* These may have been set in the player code section above */
     if (!skop) skop = hitter->chosen_skill;
@@ -1536,7 +1551,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
 
 
     /* If you didn't kill yourself, and your not the wizard */
-    if(hitter!=op && !QUERY_FLAG(op, FLAG_WAS_WIZ)) {
+    if(owner!=op && !QUERY_FLAG(op, FLAG_WAS_WIZ)) {
 	int exp;
 
 	exp = calc_skill_exp(owner,op, skop);
@@ -1588,7 +1603,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
 		for(pl=first_player;pl!=NULL;pl=pl->next) {
 		    if(pl->ob->contr->party_number==no && on_same_map(pl->ob, owner)) {
 			nexp=(pl->ob->level+4)*share;
-			change_exp(pl->ob,nexp, skop->skill, SK_EXP_TOTAL);
+			change_exp(pl->ob,nexp, skill, SK_EXP_TOTAL);
 			given+=nexp;
 		    }
 		}
@@ -1989,7 +2004,7 @@ void confuse_player(object *op, object *hitter, int dam)
     
     tmp = present_in_ob_by_name(FORCE,"confusion", op);
     if(!tmp) {
-	tmp = get_archetype("force");
+	tmp = get_archetype(FORCE_NAME);
 	tmp = insert_ob_in_ob(tmp,op);
     }
     
