@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_chat_c =
- *   "$Id: c_chat.c,v 1.17 2003/11/07 19:54:49 ryo_saeba Exp $";
+ *   "$Id: c_chat.c,v 1.19 2004/12/25 18:09:29 ryo_saeba Exp $";
  */
 
 /*
@@ -97,31 +97,46 @@ int command_orcknuckle(object *op, char *params)
     return 0;
 }
 
-int command_shout (object *op, char *params)
+static int command_tell_all(object *op, char *params, int pri, int color, char *desc)
 {
     int evtid;
     CFParm CFP;
-	if (op->contr->no_shout == 1){
-		new_draw_info(NDI_UNIQUE, 0,op,"You are no longer allowed to shout.");
-		return 1;
-	}else{
-	
-    if (params == NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Shout what?");
+
+    if (op->contr->no_shout == 1){
+	new_draw_info(NDI_UNIQUE, 0,op,"You are no longer allowed to shout or chat.");
+	return 1;
+    } else {
+	if (params == NULL) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Shout/Chat what?");
+	    return 1;
+	}
+	new_draw_info_format(NDI_UNIQUE | NDI_ALL | color, pri, NULL, 
+		 "%s %s: %s", op->name, desc, params);
+ 
+	/* GROS : Here we handle the SHOUT global event */
+	evtid = EVENT_SHOUT;
+	CFP.Value[0] = (void *)(&evtid);
+	CFP.Value[1] = (void *)(op);
+	CFP.Value[2] = (void *)(params);
+	/* Pass in priority - not sure if the plugin can use it or not */
+	CFP.Value[3] = (void *)(&pri);
+	GlobalEvent(&CFP);
 	return 1;
     }
-    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_RED, 1, NULL, 
-		 "%s shouts: %s", op->name, params);
-    /* GROS : Here we handle the SHOUT global event */
-    evtid = EVENT_SHOUT;
-    CFP.Value[0] = (void *)(&evtid);
-    CFP.Value[1] = (void *)(op);
-    CFP.Value[2] = (void *)(params);
-    GlobalEvent(&CFP);
-    return 1;
-	}
 }
 
+int command_shout (object *op, char *params)
+{
+    return command_tell_all(op, params, 1, NDI_RED, "shouts");
+}
+
+int command_chat (object *op, char *params)
+{
+    return command_tell_all(op, params, 9, NDI_BLUE, "chats");
+}
+
+
+
 int command_tell (object *op, char *params)
 {
     char buf[MAX_BUF],*name = NULL ,*msg = NULL;
@@ -419,12 +434,14 @@ static int basic_emote(object *op, char *params, int emotion)
 	    break;
 	} /*case*/
 	new_info_map_except(NDI_WHITE, op->map, op, buf);
-	new_draw_info(NDI_UNIQUE, 0, op, buf2);
+	new_draw_info(NDI_UNIQUE|NDI_WHITE, 0, op, buf2);
 	return(0);
     } else {
         for(pl=first_player;pl!=NULL;pl=pl->next) {
-	    if(strncasecmp(pl->ob->name, params, MAX_NAME)==0 &&
-	       pl->ob->map == op->map && pl->ob != op) {
+    	  if(strncasecmp(pl->ob->name, params, MAX_NAME)==0 &&
+            pl->ob->map == op->map && pl->ob != op &&
+            !(QUERY_FLAG(pl->ob,FLAG_WIZ) && pl->ob->contr->hidden)) {
+            /* Hidden dms are not affected by emotions*/
 		switch(emotion) {
 		case EMOTE_NOD:
 		    sprintf(buf, "You nod solemnly to %s.", pl->ob->name);
@@ -611,7 +628,7 @@ static int basic_emote(object *op, char *params, int emotion)
 			  op->name);
 		  break;
 		} /*case*/
-		new_draw_info(NDI_UNIQUE, 0, op, buf);
+		new_draw_info(NDI_UNIQUE|NDI_WHITE, 0, op, buf);
 		new_draw_info(NDI_UNIQUE|NDI_WHITE, 0, pl->ob, buf2);
 		new_info_map_except2(NDI_WHITE, op->map, op, pl->ob, buf3);
 		return(0);
@@ -725,7 +742,7 @@ static int basic_emote(object *op, char *params, int emotion)
 		    sprintf(buf2, "You look away from %s.", op->name);
 		    break;
 		}/*case*/
-		new_draw_info(NDI_UNIQUE, 0, op, buf);
+		new_draw_info(NDI_UNIQUE|NDI_WHITE, 0, op, buf);
 		new_info_map_except(NDI_WHITE, op->map, op, buf2);
 		return(0);
 	    }/*if self*/
