@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_misc_c =
- *   "$Id: c_misc.c,v 1.39 2004/03/26 21:59:28 ryo_saeba Exp $";
+ *   "$Id: c_misc.c,v 1.63 2005/07/16 21:10:39 akirschbaum Exp $";
  */
 
 /*
@@ -198,6 +198,25 @@ void malloc_info(object *op) {
   new_draw_info(NDI_UNIQUE, 0,op,errmsg);
 }
 
+/* 
+ * Pretty much identical to current map_info, but on a bigger scale 
+ * This function returns the name of the players current region, and
+ * a description of it. It is there merely for flavour text.
+ */
+void current_region_info(object *op) {
+    /* 
+     * Ok I /suppose/ I should write a seperate function for this, but it isn't
+     * going to be /that/ slow, and won't get called much
+     */
+    region *r = get_region_by_name(get_name_of_region_for_map(op->map));
+
+    if (!r)
+	return;
+	/* This should only be possible if regions are not operating on this server. */
+
+    new_draw_info_format(NDI_UNIQUE, 0,op,   
+	"You are in %s. \n %s", get_region_longname(r), get_region_msg(r));
+}
 
 void current_map_info(object *op) {
     mapstruct *m = op->map;
@@ -206,7 +225,7 @@ void current_map_info(object *op) {
 	return;
 
     new_draw_info_format(NDI_UNIQUE, 0,op,   
-	"%s (%s)", m->name, m->path);
+	"%s (%s) in %s", m->name, m->path, get_name_of_region_for_map(m));
 
     if (QUERY_FLAG(op,FLAG_WIZ)) {
 	new_draw_info_format(NDI_UNIQUE, 0, op,
@@ -231,43 +250,227 @@ int command_malloc_verify(object *op, char *parms)
 	else
 		new_draw_info(NDI_UNIQUE, 0,op,"Heap checks out OK.");
 	return 1;
-}
+	}
 #endif
 
-int command_who (object *op, char *params)
-{
+int command_whereabouts(object *op, char *params) {
+
+    region *reg;
     player *pl;
-    char buf[MAX_BUF];
+    
+    /*
+     * reset the counter on the region, then use it to store the number of
+     * players there.
+     * I don't know how thread-safe this would be, I suspect not very....
+     */
+    for (reg=first_region;reg!=NULL;reg=reg->next) {
+        reg->counter=0;
+    }
+    for (pl=first_player;pl!=NULL;pl=pl->next)
+        if (pl->ob->map!=NULL)
+            get_region_by_map(pl->ob->map)->counter++;
 
-    if (first_player != (player *) NULL)
-	new_draw_info(NDI_UNIQUE, 0,op,"Players:");
+     /* we only want to print out by places with a 'longname' field...*/
+    for (reg=first_region;reg!=NULL;reg=reg->next) {
+	if (reg->longname==NULL && reg->counter>0) {
+	    if(reg->parent !=NULL) {
+		reg->parent->counter+=reg->counter;
+		reg->counter=0;
+	    }
+	    else /*uh oh, we shouldn't be here. */
+		LOG(llevError,"command_whereabouts() Region %s with no longname has no parent", reg->name);
+	}
+    }
+	new_draw_info_format(NDI_UNIQUE, 0,op,
+	"In the world currently there are:");
+	for (reg=first_region;reg!=NULL;reg=reg->next)
+	    if(reg->counter>0)
+		new_draw_info_format(NDI_UNIQUE, 0,op,
+		    "%u players in %s", reg->counter, get_region_longname(reg));
+    return 1;
+}
 
-    for(pl=first_player;pl!=NULL;pl=pl->next) {
-	if(pl->ob->map == NULL)
+typedef struct
+    {
+      char namebuf[MAX_BUF];
+      int login_order;
+    } chars_names;
+
+/*local functon for qsort comparison*/
+static int name_cmp (const chars_names *c1, const chars_names *c2)
+    {
+      return strcasecmp (c1->namebuf, c2->namebuf);
+    }
+
+int command_who (object *op, char *params) {
+    player *pl;
+    uint16 i;
+    region *reg;
+    char* format;
+    int num_players = 0;
+    int num_wiz = 0;
+    int num_afk = 0;
+    chars_names *chars = NULL;
+    
+    /* 
+     * The who formats are defined in config to be blank. They should have been
+     * overridden by the settings file, if there are no entries however, it will
+     * have stayed blank. Since this probably isn't what is wanted, we will check if
+     * new formats have been specified, and if not we will use the old defaults.
+     */
+    if (!strcmp(settings.who_format,"")) 
+    	strcpy(settings.who_format, "%N_%T%t%h%d%n[%m]");
+    if (!strcmp(settings.who_wiz_format,"")) 
+    	strcpy(settings.who_wiz_format, "%N_%T%t%h%d%nLevel %l [%m](@%i)(%c)");
+    if (op == NULL || QUERY_FLAG(op, FLAG_WIZ))
+    	format=settings.who_wiz_format;
+    else
+    	format=settings.who_format;
+
+    reg=get_region_from_string(params);
+    
+    for (pl=first_player;pl!=NULL;pl=pl->next) {
+	if (pl->ob->map == NULL)
 	    continue;
 	if (pl->hidden && !QUERY_FLAG(op, FLAG_WIZ)) continue;
+	
+	if(!region_is_child_of_region(get_region_by_map(pl->ob->map),reg)) continue;
 
 	if (pl->state==ST_PLAYING || pl->state==ST_GET_PARTY_PASSWORD) {
 
-	    if(op == NULL || QUERY_FLAG(op, FLAG_WIZ))
-		(void) sprintf(buf,"%s the level %d %s (@%s) [%s]%s%s%s (%d)",
-		       pl->ob->name,
-		       pl->ob->level,
-		       (pl->own_title[0]=='\0'?pl->title:pl->own_title),
-		       pl->socket.host,
-		       pl->ob->map->path,
-		       QUERY_FLAG(pl->ob,FLAG_WIZ)?" [WIZ]":"",
-		       pl->hidden?"[HID]":"",
-		       pl->peaceful?"Peaceful":"Hostile",pl->ob->count);
-	    else
-		(void) sprintf(buf,"%s the %s [%s]%s%s",pl->ob->name,
-		       (pl->own_title[0]=='\0'?pl->title:pl->own_title),
-		       pl->ob->map->path,
-		       QUERY_FLAG(pl->ob,FLAG_WIZ)?" [WIZ]":"",
-		       pl->peaceful?"Peaceful":"Hostile");
-	    new_draw_info(NDI_UNIQUE, 0,op,buf);
+	    num_players++;
+	    chars = (chars_names *) realloc(chars, num_players*sizeof(chars_names));
+            if (chars == NULL) {
+	        new_draw_info(NDI_UNIQUE, 0, op, "who failed - out of memory!");
+                return 0;
+            } 
+	    sprintf(chars[num_players-1].namebuf, "%s", pl->ob->name);
+	    chars[num_players-1].login_order = num_players;
+	    /*Check for WIZ's & AFK's*/
+	    if (QUERY_FLAG(pl->ob,FLAG_WIZ))
+	      num_wiz++;
+	    if (QUERY_FLAG(pl->ob,FLAG_AFK))
+	      num_afk++;
 	}
     }
+    if (first_player != (player *) NULL) {
+    	if (reg == NULL) 
+            new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players (%d) -- WIZ(%d) AFK(%d)", 
+      	    	num_players, num_wiz, num_afk);
+	else if (reg->longname == NULL)
+	    new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d)", 
+      	    	reg->name, num_players, num_wiz, num_afk);
+	else
+	    new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d)", 
+      	    	reg->longname, num_players, num_wiz, num_afk);
+    }
+    qsort (chars, num_players, sizeof(chars_names), name_cmp);
+    for (i=0;i<num_players;i++)
+	display_who_entry(op, find_player(chars[i].namebuf), format);
+    free(chars);    
+    return 1;
+}
+
+/* Display a line of 'who' to op, about pl, using the formatting specified by format */
+void display_who_entry(object *op, player *pl, char *format) {
+    char tmpbuf[MAX_BUF];
+    char outbuf[MAX_BUF];
+    sint16 i;
+    outbuf[0]='\0'; /* we strcat to this, so reset it here. */
+    if (pl==NULL) {
+    	LOG(llevError,"display_who_entry(): I was passed a null player");
+	return;
+    }
+    for (i=0;i<=strlen(format);i++) {
+        if (format[i]=='%') {
+            i++;
+            get_who_escape_code_value(tmpbuf,format[i],pl);
+            strcat(outbuf, tmpbuf);
+        }
+        else if (format[i]=='_')
+            strcat(outbuf," "); /* allow '_' to be used in place of spaces */
+        else {
+            sprintf(tmpbuf,"%c",format[i]);
+            strcat(outbuf,tmpbuf);
+        }
+    }
+    new_draw_info(NDI_UNIQUE, 0, op, outbuf);
+}
+
+/* Returns the value of the escape code used in the who format specifier
+ * the values are:
+ * N	Name of character
+ * t	title of character
+ * T    the optional "the " sequence value (depend if player has own_title or not)
+ * c	count
+ * n	newline
+ * h	[Hostile] if character is hostile, nothing otherwise
+ * d	[WIZ] if character is a dm, nothing otherwise
+ * a	[AFK] if character is afk, nothing otherwise
+ * l	the level of the character
+ * m	the map path the character is currently on
+ * M	the map name of the map the character is currently on
+ * r	the region name (eg scorn, wolfsburg)
+ * R	the regional title (eg The Kingdom of Scorn, The Port of Wolfsburg)
+ * i	player's ip adress
+ * %	a literal %
+ * _	a literal underscore
+ */
+  
+void get_who_escape_code_value(char *return_val, const char letter, player *pl) {
+    
+    switch (letter) {
+	case 'N' :    strcpy(return_val, pl->ob->name);
+		          break;
+	case 't' :    strcpy(return_val,(pl->own_title[0]=='\0'?pl->title:pl->own_title));
+			  break;
+    case 'T' :    if (pl->own_title[0]=='\0')
+                      strcpy(return_val,"the ");
+                  else
+                      *return_val='\0';
+              break;
+	case 'c' :    sprintf(return_val,"%d",pl->ob->count);
+			  break;
+	case 'n' :    strcpy(return_val, "\n"); 
+		          break;
+	case 'h' :    strcpy(return_val,pl->peaceful?"":" [Hostile]");
+			  break;
+	case 'l' :    sprintf(return_val,"%d",pl->ob->level);
+			  break;
+	case 'd' :    strcpy(return_val,(QUERY_FLAG(pl->ob,FLAG_WIZ)?" [WIZ]":""));
+			  break;
+	case 'a' :    strcpy(return_val,(QUERY_FLAG(pl->ob,FLAG_AFK)?" [AFK]":""));
+			  break;
+	case 'm' :    strcpy(return_val,pl->ob->map->path);
+		 	  break;
+	case 'M' :    strcpy(return_val,pl->ob->map->name);
+			  break;		
+	case 'r' :    strcpy(return_val,get_name_of_region_for_map(pl->ob->map));
+			  break;
+	case 'R' :    strcpy(return_val,get_region_longname(get_region_by_map(pl->ob->map)));
+			  break;
+	case 'i' :    strcpy(return_val,pl->socket.host);
+			  break;
+	case '%' :    strcpy(return_val, "%"); 
+		          break;
+	case '_' :    strcpy(return_val, "_"); 
+		          break;
+    }
+
+}
+
+
+int command_afk (object *op, char *params)
+{
+    if QUERY_FLAG(op,FLAG_AFK) { 
+       CLEAR_FLAG(op,FLAG_AFK);
+       new_draw_info(NDI_UNIQUE, 0, op, "You are no longer AFK");
+    }    
+    else 
+    {
+       SET_FLAG(op,FLAG_AFK);
+       new_draw_info(NDI_UNIQUE, 0, op, "You are now AFK");
+    }
     return 1;
 }
 
@@ -283,6 +486,12 @@ int command_mapinfo (object *op, char *params)
     return 1;
   }
 
+int command_whereami (object *op, char *params)
+{
+    current_region_info(op);
+    return 1;
+}
+
  int command_maps (object *op, char *params)
 {
     map_info(op,params);
@@ -451,6 +660,8 @@ int command_dumpbelow (object *op, char *params)
   if (op && op->below) {
 	  dump_object(op->below);
 	  new_draw_info(NDI_UNIQUE, 0,op,errmsg);
+      /* Let's push that item on the dm's stack */
+      dm_stack_push( op->contr, op->below->count );
       }
   return 0;
 }
@@ -631,13 +842,36 @@ int command_statistics(object *pl, char *params)
     new_draw_info_format(NDI_UNIQUE, 0, pl, "Cha         %2d/ %3d/%3d",
 	pl->contr->orig_stats.Cha, pl->stats.Cha, 20+pl->arch->clone.stats.Cha);
     new_draw_info_format(NDI_UNIQUE, 0, pl, "\nAttack Mode: %s",pl->contr->peaceful? "Peaceful":"Hostile");
-	
+
+    /* If dragon player, let's display natural resistances */
+    if ( is_dragon_pl( pl ) )
+        {
+        int attack;
+        object* tmp;
+        for ( tmp = pl->inv; tmp != NULL; tmp = tmp->below )
+            {
+            if ( ( tmp->type == FORCE ) && ( strcmp( tmp->arch->name, "dragon_skin_force" )== 0 ) )
+                {
+                new_draw_info( NDI_UNIQUE, 0, pl, "\nNatural skin resistances:" );
+                for ( attack = 0; attack < NROFATTACKS; attack++ )
+                    {
+                    if ( atnr_is_dragon_enabled( attack ) )
+                        {
+                        new_draw_info_format( NDI_UNIQUE, 0, pl, "%s: %d", change_resist_msg[ attack ], tmp->resist[ attack ] );
+                        }
+                    }
+                break;
+                }
+            }
+        }
+
    /* Can't think of anything else to print right now */
    return 0;
 }
 
 int command_fix_me(object *op, char *params)
 {
+    sum_weight(op);
     fix_player(op);
     return 1;
 }
@@ -662,7 +896,10 @@ int command_players(object *op, char *paramss)
 
 		strcpy(t,Entry->d_name);
 		if (stat(buf,&Stat)==0) {
-		    if ((Stat.st_mode & S_IFMT)==S_IFDIR) {
+		    /* This was not posix compatible
+             * if ((Stat.st_mode & S_IFMT)==S_IFDIR) {
+             */
+            if (S_ISDIR(Stat.st_mode)){
 			char buf2[MAX_BUF];
 			struct tm *tm=localtime(&Stat.st_mtime);
 			sprintf(buf2,"%s\t%04d %02d %02d %02d %02d %02d",
@@ -781,7 +1018,7 @@ int command_bowmode(object *op, char *params)
 int command_petmode(object *op, char *params)
 {
     petmode_t oldtype=op->contr->petmode;
-    static char *types[]={"normal", "sad", "defend"};
+    static char *types[]={"normal", "sad", "defend", "arena"};
 
     if (!params) {
 	new_draw_info_format(NDI_UNIQUE, 0, op, "petmode is set to %s",
@@ -795,10 +1032,12 @@ int command_petmode(object *op, char *params)
 	op->contr->petmode=pet_sad;
     else if (!strcmp(params,"defend")) 
 	op->contr->petmode=pet_defend;
+    else if (!strcmp(params,"arena")) 
+	op->contr->petmode=pet_arena;
     else {
 	new_draw_info_format(NDI_UNIQUE, 0, op,
 	    "petmode: Unknown options %s, valid options are normal,"
-	    "sad (seek and destroy), defend", params);
+	    "sad (seek and destroy), defend, arena", params);
 	return 0;
     }
     new_draw_info_format(NDI_UNIQUE, 0, op, "petmode %s set to %s",
@@ -1044,7 +1283,7 @@ int command_help (object *op, char *params)
    * No_help -escape
    */
  nohelp:
-  sprintf(line, "No help availble on '%s'", params);
+  sprintf(line, "No help available on '%s'", params);
   new_draw_info(NDI_UNIQUE, 0,op, line);
   return 0;
 }
@@ -1310,3 +1549,83 @@ int command_kill_pets(object *op, char *params)
     new_draw_info(NDI_UNIQUE, 0, op, "Your pets have been killed.");
     return 0;
 }
+
+/**
+ * Displays all non start/end tags for specified quest.
+ **/
+static void display_quest_details( object* pl, object* quest )
+    {
+    object* item;
+
+    new_draw_info_format( NDI_WHITE, 0, pl, "Quest: %s\n%s\n",
+        quest_get_name( quest ),
+        quest->lore ? quest->lore : "(no description available)" );
+
+    for ( item = pl->inv; item; item = item->below )
+        {
+        if ( ( item != quest )
+            && ( quest_is_quest_marker( item ) )
+            && ( quest_is_same_quest( quest->slaying, item->slaying ) )
+            && ( item->lore ) )
+            {
+            new_draw_info( NDI_WHITE, 0, pl, item->lore );
+            new_draw_info( NDI_WHITE, 0, pl, "------\n" );
+            }
+        }
+    }
+
+/**
+ * Displays quest informations to player.
+ * Acceptable parameters:
+ *  * finished => finished quests only
+ *  * <name> => only this particular quest, finished or not, with details
+ *  * nothing => all current quests
+ *
+ * For current quests, will display either the lore of the non start tags,
+ *  or the lore of start tag if no other tag.
+ **/
+int command_quests( object *pl, char *params )
+    {
+    object* item;
+
+    if ( params && !strcmp( params, "finished" ) )
+        {
+        new_draw_info( NDI_WHITE, 0, pl, "Completed quests:\n" );
+        for ( item = pl->inv; item; item = item->below )
+            {
+            if ( quest_is_quest_marker( item ) && quest_is_end( item->slaying ) )
+                {
+                new_draw_info( NDI_WHITE, 0, pl, quest_get_name( item ) );
+                new_draw_info( NDI_WHITE, 0, pl, "\n" );
+                }
+            }
+        return 1;
+        }
+
+    if ( params )
+        {
+        for ( item = pl->inv; item; item = item->below )
+            {
+            if ( quest_is_quest_marker( item )
+                && strstr( quest_get_name( item ), params ) )
+                {
+                display_quest_details( pl, item );
+                }
+            }
+        return 1;
+        }
+
+    /* Display current quests */
+    new_draw_info( NDI_WHITE, 0, pl, "Current quests:\n" );
+    for ( item = pl->inv; item; item = item->below )
+        {
+        if ( quest_is_quest_marker( item )
+            && quest_is_start( item->slaying ) )
+            {
+            new_draw_info( NDI_WHITE, 0, pl, quest_get_name( item ) );
+            new_draw_info( NDI_WHITE, 0, pl, "\n" );
+            }
+        }
+
+    return 1;
+    }
