@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_new_c =
- *   "$Id: c_new.c,v 1.7 2003/10/21 17:16:29 ryo_saeba Exp $";
+ *   "$Id: c_new.c,v 1.8 2005/01/16 11:31:38 tchize Exp $";
  */
 
 /*
@@ -70,6 +70,14 @@ int execute_newserver_command(object *pl, char *command)
 
     pl->contr->has_hit=0;
 
+    /*
+     * remove trailing spaces from commant
+     */
+    cp=command+strlen(command)-1;
+    while ( (cp>=command) && (*cp==' ')){
+        *cp='\0';
+        cp--;
+    }
     cp=strchr(command, ' ');
     if (cp) {
 	*(cp++) ='\0';
