@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.40 2003/09/16 04:58:07 temitchell Exp $";
+ *   "$Id: commands.c,v 1.51 2005/07/14 16:21:34 ryo_saeba Exp $";
  */
 
 /*
@@ -67,6 +67,7 @@ CommArray_s Commands[] = {
   {"cast", command_cast,	0.2},	/* Is this right? */
   {"disarm", command_disarm,	1.0},
   {"dm", command_dm,		0.0},
+  {"dmhide", command_dmhide,		0.0}, /* Like dm, but don't tell a dm arrived, hide player */
   {"drop", command_drop,	1.0},
   {"dropall", command_dropall,	1.0},
   {"examine", command_examine,	0.5},
@@ -98,6 +99,7 @@ CommArray_s Commands[] = {
   {"shutdown", command_shutdown, 0.0},
   {"skills", command_skills,	0.0},	/* shows player list of skills */
   {"use_skill", command_uskill, 1.0},
+  {"quests", command_quests, 0.0},
   {"ready_skill", command_rskill, 1.0},
   {"search",command_search,	1.0},
   {"search-items", command_search_items,	0.0},
@@ -107,6 +109,8 @@ CommArray_s Commands[] = {
   {"throw", command_throw,	1.0},
   {"time", command_time,	0.0},
   {"weather", command_weather,	0.0},
+  {"whereabouts", command_whereabouts, 0.0},
+  {"whereami", command_whereami, 0.0},
   {"title", command_title,	0.0},
   {"usekeys", command_usekeys,	0.0},
   {"bowmode", command_bowmode, 0.0},
@@ -114,6 +118,7 @@ CommArray_s Commands[] = {
   {"version", command_version,	0.0},
   {"wimpy", command_wimpy,	0.0},
   {"who", command_who,		0.0},
+  {"afk", command_afk,          0.0},
 
   {"stay", command_stay,	1.0}, /* 1.0 because it is used when using a
 				       *  skill on yourself */
@@ -135,6 +140,7 @@ CommArray_s CommunicationCommands [] = {
   {"reply", command_reply,		0.0},
   {"say", command_say,			0.1},
   {"shout", command_shout,		0.1},
+  {"chat", command_chat,		0.1},
   {"me", command_me,                   0.1},
   {"nod", command_nod,			0.0},
   {"dance", command_dance,		0.0},
@@ -215,6 +221,7 @@ CommArray_s WizCommands [] = {
   {"banish", command_banish,0.0},
   {"create", command_create,0.0},
   {"debug", command_debug,0.0},
+  {"diff", command_diff, 0.0 },
   {"dump", command_dump,0.0},
   {"dumpbelow", command_dumpbelow,0.0},
   {"dumpfriendlyobjects", command_dumpfriendlyobjects,0.0},
@@ -240,8 +247,12 @@ CommArray_s WizCommands [] = {
   {"remove", command_remove,0.0},
   {"reset", command_reset,0.0},
   {"set_god", command_setgod, 0.0},
-  {"speed", command_speed,0.0},
+  {"server_speed", command_speed,0.0},
   {"ssdumptable", command_ssdumptable,0.0},
+  {"stack_clear", command_stack_clear, 0.0 },
+  {"stack_list", command_stack_list, 0.0},
+  {"stack_pop", command_stack_pop, 0.0 },
+  {"stack_push", command_stack_push, 0.0 },
   {"stats", command_stats,0.0},
   {"style_info", command_style_map_info, 0.0},	/* Costly command, so make it wiz only */
   {"summon", command_summon,0.0},
@@ -249,7 +260,7 @@ CommArray_s WizCommands [] = {
   {"toggle_shout", command_toggle_shout,0.0},
   {"wizpass", command_wizpass,0.0},
   {"overlay_save", command_save_overlay, 0.0},
-  {"possess", command_possess, 0.0},
+/*  {"possess", command_possess, 0.0}, */
   {"mon_aggr", command_mon_aggr, 0.0},
   {"loadtest", command_loadtest, 0.0},
 };
@@ -276,6 +287,7 @@ const int Socket_CommandsSize =sizeof(Socket_Commands) / sizeof(CommArray_s);
  */
 CommArray_s Socket2_Commands[] = {
   {"shout", command_shout,	0.1},
+  {"chat", command_chat,	0.1},
   {"tell", command_tell,	0.1},
 };
 
@@ -399,7 +411,14 @@ int parse_string(object *op, char *str)
 #ifdef INPUT_DEBUG
     LOG(llevDebug, "Command: '%s'\n", str);
 #endif
-
+    /*
+     * remove trailing spaces
+     */
+    cp = str+strlen(str)-1;
+    while ( (cp>=str) && (*cp==' ')){
+        *cp='\0';
+	cp--;
+    }
     /*
      * No arguments?
      */
@@ -451,7 +470,7 @@ int parse_command(object *op, char *str) {
   int i;
   /* if it's a keybinding command, ignore semicolons */
   if(strstr(str,"bind")) return parse_string(op,str);
-
+printf("parsin command '%s'\n",str);
   /* If on a socket, you can not do complex commands. */
   if(op && (tmp=strchr(str,';'))!=NULL)  /* we've found a ';' do the 1st and recurse */
 	 {
