@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_gods_c =
- *   "$Id: gods.c,v 1.42 2004/04/18 09:32:30 ryo_saeba Exp $";
+ *   "$Id: gods.c,v 1.47 2005/08/10 13:38:31 ryo_saeba Exp $";
  */
 
 /*
@@ -76,8 +76,8 @@ object *find_god(char *name) {
 
 /**
  * Determines if op worships a god.
- * Returns the godname if they do. In the case of an NPC, if they have
- * no god, we give them a random one. -b.t.
+ * Returns the godname if they do or "none" if they have no god.
+ * In the case of an NPC, if they have no god, we give them a random one. -b.t.
  */
 
 char *determine_god(object *op) {
@@ -232,15 +232,17 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
         CFParm CFP;
         CFParm* CFR;
         int k, l, m;
+        uint32 n;
         k = EVENT_APPLY;
         l = SCRIPT_FIX_ALL;
         m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = pl;
         CFP.Value[2] = altar;
         CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
@@ -641,6 +643,8 @@ static int god_removes_curse (object *op, int remove_damnation)
     int success = 0;
 
     for (tmp = op->inv; tmp; tmp = tmp->below) {
+	if (tmp->invisible)
+	    continue;
         if (QUERY_FLAG (tmp, FLAG_DAMNED) && ! remove_damnation)
             continue;
         if (QUERY_FLAG (tmp, FLAG_CURSED) || QUERY_FLAG (tmp, FLAG_DAMNED)) {
@@ -886,7 +890,7 @@ void god_intervention (object *op, object *god, object *skill)
             archetype *at;
             int i;
 
-            if ((at = find_archetype("depletion")) == NULL) {
+            if ((at = find_archetype(ARCH_DEPLETION)) == NULL) {
                 LOG (llevError, "Could not find archetype depletion.\n");
                 continue;
             }
@@ -1054,6 +1058,12 @@ int tailor_god_spell(object *spellop, object *caster) {
 
     if (caster->type==SPELL_EFFECT || caster->type == SPELL) caster_is_spell=1;
 
+    /* if caster is a rune or the like, it doesn't worship anything.  However,
+     * if this object is owned by someone, then the god that they worship
+     * is relevant, so use that.
+     */
+    if (!god && get_owner(caster)) god=find_god(determine_god(get_owner(caster)));
+
     if ( ! god || (spellop->attacktype & AT_HOLYWORD && !god->race)) {
         if ( ! caster_is_spell)
             new_draw_info(NDI_UNIQUE, 0, caster,
