@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.54 2004/02/04 07:46:12 tanner Exp $";
+ *   "$Id: init.c,v 1.65 2005/07/16 21:10:39 akirschbaum Exp $";
  */
 
 /*
@@ -57,6 +57,7 @@ void set_confdir(char *path) { settings.confdir=path; }
 void set_localdir(char *path) { settings.localdir=path; }
 void set_mapdir(char *path) { settings.mapdir=path; }
 void set_archetypes(char *path) { settings.archetypes=path; }
+void set_regions(char *path) { settings.regions=path; }
 void set_treasures(char *path) { settings.treasures=path; }
 void set_uniquedir(char *path) { settings.uniquedir=path; }
 void set_playerdir(char *path) { settings.playerdir=path; }
@@ -115,12 +116,20 @@ struct Command_Line_Options options[] = {
 {"-local",1,1, set_localdir},
 {"-maps", 1, 1, set_mapdir},
 {"-arch", 1, 1, set_archetypes},
+{"-regions", 1, 1, set_regions},
 {"-playerdir", 1, 1, set_playerdir},
 {"-treasures", 1, 1, set_treasures},
 {"-uniquedir", 1, 1, set_uniquedir},
 {"-tmpdir", 1, 1, set_tmpdir},
 {"-log", 1, 1, set_logfile},
 
+#ifdef WIN32
+/* Windows service stuff */
+{"-regsrv", 0, 1, service_register},
+{"-unregsrv", 0, 1, service_unregister},
+{"-srv", 0, 1, service_handle},
+#endif
+
 /* Pass 2 functions.  Most of these could probably be in pass 1, 
  * as they don't require much of anything to bet set up.
  */
@@ -354,7 +363,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp,"off") || !strcasecmp(cp,"false")) {
 		settings.meta_on=FALSE;
 	    } else {
-		LOG(llevError,"load_settings: Unkown value for metaserver_notification: %s\n",
+		LOG(llevError,"load_settings: Unknown value for metaserver_notification: %s\n",
 		    cp);
 	    }
 	} else if (!strcasecmp(buf,"metaserver_server")) {
@@ -388,7 +397,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 0)
-		LOG(llevError, "load_settings: worldmapstartx must be at least"
+		LOG(llevError, "load_settings: worldmapstartx must be at least "
 		    "0, %d is invalid\n", size);
 	    else
 		settings.worldmapstartx = size;
@@ -396,7 +405,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 0)
-		LOG(llevError, "load_settings: worldmapstarty must be at least"
+		LOG(llevError, "load_settings: worldmapstarty must be at least "
 		    "0, %d is invalid\n", size);
 	    else
 		settings.worldmapstarty = size;
@@ -404,7 +413,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 1)
-		LOG(llevError, "load_settings: worldmaptilesx must be greater"
+		LOG(llevError, "load_settings: worldmaptilesx must be greater "
 		    "than 1, %d is invalid\n", size);
 	    else
 		settings.worldmaptilesx = size;
@@ -412,7 +421,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 1)
-		LOG(llevError, "load_settings: worldmaptilesy must be greater"
+		LOG(llevError, "load_settings: worldmaptilesy must be greater "
 		    "than 1, %d is invalid\n", size);
 	    else
 		settings.worldmaptilesy = size;
@@ -420,7 +429,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 1)
-		LOG(llevError, "load_settings: worldmaptilesizex must be"
+		LOG(llevError, "load_settings: worldmaptilesizex must be "
 		    "greater than 1, %d is invalid\n", size);
 	    else
 		settings.worldmaptilesizex = size;
@@ -428,7 +437,7 @@ static void load_settings()
 	    int size = atoi(cp);
 
 	    if (size < 1)
-		LOG(llevError, "load_settings: worldmaptilesizey must be"
+		LOG(llevError, "load_settings: worldmaptilesizey must be "
 		    "greater than 1, %d is invalid\n", size);
 	    else
 		settings.worldmaptilesizey = size;
@@ -436,7 +445,7 @@ static void load_settings()
 	    int lev = atoi(cp);
 
 	    if (lev < 0)
-		LOG(llevError, "load_settings: dynamiclevel must be"
+		LOG(llevError, "load_settings: dynamiclevel must be "
 		    "at least 0, %d is invalid\n", lev);
 	    else
 		settings.dynamiclevel = lev;
@@ -454,7 +463,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.not_permadeth=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for not_permadeth"
+		LOG(llevError, "load_settings: Unknown value for not_permadeth"
 		    ": %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "resurrection")) {
@@ -463,7 +472,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.resurrection=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for resurrection"
+		LOG(llevError, "load_settings: Unknown value for resurrection"
 		    ": %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "set_title")) {
@@ -472,7 +481,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.set_title=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for set_title"
+		LOG(llevError, "load_settings: Unknown value for set_title"
 		    ": %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "search_items")) {
@@ -481,7 +490,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.search_items=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for search_items"
+		LOG(llevError, "load_settings: Unknown value for search_items"
 		    ": %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "spell_encumbrance")) {
@@ -490,7 +499,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.spell_encumbrance=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for "
+		LOG(llevError, "load_settings: Unknown value for "
 		    "spell_encumbrance: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "spell_failure_effects")) {
@@ -499,7 +508,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.spell_failure_effects=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "spell_failure_effects: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "casting_time")) {
@@ -508,7 +517,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.casting_time=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "casting_time: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "real_wiz")) {
@@ -517,7 +526,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.real_wiz=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "real_wiz: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "recycle_tmp_maps")) {
@@ -526,7 +535,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.recycle_tmp_maps=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "recycle_tmp_maps: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "explore_mode")) {
@@ -535,16 +544,22 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.explore_mode=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "explore_mode: %s\n", cp);
 	    }
+	} else if (!strcasecmp(buf,"who_format")) {
+	    if (has_val)
+	    	strcpy(settings.who_format, cp);
+	} else if (!strcasecmp(buf,"who_wiz_format")) {
+	    if (has_val)
+	    	strcpy(settings.who_wiz_format, cp);      
 	} else if (!strcasecmp(buf, "spellpoint_level_depend")) {
 	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
 		settings.spellpoint_level_depend=TRUE;
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.spellpoint_level_depend=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "spellpoint_level_depend: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "stat_loss_on_death")) {
@@ -553,7 +568,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.stat_loss_on_death=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "stat_loss_on_death: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "use_permanent_experience")) {
@@ -562,7 +577,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.use_permanent_experience=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "use_permanent_experience: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "balanced_stat_loss")) {
@@ -571,7 +586,7 @@ static void load_settings()
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
 		settings.balanced_stat_loss=FALSE;
 	    } else {
-		LOG(llevError, "load_settings: Unkown value for"
+		LOG(llevError, "load_settings: Unknown value for "
 		    "balanced_stat_loss: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf,"simple_exp")) {
@@ -580,16 +595,24 @@ static void load_settings()
 	    } else if (!strcasecmp(cp,"off") || !strcasecmp(cp,"false")) {
 		settings.simple_exp=FALSE;
 	    } else {
-		LOG(llevError,"load_settings: Unkown value for simple_exp: %s\n",
+		LOG(llevError,"load_settings: Unknown value for simple_exp: %s\n",
 		    cp);
 	    }
 	} else if (!strcasecmp(buf, "item_power_factor")) {
 	    float tmp = atof(cp);
 	    if (tmp < 0)
-		LOG(llevError, "load_settings: item_power_factor must be a postive number (%f < 0)\n",
+		LOG(llevError, "load_settings: item_power_factor must be a positive number (%f < 0)\n",
 		    tmp);
 	    else
 		settings.item_power_factor = tmp;
+	} else if (!strcasecmp(buf, "pk_luck_penalty")) {
+	    sint16 val = atoi(cp);
+
+	    if (val < -100 || val >100)
+		LOG(llevError, "load_settings: pk_luck_penalty must be between -100 and 100"
+		    ", %d is invalid\n", val);
+	    else
+		settings.pk_luck_penalty = val;
 	} else if (!strcasecmp(buf, "set_friendly_fire")) {
 	    int val = atoi(cp);
 
@@ -598,7 +621,43 @@ static void load_settings()
 		    ", %d is invalid\n", val);
 	    else
 		settings.set_friendly_fire = val;	
-	} else {
+    } else if ( !strcasecmp( buf, "armor_max_enchant" ) ) {
+        int max_e = atoi( cp );
+        if ( max_e <= 0 )
+            LOG( llevError, "load_settings: armor_max_enchant is %d\n", max_e );
+        else
+            settings.armor_max_enchant = max_e;
+    } else if ( !strcasecmp( buf, "armor_weight_reduction" ) ) {
+        int wr = atoi( cp );
+        if ( wr < 0 )
+            LOG( llevError, "load_settings: armor_weight_reduction is %d\n", wr );
+        else
+            settings.armor_weight_reduction = wr;
+    } else if ( !strcasecmp( buf, "armor_weight_linear" ) ) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.armor_weight_linear=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.armor_weight_linear=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: unknown value for armor_weight_linear: %s\n", cp);
+        }
+
+    } else if ( !strcasecmp( buf, "armor_speed_improvement" ) ) {
+        int wr = atoi( cp );
+        if ( wr < 0 )
+            LOG( llevError, "load_settings: armor_speed_improvement is %d\n", wr );
+        else
+            settings.armor_speed_improvement = wr;
+    } else if ( !strcasecmp( buf, "armor_speed_linear" ) ) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.armor_speed_linear = TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.armor_speed_linear = FALSE;
+	    } else {
+		LOG(llevError, "load_settings: unknown value for armor_speed_linear: %s\n", cp);
+        }
+
+    } else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
     }
@@ -635,7 +694,6 @@ void init(int argc, char **argv) {
 
     init_startup();	/* Write (C), check shutdown/forbid files */
     init_signals();	/* Sets up signal interceptions */
-    setup_library();	/* Set up callback function pointers */
     init_commands();	/* Sort command tables */
     read_map_log();	/* Load up the old temp map files */
     init_skills();
@@ -681,13 +739,14 @@ void help() {
     printf(" -local      Read/write local data (hiscore, unique items, etc.)\n");
     printf(" -maps       Sets the directory for maps.\n");
     printf(" -arch       Sets the archetype file to use.\n");
+    printf(" -regions    Sets the regions file to use.\n");
     printf(" -playerdir  Sets the directory for the player files.\n");
     printf(" -treasures	 Sets the treasures file to use.\n");
     printf(" -uniquedir  Sets the unique items/maps directory.\n");
     printf(" -tmpdir     Sets the directory for temporary files (mostly maps.)\n");
     printf(" -m          Lists out suggested experience for all monsters.\n");
     printf(" -m2         Dumps out abilities.\n");
-    printf(" -m3         Dumps out artifactt information.\n");
+    printf(" -m3         Dumps out artifact information.\n");
     printf(" -m4         Dumps out spell information.\n");
     printf(" -m5         Dumps out skill information.\n");
     printf(" -m6         Dumps out race information.\n");
@@ -702,6 +761,7 @@ void init_beforeplay() {
   init_archetypes(); /* If not called before, reads all archetypes from file */
   init_artifacts();  /* If not called before, reads all artifacts from file */
   init_spells();     /* If not called before, links archtypes used by spells */
+  init_regions();    /* If not called before, reads all regions from file */
   init_archetype_pointers(); /* Setup global pointers to archetypes */
   init_races();	   /* overwrite race designations using entries in lib/races file */ 
   init_gods();	/* init linked list of gods from archs*/ 
@@ -825,7 +885,7 @@ void compile_info() {
   exit(0);
 #else
   execl("/bin/uname", "uname", "-a", NULL);
-  LOG(llevError, "Opps, should't have gotten here.");
+  LOG(llevError, "Oops, shouldn't have gotten here.\n");
   perror("execl");
   exit(-1);
 #endif
@@ -866,6 +926,7 @@ void rec_sigpipe(int i) {
  * be looked at later on, and maybe fix the problem that caused it to
  * dump core.  There is no reason that SIGPIPES should be fatal
  */
+  LOG(llevError,"\nSIGPIPE--------------\n------------\n--------\n---\n");
 #if 1 && !defined(WIN32) /* ***win32: we don't want send SIGPIPE */
   LOG(llevInfo,"\nReceived SIGPIPE, ignoring...\n");
   signal(SIGPIPE,rec_sigpipe);/* hocky-pux clears signal handlers */
@@ -904,6 +965,7 @@ void init_signals() {
 #ifndef DEBUG
   signal(SIGQUIT,rec_sigquit);
   signal(SIGSEGV,rec_sigsegv);
+  LOG(llevInfo,"\n---------registering SIGPIPE\n");
   signal(SIGPIPE,rec_sigpipe);
 #ifdef SIGBUS
   signal(SIGBUS,rec_sigbus);
@@ -913,36 +975,6 @@ void init_signals() {
 #endif /* win32 */
 }
 
-/*
- * init_library: Set up the function pointers which will point
- * back from the library into the server.
- */
-void setup_library() {
-  set_emergency_save(emergency_save);
-  set_clean_tmp_files(clean_tmp_files);
-  set_fix_auto_apply(fix_auto_apply);
-  set_remove_friendly_object(remove_friendly_object);
-  set_process_active_maps(process_active_maps);
-  set_update_buttons(update_buttons);
-  set_draw_info(new_draw_info);
-  set_move_apply(move_apply);
-  set_monster_check_apply(monster_check_apply);
-  set_move_teleporter(move_teleporter);
-  set_move_firewall(move_firewall);
-  set_move_creator(move_creator);
-  set_move_trigger_marker(move_marker);
-  set_move_duplicator(move_duplicator);
-  set_trap_adjust(trap_adjust);
-  set_esrv_send_item(esrv_send_item);
-  set_esrv_del_item(esrv_del_item);
-  set_esrv_update_item(esrv_update_item);
-  set_info_map(new_info_map);
-  set_set_darkness_map(set_darkness_map);
-  set_dragon_gain_func(dragon_ability_gain);
-  set_weather_effect_func(weather_effect);
-  set_find_skill_by_number_func(find_skill_by_number);
-}
-
 /* init_races() - reads the races file in the lib/ directory, then
  * overwrites old 'race' entries. This routine allow us to quickly
  * re-configure the 'alignment' of monsters, objects. Useful for
