@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c,v 1.46 2004/03/12 19:04:15 ryo_saeba Exp $";
+ *   "$Id: login.c,v 1.50 2005/07/14 09:18:55 akirschbaum Exp $";
  */
 
 /*
@@ -159,6 +159,11 @@ int check_name(player *me,char *name) {
 	    return 0;
 	}
 
+    if (*name=='\0') {
+	new_draw_info(NDI_UNIQUE, 0,me->ob,"Null names are not allowed.");
+	return 0;
+    }
+
     if(!playername_ok(name)) {
 	new_draw_info(NDI_UNIQUE, 0,me->ob,"That name contains illegal characters.");
 	return 0;
@@ -180,7 +185,7 @@ int create_savedir_if_needed(char *savedir)
     return 0;
   } else {
     stat(savedir, buf);
-    if ((buf->st_mode & S_IFDIR) == 0)
+    if (!S_ISDIR(buf->st_mode))
       if (mkdir(savedir, SAVE_DIR_MODE))
 	{
 	perror("Unable to create player savedir,");
@@ -442,7 +447,7 @@ void check_login(object *op) {
     } else {
 	elapsed_save_time = time(NULL) - statbuf.st_mtime;
 	if (elapsed_save_time<0) {
-	    LOG(llevError,"Player file %s was saved in the future? (%d time)\n", elapsed_save_time);
+	    LOG(llevError,"Player file %s was saved in the future? (%d time)\n", filename, elapsed_save_time);
 	    elapsed_save_time=0;
 	}
     }
@@ -611,7 +616,8 @@ void check_login(object *op) {
 
     CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
 
-    strncpy(pl->title, op->arch->clone.name,MAX_NAME);
+    strncpy(pl->title, op->arch->clone.name, sizeof(pl->title)-1);
+    pl->title[sizeof(pl->title)-1] = '\0';
     
     /* If the map where the person was last saved does not exist,
      * restart them on their home-savebed. This is good for when
