@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_main_c =
- *    "$Id: main.c,v 1.78 2004/04/03 08:05:25 mwedel Exp $";
+ *    "$Id: main.c,v 1.92 2005/07/20 08:38:11 akirschbaum Exp $";
  */
 
 /*
@@ -463,7 +463,7 @@ char *clean_path(char *file)
  * are getting passed a string that points to a unique map
  * path.
  */
-char *unclean_path(char *src)
+char *unclean_path(const char *src)
 {
     static char newpath[MAX_BUF],*cp;
 
@@ -498,7 +498,6 @@ static void enter_random_map(object *pl, object *exit_ob)
     if (exit_ob->msg) set_random_map_variable(&rp,exit_ob->msg);
     rp.origin_x = exit_ob->x;
     rp.origin_y = exit_ob->y;
-    rp.generate_treasure_now = 1;
     strcpy(rp.origin_map, pl->map->path);
 
     /* If we have a final_map, use it as a base name to give some clue
@@ -804,10 +803,24 @@ void process_players1(mapstruct *map)
     for(flag=1;flag!=0;) {
 	flag=0;
 	for(pl=first_player;pl!=NULL;pl=plnext) {
-
-
 	    plnext=pl->next; /* In case a player exits the game in handle_player() */
-	    if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
+
+	    if (pl->ob == NULL) continue;
+
+	    if (map!=NULL && pl->ob->map!=map) continue;
+
+            if(pl->ob->speed_left>0) {
+		if (handle_newcs_player(pl->ob))
+		    flag=1;
+	    } /* end if player has speed left */
+
+	    /* If the player is not actively playing, don't make a
+	     * backup save - nothing to save anyway.  Plus, the
+	     * map may not longer be valid.  This can happen when the
+	     * player quits - they exist for purposes of tracking on the map,
+	     * but don't actually reside on any actual map.
+	     */
+	    if (QUERY_FLAG(pl->ob, FLAG_REMOVED)) continue;
 
 #ifdef AUTOSAVE
 	    /* check for ST_PLAYING state so that we don't try to save off when
@@ -826,14 +839,6 @@ void process_players1(mapstruct *map)
 		}
 	    }
 #endif
-            if(pl->ob == NULL) {
-                /* I take it this should never happen,
-                   but it seems to anyway :( */
-                flag = 1;
-            } else if(pl->ob->speed_left>0) {
-		if (handle_newcs_player(pl->ob))
-		    flag=1;
-	    } /* end if player has speed left */
 	} /* end of for loop for all the players */
     } /* for flag */
     for(pl=first_player;pl!=NULL;pl=pl->next) {
@@ -929,6 +934,25 @@ void process_events (mapstruct *map)
 	    continue;
 	}
 
+	/* I've seen occasional crashes due to this - the object is removed,
+	 * and thus the map it points to (last map it was on) may be bogus
+	 * The real bug is to try to find out the cause of this - someone
+	 * is probably calling remove_ob without either an insert_ob or
+	 * free_object afterwards, leaving an object dangling.  But I'd
+	 * rather log this and continue on instead of crashing.
+	 * Don't remove players - when a player quits, the object is in
+         * sort of a limbo, of removed, but something we want to keep
+	 * around.
+	 */
+	if (QUERY_FLAG (op, FLAG_REMOVED) && op->type != PLAYER && 
+	    op->map && op->map->in_memory != MAP_IN_MEMORY) {
+	    LOG (llevError, "BUG: process_events(): Removed object on list\n");
+	    dump_object(op);
+	    LOG(llevError, errmsg);
+	    free_object(op);
+	    continue;
+	}
+
 	if ( ! op->speed) {
 	    LOG (llevError, "BUG: process_events(): Object %s has no speed, "
 		 "but is on active list\n", op->arch->name);
@@ -960,6 +984,20 @@ void process_events (mapstruct *map)
 	}
 
 	if (op->speed_left > 0) {
+#if 0
+	    /* I've seen occasional crashes in move_symptom() with it
+	     * crashing because op is removed - add some debugging to
+	     * track if it is removed at this point.
+	     * This unfortunately is a bit too verbose it seems - not sure
+	     * why - I think what happens is a map is freed or something and
+	     * some objects get 'lost' - removed never to be reclaimed.
+	     * removed objects generally shouldn't exist.
+	     */
+	    if (QUERY_FLAG(op, FLAG_REMOVED)) {
+		LOG(llevDebug,"process_events: calling process_object with removed object %s\n",
+		    op->name?op->name:"null");
+	    }
+#endif
 	    --op->speed_left;
 	    process_object (op);
 	    if (was_destroyed (op, tag))
@@ -1067,7 +1105,13 @@ void leave(player *pl, int draw_exit) {
 	}
 	pl->ob->type = DEAD_OBJECT; /* To avoid problems with inventory window */
     }
-  if (pl!=NULL && draw_exit) new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL, buf);
+    /* If a hidden dm dropped connection do not create
+    * inconsistencies by showing that they have left the game 
+    */
+    if (!(QUERY_FLAG(pl->ob,FLAG_WIZ) && pl->ob->contr->hidden) && 
+        (pl!=NULL && draw_exit) && 
+	   (pl->state != ST_GET_NAME && pl->state!=ST_GET_PASSWORD && pl->state != ST_CONFIRM_PASSWORD)) 
+	    new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL, buf);
 }
 
 int forbid_play()
@@ -1164,7 +1208,8 @@ int main(int argc, char **argv)
   int evtid;
   CFParm CFP;
 #ifdef WIN32 /* ---win32 this sets the win32 from 0d0a to 0a handling */
-	_fmode = _O_BINARY ;
+    _fmode = _O_BINARY ;
+    bRunning = 1;
 #endif
 
 #ifdef DEBUG_MALLOC_LEVEL
@@ -1175,7 +1220,12 @@ int main(int argc, char **argv)
   settings.argv=argv;
   init(argc, argv);
   initPlugins();        /* GROS - Init the Plugins */
+#ifdef WIN32
+  while ( bRunning )
+      {
+#else
   for(;;) {
+#endif 
     nroferrors = 0;
 
     doeric_server();
@@ -1190,5 +1240,7 @@ int main(int argc, char **argv)
 
     sleep_delta();	/* Slepp proper amount of time before next tick */
   }
+  emergency_save( 0 );
+  cleanup( );
   return 0;
 }
