@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c,v 1.69 2004/04/14 07:24:30 mwedel Exp $";
+ *    "$Id: monster.c,v 1.82 2005/07/28 07:23:19 qal21 Exp $";
  */
 
 /*
@@ -80,10 +80,13 @@ object *check_enemy(object *npc, rv_vector *rv) {
 	     QUERY_FLAG(npc->enemy, FLAG_NEUTRAL))
 		npc->enemy = NULL;
 
-	else if (QUERY_FLAG(npc, FLAG_FRIENDLY) && 
-		 (QUERY_FLAG(npc->enemy, FLAG_FRIENDLY) || 
-		  npc->enemy->type == PLAYER || npc->enemy == npc->owner))
-		    npc->enemy = NULL;
+  	else if (QUERY_FLAG(npc, FLAG_FRIENDLY) && (
+ 		 	(QUERY_FLAG(npc->enemy, FLAG_FRIENDLY) && !(should_arena_attack(npc, npc->owner, npc->enemy))) || 
+ 		 	((npc->enemy->type == PLAYER) && !(should_arena_attack(npc, npc->owner, npc->enemy))) 
+ 			 || npc->enemy == npc->owner))
+  		 npc->enemy = NULL;
+		    
+		    
 	else if (!QUERY_FLAG(npc, FLAG_FRIENDLY) && 
              (!QUERY_FLAG(npc->enemy, FLAG_FRIENDLY) && npc->enemy->type!=PLAYER))
                 npc->enemy=NULL;
@@ -143,7 +146,7 @@ object *find_nearest_living_creature(object *npc) {
                 tmp=tmp->above;
 
 	    if (!tmp) {
-		LOG(llevDebug,"find_nearest_living_creature: map %s (%d,%d) has is_alive set but did not find a mosnter?\n",
+		LOG(llevDebug,"find_nearest_living_creature: map %s (%d,%d) has is_alive set but did not find a monster?\n",
 		    m->path, nx, ny);
 	    }
 	    else {
@@ -770,11 +773,11 @@ int monster_cast_spell(object *head, object *part,object *pl,int dir, rv_vector
 	    return 0;
 	}
 	if (spell_item->type == SPELLBOOK) {
+	    if (!spell_item->inv) {
+		LOG(llevError,"spellbook %s does not contain a spell?\n", spell_item->name);
+		return 0;
+	    }
 	    spell_item=spell_item->inv;
-	    if (spell_item->inv) {
-		LOG(llevError,"spellbook %s does contain a spell?", spell_item->name);
-	    } else
-		spell_item=spell_item->inv;
 	}
     }
     else
@@ -894,8 +897,10 @@ int monster_use_skill(object *head, object *part, object *pl,int dir) {
 
 /* Monster will use a ranged spell attack. */
 
-int monster_use_range(object *head,object *part,object *pl,int dir) {
+int monster_use_range(object *head,object *part,object *pl,int dir)
+    {
     object *wand, *owner;
+    int at_least_one = 0;
 
     if(!(dir=path_to_player(part,pl,0)))
 	return 0;
@@ -909,49 +914,55 @@ int monster_use_range(object *head,object *part,object *pl,int dir) {
 	dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
 
     for(wand=head->inv;wand!=NULL;wand=wand->below)
-	if(QUERY_FLAG(wand,FLAG_APPLIED) &&
-	   (wand->type == WAND || wand->type == ROD || wand->type==HORN))
-		break;
+        {
+        if (wand->type == WAND)
+            {
+            /* Found a wand, let's see if it has charges left */
+            at_least_one = 1;
+	        if( wand->stats.food<=0 )
+                continue;
 
-    if(wand==NULL) {
-	LOG(llevError,"Error: Monster %s (%d) HAS_READY_RANG() without range.\n",
+            cast_spell( head, wand, dir, wand->inv, NULL );
+
+            if ( !( --wand->stats.food ) )
+                {
+		        if ( wand->arch )
+                    {
+		            CLEAR_FLAG(wand, FLAG_ANIMATE);
+		            wand->face = wand->arch->clone.face;
+		            wand->speed = 0;
+		            update_ob_speed(wand);
+		            }
+                }
+            /* Success */
+            return 1;
+            }
+        else if ( wand->type == ROD || wand->type==HORN ) {
+            /* Found rod/horn, let's use it if possible */
+	    at_least_one = 1;
+	    if( wand->stats.hp < MAX( wand->inv->stats.sp, wand->inv->stats.grace ) )
+                continue;
+
+	    /* drain charge before casting spell - can be a case where the
+	     * spell destroys the monster, and rod, so if done after, results
+	     * in crash.
+	     */
+	    drain_rod_charge( wand );
+            cast_spell( head, wand, dir, wand->inv, NULL );
+
+            /* Success */
+            return 1;
+            }
+        }
+
+    if ( at_least_one )
+        return 0;
+
+    LOG(llevError,"Error: Monster %s (%d) HAS_READY_RANG() without wand/horn/rod.\n",
             head->name,head->count);
 	CLEAR_FLAG(head, FLAG_READY_RANGE);
 	return 0;
     }
-    if (!wand->inv) {
-	LOG(llevError,"Wand %s lacks spell\n", wand->name);
-	return 0;
-    }
-    if (wand->type == WAND) {
-	if(wand->stats.food<=0) {
-	    manual_apply(head,wand,0);
-	    CLEAR_FLAG(head, FLAG_READY_RANGE);
-	    if (wand->arch) {
-		CLEAR_FLAG(wand, FLAG_ANIMATE);
-		wand->face = wand->arch->clone.face;
-		wand->speed = 0;
-		update_ob_speed(wand);
-	    }
-	    return 0;
-	}
-    }
-    else {
-	if(wand->stats.hp<MAX(wand->inv->stats.sp, wand->inv->stats.grace))
-	    return 0; /* Not recharged enough yet */
-    }
-
-    /* Spell should be cast on caster (ie, heal, strength) */
-    if (wand->inv->range==0)
-	dir = 0;
-
-    if(cast_spell(part,wand,dir,wand->inv,NULL)) {
-	if (wand->type==WAND)
-	    wand->stats.food--;
-	return 1;
-    }
-    return 0;
-}
 
 int monster_use_bow(object *head, object *part, object *pl, int dir) {
     object *owner;
@@ -1246,16 +1257,40 @@ void monster_check_apply(object *mon, object *item) {
     else if (item->type == WEAPON) flag = check_good_weapon(mon,item);
     else if (IS_ARMOR(item)) flag = check_good_armour(mon,item);
     /* Should do something more, like make sure this is a better item */
-    else if (item->type == SKILL || item->type == RING || item->type==WAND ||
-	     item->type == ROD || item->type==HORN) flag=1;
+    else if (item->type == RING)
+        flag=1;
+    else if ( item->type==WAND || item->type == ROD || item->type==HORN )
+        {
+	    /* We never really 'ready' the wand/rod/horn, because that would mean the
+	    * weapon would get undone.
+	    */
+    	if (!(can_apply_object(mon, item) & CAN_APPLY_NOT_MASK))
+            {
+	        SET_FLAG(mon, FLAG_READY_RANGE);
+            SET_FLAG(item, FLAG_APPLIED);
+            }
+	    return;
+        }
     else if (item->type == BOW) {
-	/* We never really 'ready' the bow, because that would mean the
-	 * weapon would get undone.
-	 */
-	if (!(can_apply_object(mon, item) & CAN_APPLY_NOT_MASK))
-	    SET_FLAG(mon, FLAG_READY_BOW);
-	return;
-    }
+	    /* We never really 'ready' the bow, because that would mean the
+	    * weapon would get undone.
+	    */
+    	if (!(can_apply_object(mon, item) & CAN_APPLY_NOT_MASK))
+	        SET_FLAG(mon, FLAG_READY_BOW);
+	    return;
+        }
+    else if ( item->type == SKILL )
+        {
+        /*
+         * Ryo 2004-05-08
+         * skills are specials: monsters must have the 'FLAG_CAN_USE_SKILL' flag set,
+         * else they can't use the skill...
+         * Skills also don't need to get applied, so return now.
+         */
+        SET_FLAG( item, FLAG_CAN_USE_SKILL );
+        return;
+        }
+
 
     /* if we don't match one of the above types, return now.
      * can_apply_object will say that we can apply things like flesh, 
@@ -1270,7 +1305,6 @@ void monster_check_apply(object *mon, object *item) {
      */
     if (can_apply_object(mon, item) & CAN_APPLY_NOT_MASK) return;
 
-
     /* should only be applying this item, not unapplying it.
      * also, ignore status of curse so they can take off old armour.
      * monsters have some advantages after all.
@@ -1469,8 +1503,9 @@ void check_doors(object *op, mapstruct *m, int x, int y) {
  * match in 'match', and returns the portion of the message.  This
  * returned portion is in a malloc'd buf that should be freed.
  * Returns NULL if no match is found.
+ * The player is passed too, so that quest-related messages can be checked too.
  */
-static char *find_matching_message(char *msg, char *match)
+static char *find_matching_message(object* pl, char *msg, char *match)
 {
     char *cp=msg, *cp1, *cp2, regex[MAX_BUF], *cp3, gotmatch=0;
 
@@ -1509,17 +1544,18 @@ static char *find_matching_message(char *msg, char *match)
 		    }
 		}
 	    }
-	    if (gotmatch) {
+	    if (gotmatch && ( cp2 = quest_message_check( cp2 + 1, pl ) ) ) {
 		if (cp1) {
-		    cp3 = malloc(cp1 - cp2 + 1);
-		    strncpy(cp3, cp2+1, cp1 - cp2);
-		    cp3[cp1 - cp2] = 0;
+		    cp3 = malloc(cp1 - cp2);
+		    strncpy(cp3, cp2, cp1 - cp2 - 1);
+		    cp3[cp1 - cp2 - 1] = 0;
 		}
 		else {	/* if no next match, just want the rest of the string */
-		    cp3 = strdup_local(cp2 + 1);
+		    cp3 = strdup_local(cp2);
 		}
-		return cp3;
+        return cp3;
 	    }
+        gotmatch = 0;
 	    if (cp1) cp = cp1 + 1;
 	    else return NULL;
 	}
@@ -1563,7 +1599,7 @@ void communicate(object *op, char *txt) {
 
 	for(npc = get_map_ob(mp,x,y); npc != NULL; npc = npc->above) {
 	    if (npc->type == MAGIC_EAR) {
-		(void) talk_to_wall(npc, txt); /* Maybe exit after 1. success? */
+		(void) talk_to_wall(op, npc, txt); /* Maybe exit after 1. success? */
 		if (orig_map != op->map) {
 		    LOG(llevDebug,"Warning: Forced to swap out very recent map - MAX_OBJECTS should probably be increased\n");
 		    return;
@@ -1589,6 +1625,7 @@ int talk_to_npc(object *op, object *npc, char *txt) {
     object *cobj;
     CFParm CFP;
     int k, l, m;
+    uint32 n;
     char *cp, buf[MAX_BUF];
     event *evt;
 
@@ -1597,10 +1634,11 @@ int talk_to_npc(object *op, object *npc, char *txt) {
     k = EVENT_SAY;
     l = SCRIPT_FIX_ALL;
     m = 0;
+    n = 0;
     CFP.Value[0] = &k;
     CFP.Value[1] = op;
     CFP.Value[4] = txt;
-    CFP.Value[5] = &m;
+    CFP.Value[5] = &n;
     CFP.Value[6] = &m;
     CFP.Value[7] = &m;
     CFP.Value[8] = &l;
@@ -1638,7 +1676,7 @@ int talk_to_npc(object *op, object *npc, char *txt) {
     if(npc->msg == NULL || *npc->msg != '@')
 	return 0;
 
-    cp = find_matching_message(npc->msg, txt);
+    cp = find_matching_message(op, npc->msg, txt );
     if (cp) {
         sprintf(buf,"%s says:",query_name(npc));
 	new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map,buf);
@@ -1649,13 +1687,13 @@ int talk_to_npc(object *op, object *npc, char *txt) {
     return 0;
 }
 
-int talk_to_wall(object *npc, char *txt) {
+int talk_to_wall(object* pl, object *npc, char *txt) {
     char *cp;
 
     if(npc->msg == NULL || *npc->msg != '@')
 	return 0;
 
-    cp = find_matching_message(npc->msg, txt);
+    cp = find_matching_message(pl, npc->msg, txt);
     if (!cp)
 	return 0;
 
@@ -1723,6 +1761,10 @@ int can_detect_enemy (object *op, object *enemy, rv_vector *rv) {
 
     get_rangevector(op, enemy, rv, 0);
 
+    /* Monsters always ignore the DM */
+    if ( ( op->type != PLAYER ) && QUERY_FLAG( enemy, FLAG_WIZ ) )
+        return 0;
+
     /* simple check.  Should probably put some range checks in here. */
     if(can_see_enemy(op,enemy)) return 1;
 
@@ -1751,7 +1793,7 @@ int can_detect_enemy (object *op, object *enemy, rv_vector *rv) {
 	    if((sk_hide = find_skill_by_number(enemy,SK_HIDING)))
 		bonus -= sk_hide->level;
 	    else { 
-		LOG(llevError,"can_detect_enemy() got hidden player w/o hiding skill!");
+		LOG(llevError,"can_detect_enemy() got hidden player w/o hiding skill!\n");
 		make_visible(enemy);
 		radius=radius<MIN_MON_RADIUS?MIN_MON_RADIUS:radius;
 	    }
@@ -1844,15 +1886,15 @@ int stand_in_light( object *op) {
 
 
     if(!op) return 0;
-    if(op->glow_radius) return 1;
+    if(op->glow_radius > 0) return 1;
 
     if(op->map) {
-	int x, y;
+	int x, y, x1, y1;
+	
+	
 
-	/* Check the spacs with the max light radius to see if any of them
-	 * have lights, and if the light is bright enough to illuminate
-	 * this object.  Like the los.c logic, this presumes a square
-	 * lighting area.
+	/* Check the spaces with the max light radius to see if any of them
+	 * have lights, and if any of them light the player enough, then return 1.
 	 */
 	for (x = op->x - MAX_LIGHT_RADII; x< op->x + MAX_LIGHT_RADII; x++) {
 	    for (y = op->y - MAX_LIGHT_RADII; y< op->y + MAX_LIGHT_RADII; y++) {
@@ -1862,7 +1904,9 @@ int stand_in_light( object *op) {
 
 		if (get_map_flags(m, &m, nx, ny, &nx, &ny) & P_OUT_OF_MAP) continue;
 
-		if (GET_MAP_LIGHT(m, nx, ny) > MAX(abs(x - op->x), abs(y - op->y))) return 1;
+		x1 = abs(x - op->x)*abs(x - op->x);
+		y1 = abs(y - op->y)*abs(y - op->y);
+		if (isqrt(x1 + y1) <  GET_MAP_LIGHT(m, nx, ny)) return 1; 
 	    }
 	}
     }
@@ -1900,7 +1944,7 @@ int can_see_enemy (object *op, object *enemy) {
 	    if(enemy->hide) { 
 		make_visible(enemy);
 		new_draw_info(NDI_UNIQUE,0, enemy,
-			      "Your light reveals your hidding spot!");
+			      "Your light reveals your hiding spot!");
 	    }
 	    return 1;
 	} else if (enemy->hide) return 0; 
