@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_pets_c =
- *    "$Id: pets.c,v 1.22 2004/04/05 07:01:11 mwedel Exp $";
+ *    "$Id: pets.c,v 1.33 2005/08/01 20:36:25 qal21 Exp $";
  */
 
 /*
@@ -115,10 +115,13 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	    for (tmp = get_map_ob(nm, x, y); tmp != NULL; tmp = tmp->above) {
 		object *tmp2 = tmp->head == NULL?tmp:tmp->head;
 
-		if (QUERY_FLAG(tmp2,FLAG_ALIVE) && !QUERY_FLAG(tmp2,FLAG_FRIENDLY)
-		    && !QUERY_FLAG(tmp2,FLAG_UNAGGRESSIVE) && tmp2 != pet && 
-		    tmp2 != owner && tmp2->type != PLAYER &&
-		    can_detect_enemy(pet, tmp2, rv)) {
+	    if (QUERY_FLAG(tmp2,FLAG_ALIVE) && ((
+				!QUERY_FLAG(tmp2, FLAG_FRIENDLY) &&
+				(tmp2->type != PLAYER)) || 
+			should_arena_attack(pet, owner, tmp2))
+		&& !QUERY_FLAG(tmp2,FLAG_UNAGGRESSIVE) &&
+		tmp2 != pet && tmp2 != owner && 
+		can_detect_enemy(pet, tmp2, rv)) {
 
 			if (!can_see_enemy(pet, tmp2)) {
 			    if (tmp3 != NULL)
@@ -179,11 +182,12 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	    if (get_map_flags(nm, &nm, x,y, &x, &y) & P_IS_ALIVE) {
 		for (tmp = get_map_ob(nm, x, y); tmp != NULL; tmp = tmp->above) {
 		    object *tmp2 = tmp->head == NULL?tmp:tmp->head;
-
-		    if (QUERY_FLAG(tmp2,FLAG_ALIVE) &&
-			!QUERY_FLAG(tmp2,FLAG_FRIENDLY)
+		    if (QUERY_FLAG(tmp2,FLAG_ALIVE) && ((
+					!QUERY_FLAG(tmp2, FLAG_FRIENDLY) && 
+					(tmp2->type != PLAYER)) || 
+				should_arena_attack(pet, owner, tmp2))
 			&& !QUERY_FLAG(tmp2,FLAG_UNAGGRESSIVE) &&
-			tmp2 != pet && tmp2 != owner && tmp2->type != PLAYER &&
+			tmp2 != pet && tmp2 != owner && 
 			can_detect_enemy(pet, tmp2, rv)) {
 
 			    if (!can_see_enemy(pet, tmp2)) {
@@ -252,7 +256,7 @@ void remove_all_pets(mapstruct *map) {
 	follow_owner(obl->ob,owner);
 	if(QUERY_FLAG(obl->ob, FLAG_REMOVED) && FABS(obl->ob->speed) > MIN_ACTIVE_SPEED) {
 	    object *ob = obl->ob;
-	    LOG(llevMonster,"(pet failed to follow)");
+	    LOG(llevMonster,"(pet failed to follow)\n");
 	    remove_friendly_object(ob);
 	    free_object(ob);
 	}
@@ -276,7 +280,7 @@ void follow_owner(object *ob, object *owner) {
 	return;
     }
     dir = find_free_spot(ob->arch, owner->map,
-                       owner->x, owner->y, 1, SIZEOFFREE+1);
+                       owner->x, owner->y, 1, SIZEOFFREE);
 
     if (dir==-1) {
 	LOG(llevMonster,"No space for pet to follow, freeing %s.\n",ob->name);
@@ -446,16 +450,17 @@ object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem ) {
     head->direction = dir;
 
     /* need to change some monster attr to prevent problems/crashing */
-    if(head->last_heal) head->last_heal=0;
-    if(head->last_eat) head->last_eat=0;
-    if(head->last_grace) head->last_grace=0;
-    if(head->last_sp) head->last_sp=0;
+    head->last_heal=0;
+    head->last_eat=0;
+    head->last_grace=0;
+    head->last_sp=0;
+    head->other_arch=NULL;
+    head->stats.exp = 0;
+    CLEAR_FLAG(head,FLAG_CHANGING);
+    CLEAR_FLAG(head,FLAG_STAND_STILL);
+    CLEAR_FLAG(head,FLAG_GENERATOR);
+    CLEAR_FLAG(head,FLAG_SPLITTING);
     if(head->attacktype&AT_GHOSTHIT) head->attacktype=(AT_PHYSICAL|AT_DRAIN);
-    if(head->other_arch) head->other_arch=NULL;
-    if(QUERY_FLAG(head,FLAG_CHANGING)) CLEAR_FLAG(head,FLAG_CHANGING);
-    if(QUERY_FLAG(head,FLAG_STAND_STILL)) CLEAR_FLAG(head,FLAG_STAND_STILL);
-    if(QUERY_FLAG(head,FLAG_GENERATOR)) CLEAR_FLAG(head,FLAG_GENERATOR);
-    if(QUERY_FLAG(head,FLAG_SPLITTING)) CLEAR_FLAG(head,FLAG_SPLITTING);
 
     return head;
 }
@@ -737,7 +742,7 @@ object *choose_cult_monster(object *pl, object *god, int summon_level) {
 	new_draw_info_format(NDI_UNIQUE, 0,pl,
 	    "The spell fails! %s's creatures are beyond the range of your summons",
 	     god->name);
-	LOG(llevDebug,"choose_cult_monster() requested non-existant aligned race!\n");
+	LOG(llevDebug,"choose_cult_monster() requested non-existent aligned race!\n");
 	return 0; 
     }
 
@@ -786,8 +791,8 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	 */
 	for (tr=spell_ob->randomitems->items; tr; tr=tr->next) {
 	    if (level < tr->magic) break;
-	    if (tr->next == NULL || tr->next->item == NULL) break;
 	    lasttr = tr;
+	    if (tr->next == NULL || tr->next->item == NULL) break;
 	}
 	if (!lasttr) {
 	    LOG(llevError,"Treasurelist %s did not generate a valid entry in summon_object\n",
@@ -902,10 +907,11 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	    tmp->map = op->map;
 	}
 	head->direction = dir;
+	head->stats.exp = 0;
 	head = insert_ob_in_map(head, head->map, op, 0);
 	if (head && head->randomitems) {
 	    object *tmp;
-	    create_treasure(head->randomitems, head, GT_APPLY, 6, 0);
+	    create_treasure(head->randomitems, head, GT_APPLY | GT_STARTEQUIP, 6, 0);
 	    for (tmp=head->inv; tmp; tmp=tmp->below)
 		if (!tmp->nrof) SET_FLAG(tmp, FLAG_NO_DROP);
 	}
@@ -920,5 +926,81 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
     } /* for i < nrof */
     return 1;
 }
-		    
 
+/* recursively look through the owner property of objects until the real owner
+is found */
+object *get_real_owner(object *ob) {
+	object *realowner = ob;
+	
+	if (realowner == NULL) return NULL;
+	
+	while(get_owner(realowner) != NULL)
+	{
+		realowner = get_owner(realowner);
+	}
+	return realowner;
+}
+		    
+/* determines if checks so pets don't attack players or other pets should be
+overruled by the arena petmode */
+int should_arena_attack(object *pet,object *owner,object *target) {
+	object *rowner, *towner;
+	
+	/* exit if the target, pet, or owner is null. */
+	if ((target == NULL) || (pet == NULL) || (owner == NULL)) return 0;
+		
+	/* get the owners of itself and the target, this is to deal with pets of
+	pets */
+	rowner = get_real_owner(owner);
+	if (target->type != PLAYER) {
+		towner = get_real_owner(target);
+	} else {
+		towner = 0;	
+	}
+	
+	/* if the pet has now owner, exit with error */
+	if (rowner == NULL) {
+		LOG(llevError,"Pet has no owner.\n");
+		return 0;
+	}
+
+	/* if the target is not a player, and has no owner, we shouldn't be here
+	*/
+	if ((towner == NULL) && (target->type != PLAYER)) {
+		LOG(llevError,"Target is not a player but has no owner. We should not be here.\n");
+		return 0;
+	}
+	
+	/* make sure that the owner is a player */
+	if (rowner->type != PLAYER) return 0;
+	
+	/* abort if the petmode is not arena */
+	if (rowner->contr->petmode != pet_arena) return 0;
+	
+	/* abort if the pet, it's owner, or the target is not on battleground*/
+	if (!(op_on_battleground(pet, NULL, NULL) && 
+	     op_on_battleground(owner, NULL, NULL) && 
+	     op_on_battleground(target, NULL, NULL)))
+	     return 0;
+	
+	/* if the target is a monster, make sure it's owner is not the same */	
+	if ((target->type != PLAYER) && (rowner == towner)) return 0;
+
+	/* check if the target is a player which affects how it will handle
+	parties */
+	if (target->type != PLAYER) {
+		/* if the target is owned by a player make sure than make sure
+		it's not in the same party */
+		if ((towner->type == PLAYER) && (rowner->contr->party_number > 0)) {
+			if (rowner->contr->party_number == towner->contr->party_number) return 0;
+		}
+	} else {
+		/* if the target is a player make sure than make sure it's not
+		in the same party */
+		if (rowner->contr->party_number > 0){
+			if (rowner->contr->party_number == target->contr->party_number) return 0;
+		}
+	}
+	
+	return 1;
+}
