@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_resurrection_c =
- *   "$Id: resurrection.c,v 1.12 2004/03/26 21:59:28 ryo_saeba Exp $";
+ *   "$Id: resurrection.c,v 1.13 2005/07/16 21:10:39 akirschbaum Exp $";
  */
 
 /*
@@ -109,7 +109,7 @@ static int resurrect_player(object *op,char *playername,object *spell)
     if(! (liveplayer=fopen(newname,"w"))) {
 	new_draw_info_format(NDI_UNIQUE, 0, op,
 		"The soul of %s cannot be re-embodied at the moment.",playername);
-	LOG(llevError,"Cannot write player in ressurect_player!\n");
+	LOG(llevError,"Cannot write player in resurrect_player!\n");
 	fclose(deadplayer);
 	return 0;
     }
