@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_rune_c =
- *   "$Id: rune.c,v 1.37 2004/03/23 07:52:31 mwedel Exp $";
+ *   "$Id: rune.c,v 1.41 2005/07/16 21:10:39 akirschbaum Exp $";
  */
 
 /*
@@ -96,7 +96,7 @@ int write_rune(object *op,object *caster, object *spell, int dir,char *runename)
 	    if (tmp->type == SPELL) {
 		ms = item_matched_string(op, tmp, runename);
 		if (ms > bestmatch) {
-		    ms = bestmatch;
+		    bestmatch = ms;
 		    rune_spell = tmp;
 		}
 	    }
@@ -137,7 +137,7 @@ int write_rune(object *op,object *caster, object *spell, int dir,char *runename)
 	rune = rune_spell;
     } else {
 	rune = get_archetype(GENERIC_RUNE);
-	sprintf(buf,"You set off a rune of %s",rune_spell->name);
+	sprintf(buf,"You set off a rune of %s\n",rune_spell->name);
 	rune->msg=add_string(buf);
 	tmp = get_object();
 	copy_object(rune_spell, tmp);
@@ -168,8 +168,9 @@ int write_rune(object *op,object *caster, object *spell, int dir,char *runename)
     rune->direction :	    direction it will cast a spell in
     rune->dam	    :	    damage the rune will do if it doesn't cast spells
     rune->attacktype:	    type of damage it does, if not casting spells
-    rune->slaying   :       string description of the spell in the rune
-
+    rune->other_arch:       spell in the rune
+    rune->Cha       :       how hidden the rune is
+    rune->maxhp     :       number of spells the rune casts
 */
 
 void move_rune(object *op) {
@@ -221,7 +222,7 @@ void spring_trap(object *trap,object *victim)
     object *env;
     tag_t trap_tag = trap->count;
     rv_vector rv;
-
+    int i;
 
     /* Prevent recursion */
     if (trap->stats.hp <= 0)
@@ -268,12 +269,14 @@ void spring_trap(object *trap,object *victim)
 	if (was_destroyed (trap, trap_tag))
 	    return;
 
-	if (trap->inv)
-	    cast_spell(trap,trap,trap->direction,trap->inv,NULL);
-	else {
-	    spell = arch_to_object(trap->other_arch);
-	    cast_spell(trap,trap,trap->direction,spell,NULL);
-	    free_object(spell);
+	for(i = 0; i < MAX(1, trap->stats.maxhp); i++) {
+	    if (trap->inv)
+		cast_spell(trap,trap,trap->direction,trap->inv,NULL);
+	    else {
+		spell = arch_to_object(trap->other_arch);
+		cast_spell(trap,trap,trap->direction,spell,NULL);
+		free_object(spell);
+	    }
 	}
     } else {
 	rune_attack(trap,victim); 
@@ -390,7 +393,7 @@ int trap_disarm(object *disarmer, object *trap, int risk, object *skill) {
 	   +5 - disarmer->stats.Dex/2))-1), disarmer, PREFER_LOW)))
         {
             new_draw_info_format(NDI_UNIQUE, 0,disarmer,
-                    "You successfuly disarm the %s!",trap->name);
+                    "You successfully disarm the %s!",trap->name);
             destroy_object(trap);
 	    /* If it is your own trap, (or any players trap), don't you don't
 	     * get exp for it.
@@ -426,7 +429,9 @@ void trap_adjust(object *trap, int difficulty) {
      * a peak probability at difficulty 
      */
 
-    trap->level = MAX(1, rndm(0, difficulty-1) + rndm(0, difficulty-1));
+    trap->level = rndm(0, difficulty-1) + rndm(0, difficulty-1);
+    if(trap->level < 1)
+	trap->level = 1;
 
     /* set the hiddenness of the trap, similar formula to above */
     trap->stats.Cha = rndm(0, 19) + rndm(0, difficulty-1) + rndm(0, difficulty-1);
@@ -442,8 +447,11 @@ void trap_adjust(object *trap, int difficulty) {
 	    trap->stats.dam+=rndm(0, 4);
     
 	/*  the poison trap special case */
-	if(trap->attacktype & AT_POISON)
-	    trap->stats.dam = MAX(1, rndm(0, difficulty-1));  
+	if(trap->attacktype & AT_POISON) {
+	    trap->stats.dam = rndm(0, difficulty-1);
+	    if(trap->stats.dam < 1)
+		trap->stats.dam = 1;
+	}
 
 	/*  so we get an appropriate amnt of exp for AT_DEATH traps */
 	if(trap->attacktype & AT_DEATH) trap->stats.dam = 127;
