@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c,v 1.28 2004/04/25 07:17:40 mwedel Exp $";
+ *   "$Id: shop.c,v 1.37 2005/07/14 09:18:55 akirschbaum Exp $";
  */
 
 /*
@@ -36,6 +36,9 @@
 #endif
 
 
+static uint64 pay_from_container(object *pl, object *pouch, uint64 to_pay);
+
+
 #define NUM_COINS 3	/* number of coin types */
 static char *coins[] = {"platinacoin", "goldcoin", "silvercoin", NULL};
 
@@ -60,11 +63,15 @@ uint64 query_cost(object *tmp, object *who, int flag) {
     uint64 val;
     int number;	/* used to better calculate value */
     int no_bargain;
+    int identified;
+    int not_cursed;
     float diff;
     float ratio;
 
     no_bargain = flag & F_NO_BARGAIN;
-    flag &= ~F_NO_BARGAIN;
+    identified = flag & F_IDENTIFIED;
+    not_cursed = flag & F_NOT_CURSED;
+    flag &= ~(F_NO_BARGAIN|F_IDENTIFIED|F_NOT_CURSED);
 
     if (tmp->type==MONEY) return (tmp->nrof * tmp->value);
     if (tmp->type==GEM) {
@@ -76,8 +83,8 @@ uint64 query_cost(object *tmp, object *who, int flag) {
     }
     number = tmp->nrof;
     if (number==0) number=1;
-    if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) || !need_identify(tmp)) {
-	if (QUERY_FLAG(tmp, FLAG_CURSED) || QUERY_FLAG(tmp, FLAG_DAMNED))
+    if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) || !need_identify(tmp) || identified) {
+	if (!not_cursed && (QUERY_FLAG(tmp, FLAG_CURSED) || QUERY_FLAG(tmp, FLAG_DAMNED)))
 	    return 0;
 	else
 	    val=tmp->value * number;
@@ -86,7 +93,7 @@ uint64 query_cost(object *tmp, object *who, int flag) {
     else {
 	if (tmp->arch != NULL) {
 	    if (flag == F_BUY) {
-		LOG(llevError, "Asking for buy-value of unidentified object.");
+		LOG(llevError, "Asking for buy-value of unidentified object.\n");
 		val = tmp->arch->clone.value * 50 * number;
 	    }
 	    else {	/* Trying to sell something, or get true value */
@@ -105,7 +112,7 @@ uint64 query_cost(object *tmp, object *who, int flag) {
 	} else { /* No archetype with this object */
 	    LOG(llevDebug,"In sell item: Have object with no archetype: %s\n", tmp->name);
 	    if (flag == F_BUY) {
-		LOG(llevError, "Asking for buy-value of unidentified object without arch.");
+		LOG(llevError, "Asking for buy-value of unidentified object without arch.\n");
 		val = number * tmp->value * 10;
 	    }
 	    else
@@ -122,7 +129,7 @@ uint64 query_cost(object *tmp, object *who, int flag) {
      * default magical.  This is because the archetype value should have
      * already figured in that value.
      */
-    if((QUERY_FLAG(tmp, FLAG_IDENTIFIED)||!need_identify(tmp)||
+    if((QUERY_FLAG(tmp, FLAG_IDENTIFIED)||!need_identify(tmp)||identified||
 	QUERY_FLAG(tmp, FLAG_BEEN_APPLIED)) &&
         tmp->magic&&(tmp->arch==NULL||!tmp->arch->clone.magic)) {
 	    if(tmp->magic>0)
@@ -139,7 +146,7 @@ uint64 query_cost(object *tmp, object *who, int flag) {
 	 * charges.  the treasure code already sets up the value
 	 * 50 charges is used as the baseline.
 	 */
-	if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) || !need_identify(tmp))
+	if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) || !need_identify(tmp) || identified)
 	    val=(val*tmp->stats.food) / 50;
 	else /* if not identified, presume one charge */
 	    val/=50;
@@ -189,9 +196,9 @@ uint64 query_cost(object *tmp, object *who, int flag) {
 	 * reflect values (potion charisma list for 1250 gold)
 	 */
 	if(flag==F_BUY)
-          val=(4.0*(float)val*(1.0+diff));
+          val=(4*val*(long)(1000*(1+diff)))/1000;
 	else if (flag==F_SELL)
-          val=(4.0*(float)val*(1.0-diff));
+          val=(4*val*(long)(1000*(1-diff)))/1000;
 	else val *=4;
     }
 
@@ -204,7 +211,7 @@ uint64 query_cost(object *tmp, object *who, int flag) {
 	val=0;
 
     /* Unidentified stuff won't sell for more than 60gp */
-    if(flag==F_SELL && !QUERY_FLAG(tmp, FLAG_IDENTIFIED) && need_identify(tmp)) {
+    if(flag==F_SELL && !QUERY_FLAG(tmp, FLAG_IDENTIFIED) && need_identify(tmp) && !identified) {
 	 val = (val > 600)? 600:val;
     }
     return val;
@@ -310,7 +317,7 @@ uint64 query_money(object *op) {
     uint64 total=0;
 
     if (op->type!=PLAYER && op->type!=CONTAINER) {
-	LOG(llevError, "Query money called with non player/container");
+	LOG(llevError, "Query money called with non player/container\n");
 	return 0;
     }
     for (tmp = op->inv; tmp; tmp= tmp->below) {
@@ -329,19 +336,19 @@ uint64 query_money(object *op) {
  * pay_from_container function.
  * returns 0 if not possible. 1 if success
  */
-int pay_for_amount(int to_pay,object *pl) {
+int pay_for_amount(uint64 to_pay,object *pl) {
     object *pouch;
 
     if (to_pay==0) return 1;
     if (to_pay > query_money(pl)) return 0;
 
-    to_pay = pay_from_container(NULL, pl, to_pay);
+    to_pay = pay_from_container(pl, pl, to_pay);
 
     for (pouch=pl->inv; (pouch!=NULL) && (to_pay>0); pouch=pouch->below) {
 	if (pouch->type == CONTAINER
 	    && QUERY_FLAG(pouch, FLAG_APPLIED)
 	    && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
-	    to_pay = pay_from_container(NULL, pouch, to_pay);
+	    to_pay = pay_from_container(pl, pouch, to_pay);
 	}
     }
     fix_player(pl);
@@ -370,13 +377,13 @@ int pay_for_item(object *op,object *pl) {
     if (saved_money > 0)
       change_exp(pl,saved_money,"bargaining",SK_EXP_NONE);
 
-    to_pay = pay_from_container(op, pl, to_pay);
+    to_pay = pay_from_container(pl, pl, to_pay);
 
     for (pouch=pl->inv; (pouch!=NULL) && (to_pay>0); pouch=pouch->below) {
 	if (pouch->type == CONTAINER
 	    && QUERY_FLAG(pouch, FLAG_APPLIED)
 	    && (pouch->race == NULL || strstr(pouch->race, "gold"))) {
-	    to_pay = pay_from_container(op, pouch, to_pay);
+	    to_pay = pay_from_container(pl, pouch, to_pay);
 	}
     }
     if (settings.real_wiz == FALSE && QUERY_FLAG(pl, FLAG_WAS_WIZ))
@@ -393,14 +400,17 @@ int pay_for_item(object *op,object *pl) {
  * appropriately
  *
  * DAMN: This function is used for the player, then for any active
- * containers that can hold money, until the op is paid for.
+ * containers that can hold money.
+ *
+ * pouch is the container (pouch or player) to remove the coins from.
+ * to_pay is the required amount.
+ * returns the amount still missing after using "pouch".
  */
-uint64 pay_from_container(object *op, object *pouch, int to_pay) {
+static uint64 pay_from_container(object *pl, object *pouch, uint64 to_pay) {
     int count, i;
-    uint64 remain;
+    sint64 remain;
     object *tmp, *coin_objs[NUM_COINS], *next;
     archetype *at;
-    object *who;
 
     if (pouch->type != PLAYER && pouch->type != CONTAINER) return to_pay;
 
@@ -423,12 +433,12 @@ uint64 pay_from_container(object *op, object *pouch, int to_pay) {
 			    pouch->name, coins[NUM_COINS-1-i]);
 			remove_ob(tmp);
 			coin_objs[i]->nrof += tmp->nrof;
-			esrv_del_item(pouch->contr, tmp->count);
+			esrv_del_item(pl->contr, tmp->count);
 			free_object(tmp);
 		    }
 		    else {
 			remove_ob(tmp);
-			if(pouch->type==PLAYER) esrv_del_item(pouch->contr, tmp->count);
+			if(pouch->type==PLAYER) esrv_del_item(pl->contr, tmp->count);
 			coin_objs[i] = tmp;
 		    }
 		    break;
@@ -444,7 +454,7 @@ uint64 pay_from_container(object *op, object *pouch, int to_pay) {
     for (i=0; i<NUM_COINS; i++)
 	if (coin_objs[i]==NULL) {
 	    at = find_archetype(coins[NUM_COINS-1-i]);
-	    if (at==NULL) LOG(llevError, "Could not find %s archetype", coins[NUM_COINS-1-i]);
+	    if (at==NULL) LOG(llevError, "Could not find %s archetype\n", coins[NUM_COINS-1-i]);
 	    coin_objs[i] = get_object();
 	    copy_object(&at->clone, coin_objs[i]);
 	    coin_objs[i]->nrof = 0;
@@ -459,8 +469,7 @@ uint64 pay_from_container(object *op, object *pouch, int to_pay) {
 	} else {
 	    num_coins = coin_objs[i]->nrof;
 	}
-
-	remain -= (uint64)num_coins * (uint64)coin_objs[i]->value;
+	remain -= (sint64) num_coins * (sint64)coin_objs[i]->value;
 	coin_objs[i]->nrof -= num_coins;
 	/* Now start making change.  Start at the coin value
 	 * below the one we just did, and work down to
@@ -477,13 +486,12 @@ uint64 pay_from_container(object *op, object *pouch, int to_pay) {
     for (i=0; i<NUM_COINS; i++) {
 	if (coin_objs[i]->nrof) {
 	    object *tmp = insert_ob_in_ob(coin_objs[i], pouch);
-	    for (who = pouch; who && who->type!=PLAYER && who->env!=NULL; who=who->env) ;
-	    esrv_send_item(who, tmp);
-	    esrv_send_item (who, pouch);
-	    esrv_update_item (UPD_WEIGHT, who, pouch);
-	    if (pouch->type != PLAYER) {
-		esrv_send_item (who, who);
-		esrv_update_item (UPD_WEIGHT, who, who);
+
+	    esrv_send_item(pl, tmp);
+	    esrv_send_item (pl, pouch);
+	    if (pl != pouch) esrv_update_item (UPD_WEIGHT, pl, pouch);
+	    if (pl->type != PLAYER) {
+		esrv_send_item (pl, pl);
 	    }
 	} else {
 	    free_object(coin_objs[i]);
@@ -519,6 +527,7 @@ int get_payment(object *pl, object *op) {
    
     if(op!=NULL&&QUERY_FLAG(op,FLAG_UNPAID)) {
         strncpy(buf,query_cost_string(op,pl,F_BUY),MAX_BUF);
+        buf[MAX_BUF-1] = '\0';
         if(!pay_for_item(op,pl)) {
             uint64 i=query_cost(op,pl,F_BUY) - query_money(pl);
 	    CLEAR_FLAG(op, FLAG_UNPAID);
@@ -593,7 +602,7 @@ void sell_item(object *op, object *pl) {
   
     for (count=0; coins[count]!=NULL; count++) {
 	at = find_archetype(coins[count]);
-	if (at==NULL) LOG(llevError, "Could not find %s archetype", coins[count]);
+	if (at==NULL) LOG(llevError, "Could not find %s archetype\n", coins[count]);
 	else if ((i/at->clone.value) > 0) {
 	    for ( pouch=pl->inv ; pouch ; pouch=pouch->below ) {
 		if ( pouch->type==CONTAINER && QUERY_FLAG(pouch, FLAG_APPLIED) && pouch->race && strstr(pouch->race, "gold") ) {
@@ -614,7 +623,6 @@ void sell_item(object *op, object *pl) {
 			esrv_send_item (pl, pouch);
 			esrv_update_item (UPD_WEIGHT, pl, pouch);
 			esrv_send_item (pl, pl);
-			esrv_update_item (UPD_WEIGHT, pl, pl);
 		    }
 		}
 	    }
@@ -626,13 +634,16 @@ void sell_item(object *op, object *pl) {
 		tmp = insert_ob_in_ob(tmp, pl);
 		esrv_send_item (pl, tmp);
 		esrv_send_item (pl, pl);
-		esrv_update_item (UPD_WEIGHT, pl, pl);
 	    }
 	}
     }
 
     if (i!=0) 
-	LOG(llevError,"Warning - payment not zero: %d\n", i);
+#ifndef WIN32
+	LOG(llevError,"Warning - payment not zero: %llu\n", i);
+#else
+	LOG(llevError,"Warning - payment not zero: %I64u\n", i);
+#endif
 
     new_draw_info_format(NDI_UNIQUE, 0, pl,
 	"You receive %s for %s.",query_cost_string(op,pl,1),
