@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c,v 1.47 2004/04/08 06:48:51 mwedel Exp $";
+ *   "$Id: skill_util.c,v 1.53 2005/07/23 10:06:36 ryo_saeba Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -69,7 +69,7 @@ void init_skills() {
 	if (at->clone.type == SKILL) {
 	    if (skill_names[at->clone.subtype] != NULL) {
 		LOG(llevError, "init_skills: multiple skill using same subtype %d, %s, %s\n",
-		    skill_names[at->clone.subtype], at->clone.skill);
+		    at->clone.subtype, skill_names[at->clone.subtype], at->clone.skill);
 	    } else {
 		skill_names[at->clone.subtype] = add_refcount(at->clone.skill);
 	    }
@@ -229,7 +229,12 @@ object *find_skill_by_number(object *who, int skillno)
 
 int change_skill (object *who, object *new_skill, int flag)
 {
-    int old_range = who->contr->shoottype;
+    int old_range;
+
+    if ( who->type != PLAYER )
+        return 0;
+    
+    old_range = who->contr->shoottype;
 
     if (who->chosen_skill && who->chosen_skill == new_skill)
     {
@@ -604,7 +609,7 @@ learn_skill (object *pl, object *scroll) {
 
 /* Gives a percentage clipped to 0% -> 100% of a/b. */
 /* Probably belongs in some global utils-type file? */
-static int clipped_percent(int a, int b)
+static int clipped_percent(sint64 a, sint64 b)
 {
   int rv;
 
@@ -653,13 +658,13 @@ void show_skills(object *op) {
 			 buf,tmp->level,
 			 tmp->stats.exp,
 			 level_exp(tmp->level+1, op->expmul),
-			 clipped_percent(tmp->last_heal,tmp->stats.exp));
+			 clipped_percent(tmp->perm_exp,tmp->stats.exp));
 #else
 		sprintf(skills[num_skills_found++],"%slvl:%3d (xp:%lld/%lld/%d%%)",
 			 buf,tmp->level,
 			 tmp->stats.exp,
 			 level_exp(tmp->level+1, op->expmul),
-			 clipped_percent(tmp->last_heal,tmp->stats.exp));
+			 clipped_percent(tmp->perm_exp,tmp->stats.exp));
 #endif
 	    } else {
 #ifdef WIN32
@@ -871,7 +876,7 @@ static int do_skill_attack(object *tmp, object *op, char *string, object *skill)
 		if (op->current_weapon_script)
 		    FREE_AND_CLEAR_STR(op->current_weapon_script);
 		if (!tmp) {
-		    LOG(llevError,"Could not find applied weapon on %s",
+		    LOG(llevError,"Could not find applied weapon on %s\n",
 			op->name);
 		    op->current_weapon=NULL;
 		    return 0;
@@ -901,7 +906,7 @@ static int do_skill_attack(object *tmp, object *op, char *string, object *skill)
  
     /* print appropriate  messages to the player */
  
-    if(success && string!=NULL) {
+    if(success && string!=NULL && tmp && !QUERY_FLAG(tmp,FLAG_FREED)) {
 	if(op->type==PLAYER)
 	    new_draw_info_format(NDI_UNIQUE, 0,op,
 		 "You %s %s!",string,query_name(tmp));
