@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_c =
- *   "$Id: skills.c,v 1.50 2004/04/25 06:18:39 mwedel Exp $";
+ *   "$Id: skills.c,v 1.61 2005/07/27 17:54:54 ryo_saeba Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -88,6 +88,9 @@ static int adj_stealchance (object *op, object *victim, int roll) {
  * stealing from (op in attempt_steal), offset by your dexterity and
  * skill at stealing. They may notice your attempt, whether successful
  * or not. 
+ * op is the target (person being pilfered)
+ * who is the person doing the stealing.
+ * skill is the skill object (stealing).
  */
 
 static int attempt_steal(object* op, object* who, object *skill)
@@ -110,6 +113,7 @@ static int attempt_steal(object* op, object* who, object *skill)
 	    return 0;
 	} else /* help npc to detect thief next time by raising its wisdom */ 
 	    op->stats.Wis += (op->stats.Int/5)+1;
+	    if (op->stats.Wis > MAX_STAT) op->stats.Wis = MAX_STAT;
     }
     if (op->type == PLAYER && QUERY_FLAG(op, FLAG_WIZ)) {
 	new_draw_info(NDI_UNIQUE, 0, who, "You can't steal from the dungeon master!\n");
@@ -146,18 +150,25 @@ static int attempt_steal(object* op, object* who, object *skill)
 	if((chance=adj_stealchance(who,op,(stats_value+skill->level * 10 - op->level * 3)))==-1)
 	    return 0;
 	else if (roll < chance ) {
-	    if (op->type == PLAYER)
-		esrv_del_item(op->contr, tmp->count);
+	    tag_t tmp_count = tmp->count;
+
 	    pick_up(who, tmp);
 	    /* need to see if the player actually stole this item -
 	     * if it is in the players inv, assume it is.  This prevents
 	     * abuses where the player can not carry the item, so just
 	     * keeps stealing it over and over.
 	     */
-	    if (who == is_player_inv(tmp)) {
+	    if (was_destroyed(tmp, tmp_count) || tmp->env != op) {
 		/* for players, play_sound: steals item */
 		success = tmp;
 		CLEAR_FLAG(tmp, FLAG_INV_LOCKED);
+
+		/* Don't delete it from target player until we know
+		 * the thief has picked it up.  can't just look at tmp->count,
+		 * as it's possible that it got merged when picked up.
+		 */
+		if (op->type == PLAYER)
+		    esrv_del_item(op->contr, tmp_count);
 	    }
 	    break;
 	}
@@ -248,6 +259,8 @@ int steal(object* op, int dir, object *skill)
        */ 
       if (tmp->head) tmp=tmp->head;
       if(tmp->type!=PLAYER&&!QUERY_FLAG(tmp, FLAG_MONSTER)) continue;
+      /* do not reveal hidden DMs */
+      if (tmp->type == PLAYER && QUERY_FLAG(tmp, FLAG_WIZ) && tmp->contr->hidden) continue;
       if (attempt_steal(tmp, op, skill)) {
 	  if(tmp->type==PLAYER) /* no xp for stealing from another player */
 	    return 0;
@@ -428,7 +441,8 @@ static int attempt_jump (object *pl, int dir, int spaces, object *skill) {
 
 	for(tmp=get_map_ob(m, x, y); tmp;tmp=tmp->above) { 
 	    /* Jump into creature */ 
-	    if(QUERY_FLAG(tmp,FLAG_MONSTER) || tmp->type==PLAYER ) {   
+	    if(QUERY_FLAG(tmp, FLAG_MONSTER)
+	    || (tmp->type==PLAYER && (!QUERY_FLAG(tmp, FLAG_WIZ) || !tmp->contr->hidden))) {
 		new_draw_info_format(NDI_UNIQUE, 0,pl,"You jump into %s%s.", 
 		    tmp->type == PLAYER ? "" : "the ", tmp->name);
 		if(tmp->type!=PLAYER || 
@@ -510,7 +524,8 @@ static int do_skill_detect_curse(object *pl, object *skill) {
     int success=0;
  
     for(tmp=pl->inv;tmp;tmp=tmp->below)
-	if (!QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_CURSED)
+	if (!tmp->invisible
+            && !QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_CURSED)
             && (QUERY_FLAG(tmp,FLAG_CURSED) || QUERY_FLAG(tmp,FLAG_DAMNED)) &&
 	    tmp->item_power < skill->level) {
 		SET_FLAG(tmp,FLAG_KNOWN_CURSED);
@@ -518,9 +533,11 @@ static int do_skill_detect_curse(object *pl, object *skill) {
 		success+= calc_skill_exp(pl,tmp, skill);
 	}
 
-    /* Check ground, too */
+    /* Check ground, too, but only objects the player could pick up */
     for(tmp=get_map_ob(pl->map,pl->x,pl->y);tmp;tmp=tmp->above)
-	if (!QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_CURSED)
+	if (can_pick(pl, tmp) &&
+	    !QUERY_FLAG(tmp,FLAG_IDENTIFIED) && 
+	    !QUERY_FLAG(tmp,FLAG_KNOWN_CURSED)
             && (QUERY_FLAG(tmp,FLAG_CURSED) || QUERY_FLAG(tmp,FLAG_DAMNED)) &&
 	    tmp->item_power < skill->level) {
 		SET_FLAG(tmp,FLAG_KNOWN_CURSED);
@@ -536,16 +553,19 @@ static int do_skill_detect_magic(object *pl, object *skill) {
     int success=0;
 
     for(tmp=pl->inv;tmp;tmp=tmp->below)
-        if(!QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)
+        if(!tmp->invisible
+	    && !QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)
 	    && (is_magical(tmp)) && tmp->item_power < skill->level) { 
             	SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
 		esrv_update_item(UPD_FLAGS, pl, tmp);
 		success+=calc_skill_exp(pl,tmp, skill);
 	}
 
-    /* Check ground, too */
+    /* Check ground, too, but like above, only if the object can be picked up*/
     for(tmp=get_map_ob(pl->map,pl->x,pl->y);tmp;tmp=tmp->above)
-        if(!QUERY_FLAG(tmp,FLAG_IDENTIFIED) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)
+        if (can_pick(pl, tmp) && 
+	    !QUERY_FLAG(tmp,FLAG_IDENTIFIED) && 
+	    !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)
 	    && (is_magical(tmp)) && tmp->item_power < skill->level) { 
             	SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
 		esrv_update_item(UPD_FLAGS, pl, tmp);
@@ -631,7 +651,7 @@ int skill_ident(object *pl, object *skill) {
 
 	case SK_ALCHEMY:
 	    success += do_skill_ident(pl,POTION,skill) + do_skill_ident(pl,POISON,skill)
-		    + do_skill_ident(pl,AMULET,skill) + do_skill_ident(pl,CONTAINER,skill)
+		    + do_skill_ident(pl,CONTAINER,skill)
 		    + do_skill_ident(pl,DRINK,skill) + do_skill_ident(pl,INORGANIC,skill);
 	    break;
 
@@ -641,7 +661,8 @@ int skill_ident(object *pl, object *skill) {
 	    break; 
 
 	case SK_JEWELER:
-	    success += do_skill_ident(pl,GEM,skill) + do_skill_ident(pl,RING,skill);
+	    success += do_skill_ident(pl,GEM,skill) + do_skill_ident(pl,RING,skill) +
+		do_skill_ident(pl,AMULET,skill);
 	    break; 
 
 	case SK_LITERACY:
@@ -668,7 +689,7 @@ int skill_ident(object *pl, object *skill) {
 	    break;
 
 	default:
-	    LOG(llevError,"Error: bad call to skill_ident()");
+	    LOG(llevError,"Error: bad call to skill_ident()\n");
 	    return 0;
 	    break;
     }
@@ -738,7 +759,7 @@ int use_oratory(object *pl, int dir, object *skill) {
     if(QUERY_FLAG(tmp,FLAG_FRIENDLY)&&(tmp->move_type==PETMOVE)){  
 	if(get_owner(tmp)==pl) {
 	    new_draw_info(NDI_UNIQUE, 0,pl, 
-		      "Your follower loves your speach.\n");
+		      "Your follower loves your speech.\n");
 	    return 0;
 	} else if (skill->level > tmp->level) {
 	    /* you steal the follower.  Perhaps we should really look at the
@@ -776,7 +797,7 @@ int use_oratory(object *pl, int dir, object *skill) {
     /* Charm failed.  Creature may be angry now */
     else if((skill->level+((pl->stats.Cha-10)/2)) < random_roll(1, 2*tmp->level, pl, PREFER_LOW)) {
 	new_draw_info_format(NDI_UNIQUE, 0,pl, 
-		  "Your speach angers the %s!\n",query_name(tmp)); 
+		  "Your speech angers the %s!\n",query_name(tmp)); 
 	if(QUERY_FLAG(tmp,FLAG_FRIENDLY)) {
 	    CLEAR_FLAG(tmp,FLAG_FRIENDLY);
 	    remove_friendly_object(tmp);
@@ -1077,15 +1098,17 @@ static int write_note(object *pl, object *item, char *msg, object *skill) {
     if ((evt = find_event(item, EVENT_TRIGGER)) != NULL) {
 	CFParm CFP;
 	int k, l, m;
+	uint32 n;
 	k = EVENT_TRIGGER;
 	l = SCRIPT_FIX_ALL;
 	m = 0;
+	n = 0;
 	CFP.Value[0] = &k;
 	CFP.Value[1] = pl;
 	CFP.Value[2] = item;
 	CFP.Value[3] = NULL;
 	CFP.Value[4] = msg;
-	CFP.Value[5] = &m;
+	CFP.Value[5] = &n;
 	CFP.Value[6] = &m;
 	CFP.Value[7] = &m;
 	CFP.Value[8] = &l;
@@ -1151,13 +1174,13 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
 	      "You need a spell readied in order to inscribe!"); 
 	return 0; 
     }
-    if(chosen_spell->stats.grace > pl->stats.grace) {
+    if(SP_level_spellpoint_cost(pl,chosen_spell,SPELL_GRACE) > pl->stats.grace) {
        	new_draw_info_format(NDI_UNIQUE,0,pl,
 	     "You don't have enough grace to write a scroll of %s.", 
 			     chosen_spell->name);
 	return 0;
     }
-    if(chosen_spell->stats.sp > pl->stats.sp) {
+    if(SP_level_spellpoint_cost(pl,chosen_spell,SPELL_MANA) > pl->stats.sp) {
        	new_draw_info_format(NDI_UNIQUE,0,pl,
 	     "You don't have enough mana to write a scroll of %s.", 
 			     chosen_spell->name);
@@ -1180,8 +1203,8 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
     if(QUERY_FLAG(pl,FLAG_CONFUSED)) confused = 1;
 
     /* Lost mana/grace no matter what */
-    pl->stats.grace-=chosen_spell->stats.grace;
-    pl->stats.sp-=chosen_spell->stats.sp;
+    pl->stats.grace-=SP_level_spellpoint_cost(pl,chosen_spell,SPELL_GRACE);
+    pl->stats.sp-=SP_level_spellpoint_cost(pl,chosen_spell,SPELL_MANA);
 
     if (random_roll(0, chosen_spell->level*4-1, pl, PREFER_LOW) < skill->level) {
 	if (scroll->nrof > 1) {
@@ -1230,6 +1253,7 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
 	esrv_send_item(pl, newscroll);
 	success = calc_skill_exp(pl,newscroll, skill);
 	if(!confused) success *= 2;
+    success = success * skill->level;
 	return success; 
 
     } else { /* Inscription has failed */
@@ -1627,15 +1651,17 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
     {
         CFParm CFP;
         int k, l, m;
+        uint32 n;
         k = EVENT_THROW;
         l = SCRIPT_FIX_ACTIVATOR;
         m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = op;
         CFP.Value[2] = throw_ob;
         CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
