@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_attack_c =
- *   "$Id: spell_attack.c,v 1.10 2004/04/04 16:37:02 temitchell Exp $";
+ *   "$Id: spell_attack.c,v 1.20 2005/07/19 20:59:34 ryo_saeba Exp $";
  */
 
 
@@ -53,7 +53,7 @@ void check_spell_knockback(object *op) {
 	int frictionmod=2; /*poor man's physics - multipy targets weight by this amount */
 
 	if(! op->weight) { /*shouldn't happen but if cone object has no weight drop out*/
-		/*LOG (llevDebug, "DEBUG: arch weighs nothing");*/
+		/*LOG (llevDebug, "DEBUG: arch weighs nothing\n");*/
 		return;
 	}else{
 	   weight_move = op->weight +(op->weight * op->level) / 3;
@@ -363,7 +363,7 @@ void explosion(object *op) {
 void explode_bullet(object *op)
 {
     tag_t op_tag = op->count;
-    object *tmp;
+    object *tmp, *owner;
 
     if (op->other_arch == NULL) {
 	LOG (llevError, "BUG: explode_bullet(): op without other_arch\n");
@@ -405,9 +405,9 @@ void explode_bullet(object *op)
     if (tmp->skill) FREE_AND_CLEAR_STR(tmp->skill);
     if (op->skill) tmp->skill = add_refcount(op->skill);
 
-
-    if ((tmp->attacktype & AT_HOLYWORD || tmp->attacktype & AT_GODPOWER) &&
-      !tailor_god_spell(tmp, op)) {
+    owner = get_owner(op);
+    if ((tmp->attacktype & AT_HOLYWORD || tmp->attacktype & AT_GODPOWER) && owner &&
+      !tailor_god_spell(tmp, owner)) {
 	remove_ob (op);
 	free_object (op);
 	return;
@@ -427,6 +427,10 @@ void explode_bullet(object *op)
 	tmp->stats.dam = op->dam_modifier;
 	tmp->range = op->stats.maxhp;
 	tmp->duration = op->stats.hp;
+	/* Used for spell tracking - just need a unique val for this spell -
+	 * the count of the parent should work fine.
+	 */
+	tmp->stats.maxhp = op->count;
     }
 
     /* Set direction of cone explosion */
@@ -1154,7 +1158,7 @@ void move_missile(object *op) {
  * Destruction
  ****************************************************************************/
 /* make_object_glow() - currently only makes living objects glow.
- * we do this by creating a "force" and inserting it in the 
+ * we do this by creating a force and inserting it in the 
  * object. if time is 0, the object glows permanently. To truely
  * make this work for non-living objects, we would have to 
  * give them the capability to have an inventory. b.t.
@@ -1167,7 +1171,7 @@ int make_object_glow(object *op, int radius, int time) {
     if(op->path_denied&PATH_LIGHT) 
 	return 0;
 
-    tmp=get_archetype("force");
+    tmp=get_archetype(FORCE_NAME);
     tmp->speed = 0.01;
     tmp->stats.food = time;
     SET_FLAG(tmp, FLAG_IS_USED_UP);
@@ -1216,6 +1220,8 @@ int cast_destruction(object *op, object *caster, object *spell_ob) {
     else if (caster->skill) op->skill = caster->skill;
     else op->skill = NULL;
 
+    change_skill(op,  find_skill_by_name(op, op->skill), 1);
+
     for(i= -range; i<range; i++) {
 	for(j=-range; j<range ; j++) {
 	    m = op->map;
@@ -1293,7 +1299,7 @@ int cast_curse(object *op, object *caster, object *spell_ob, int dir) {
     }
 
     if(force==NULL) {
-	force=get_archetype("force");
+	force=get_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1351,7 +1357,7 @@ int cast_curse(object *op, object *caster, object *spell_ob, int dir) {
  * effects monsters, it seems best to put it into this file
  ***********************************************************************/
 
-/* This covers the variosu spells that change the moods of monsters -
+/* This covers the various spells that change the moods of monsters -
  * makes them angry, peacful, friendly, etc.
  */
 int mood_change(object *op, object *caster, object *spell) {
@@ -1391,7 +1397,7 @@ int mood_change(object *op, object *caster, object *spell) {
 	    /* If there is nothing living on this space, no need to go further */
 	    if (!(mflags & P_IS_ALIVE)) continue;
 
-	    for (tmp = get_map_ob(m, x, y); tmp; tmp = tmp->above)
+	    for (tmp = get_map_ob(m, nx, ny); tmp; tmp = tmp->above)
 		if (QUERY_FLAG(tmp, FLAG_MONSTER)) break;
 
 	    /* There can be living objects that are not monsters */
@@ -1411,14 +1417,34 @@ int mood_change(object *op, object *caster, object *spell) {
 		for (at=0; at < NROFATTACKS; at++)
 		    if (spell->attacktype & (1 << at))
 			if (best_at == -1 || head->resist[at] > head->resist[best_at]) best_at = at;
+
+            if (best_at == -1) at=0;
+	        else {
+		        if (head->resist[best_at] == 100) continue;
+		        else at = head->resist[best_at] / 5;
+	        }
+	        at -= level / 5;
+	        if (did_make_save(head, head->level, at)) continue;
 	    }
-	    if (best_at == -1) at=0;
-	    else {
-		if (head->resist[best_at] == 100) continue;
-		else at = head->resist[best_at] / 5;
-	    }
-	    at -= level / 5;
-	    if (did_make_save(head, head->level, at)) continue;
+        else    /* spell->attacktype */
+            /*
+                Spell has no attacktype (charm & such), so we'll have a specific saving:
+                 * if spell level < monster level, no go
+                 * else, chance of effect = 20 + min( 50, 2 * ( spell level - monster level ) )
+
+                The chance will then be in the range [20-70] percent, not too bad.
+
+                This is required to fix the 'charm monster' abuse, where a player level 1 can
+                charm a level 125 monster...
+
+                Ryo, august 14th
+            */
+            {
+            if ( head->level > level ) continue;
+            if ( random_roll( 0, 100, caster, PREFER_LOW ) >= ( 20 + MIN( 50, 2 * ( level - head->level ) ) ) )
+                /* Failed, no effect */
+                continue;
+            }
 
 	    /* Done with saving throw.  Now start effecting the monster */
 
@@ -1447,12 +1473,17 @@ int mood_change(object *op, object *caster, object *spell) {
 	    /* charm */
 	    if (QUERY_FLAG(spell, FLAG_NO_ATTACK) && !QUERY_FLAG(head, FLAG_FRIENDLY)) {
 		SET_FLAG(head, FLAG_FRIENDLY);
-		set_owner(head, op);
+        /* Prevent uncontolled outbreaks of self replicating monsters.
+           Typical use case is charm, go somwhere, use aggravation to make hostile.
+           This could lead to fun stuff like mice outbreak in bigworld and server crawl. */
+        CLEAR_FLAG(head, FLAG_GENERATOR);
+        set_owner(head, op);
 		set_spell_skill(op, caster, spell, head);
 		add_friendly_object(head);
-		head->stats.exp = 0;
 		head->move_type = PETMOVE;
 		done_one = 1;
+		change_exp(op, head->stats.exp / 2, spell->skill, SK_EXP_ADD_SKILL);
+		head->stats.exp = 0;
 	    }
 
 	    /* If a monster was effected, put an effect in */
@@ -1850,7 +1881,7 @@ int cast_cause_disease(object *op, object *caster, object *spell, int dir) {
 			 new_draw_info_format(NDI_UNIQUE, 0, op, "You inflict %s on %s!",disease->name,walk->name);
 
 			 free_object(disease); /* don't need this one anymore */
-			 flash=get_archetype("detect_magic");
+			 flash=get_archetype(ARCH_DETECT_MAGIC);
 			 flash->x = x;
 			 flash->y = y;
 			 flash->map = walk->map;
