@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_effect_c =
- *   "$Id: spell_effect.c,v 1.117 2004/04/25 07:17:40 mwedel Exp $";
+ *   "$Id: spell_effect.c,v 1.131 2005/08/10 13:38:31 ryo_saeba Exp $";
  */
 
 
@@ -48,6 +48,13 @@ void cast_magic_storm(object *op, object *tmp, int lvl)
     tmp->y=op->y; 
     tmp->range+=lvl/5;  /* increase the area of destruction */
     tmp->duration+=lvl/5;
+
+    /* Put a cap on duration for this - if the player fails in their
+     * apartment, don't want it to go on so long that it kills them
+     * multiple times.  Also, damge already increases with level,
+     * so don't really need to increase the duration as much either.
+     */
+    if (tmp->duration>=40) tmp->duration=40;
     tmp->stats.dam=lvl; /* nasty recoils! */
     tmp->stats.maxhp=tmp->count; /* tract single parent */ 
     insert_ob_in_map(tmp,op->map,op,0);
@@ -407,7 +414,7 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
 
 int cast_create_missile(object *op, object *caster,object *spell, int dir, char *stringarg)
 {
-    int missile_plus=0;
+    int missile_plus=0, bonus_plus=0;
     char *missile_name;
     object *tmp, *missile;
     tag_t tag;
@@ -421,23 +428,51 @@ int cast_create_missile(object *op, object *caster,object *spell, int dir, char
 
     missile_plus = spell->stats.dam  + SP_level_dam_adjust(caster, spell);
 
+    if (find_archetype(missile_name)==NULL) {
+	LOG(llevDebug, "Cast create_missile: could not find archetype %s\n", 
+	    missile_name);
+	return 0;
+    }
+    missile = get_archetype(missile_name);
+
     if (stringarg) {
-	if (atoi(stringarg) < missile_plus)
-	    missile_plus = atoi(stringarg);
+	/* If it starts with a letter, presume it is a description */
+	if (isalpha(*stringarg)) {
+	    artifact *al = find_artifactlist(missile->type)->items;
+
+	    for ( ; al != NULL; al=al->next) 
+		if (!strcasecmp(al->item->name, stringarg)) break;
+
+	    if (!al) {
+		free_object(missile);
+		new_draw_info_format(NDI_UNIQUE, 0, op ,"No such object %ss of %s", missile_name,
+				     stringarg);
+		return 0;
+	    }
+	    if (al->item->slaying) {
+		free_object(missile);
+		new_draw_info_format(NDI_UNIQUE, 0, op ,"You are not allowed to create %ss of %s", 
+				     missile_name, stringarg);
+		return 0;
+	    }
+	    give_artifact_abilities(missile, al->item);
+	    /* These special arrows cost something extra.  Don't have them also be magical -
+	     * otherwise, in most cases, not enough will be created.  I don't want to get into
+	     * the parsing of having to do both plus and type.
+	     */
+	    bonus_plus = 1 + (al->item->value / 5);
+	    missile_plus=0;
+	} else
+	    if (atoi(stringarg) < missile_plus)
+		missile_plus = atoi(stringarg);
     }
     if (missile_plus > 4)
 	missile_plus = 4;
     else if (missile_plus < -4)
 	missile_plus = -4;
 
-    if (find_archetype(missile_name)==NULL) {
-	LOG(llevDebug, "Cast create_missile: could not find archtype %s\n", 
-	    missile_name);
-	return 0;
-    }
-    missile = get_archetype(missile_name);
     missile->nrof = spell->duration + SP_level_duration_adjust(caster, spell);
-    missile->nrof -= 3 * missile_plus;
+    missile->nrof -= 3 * (missile_plus + bonus_plus);
     if (missile->nrof < 1)
 	missile->nrof=1;
 
@@ -469,7 +504,7 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, char
               + 50 * SP_level_duration_adjust(caster,spell_ob);
 
     if(stringarg) {
-	at=find_archetype(stringarg);
+	at=find_archetype_by_object_name(stringarg);
 	if (at==NULL || ((at->clone.type != FOOD && at->clone.type != DRINK)
 	    || (at->clone.stats.food > food_value)))
 		stringarg = NULL;
@@ -678,7 +713,7 @@ void execute_word_of_recall(object *op) {
     while(op!=NULL && op->type!=PLAYER)
 	op=op->env;
 
-    if(op!=NULL) {
+    if(op!=NULL && op->map) {
 	if (get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL) & P_NO_CLERIC)
 	    new_draw_info(NDI_UNIQUE, 0,op,"You feel something fizzle inside you.");
 	else
@@ -699,7 +734,7 @@ int cast_word_of_recall(object *op, object *caster, object *spell_ob) {
     if(op->type!=PLAYER)
 	return 0;
 
-    dummy=get_archetype("force");
+    dummy=get_archetype(FORCE_NAME);
     if(dummy == NULL){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"cast_word_of_recall: get_archetype(force) failed!\n");
@@ -753,7 +788,7 @@ int cast_wonder(object *op, object *caster, int dir, object *spell_ob) {
 	}
 	/* Prevent inifinit recursion */
 	if (newspell->subtype == SP_WONDER) {
-	    LOG(llevError,"cast_wonder: spell returned is another wonder spell!");
+	    LOG(llevError,"cast_wonder: spell returned is another wonder spell!\n");
 	    return 0;
 	}
 	return cast_spell(op,caster,dir,newspell, NULL);
@@ -764,7 +799,7 @@ int cast_wonder(object *op, object *caster, int dir, object *spell_ob) {
 
 int perceive_self(object *op) {
     char *cp=describe_item(op, op), buf[MAX_BUF];
-    archetype *at=find_archetype("depletion");
+    archetype *at=find_archetype(ARCH_DEPLETION);
     object *tmp;
     int i;
 
@@ -927,7 +962,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 	}
 	remove_ob (old_force);
 	free_object (old_force);
-	LOG (llevDebug,"\n",old_force->name);
+	LOG (llevDebug,"\n");
     }
     free_object (dummy);
 
@@ -957,12 +992,12 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 
     op_level = caster_level(caster, spell);
     if (op_level<15)
-         snprintf (portal_message,1024,"\nAir moves around you and\na huge smell of ammoniac\nrounds you as you pass\nthrough %s's portal\nPouah!\n",op->name);
+         snprintf (portal_message,1024,"\nThe air moves around you and\na huge smell of ammonia\nsurounds you as you pass\nthrough %s's tiny portal\nPouah!\n",op->name);
     else if (op_level<30)
-         snprintf (portal_message,1024,"\n%s's portal smells ozone.\nYou do a lot of movements and finally pass\nthrough the small hole in the air\n",op->name);
+         snprintf (portal_message,1024,"\n%s's portal smells of ozone.\nYou do a lot of movements and finally pass\nthrough the small hole in the air\n",op->name);
     else if (op_level<60)
-         snprintf (portal_message,1024,"\nA sort of door opens in the air in front of you,\nshowing you the path to somewhere else.\n");
-    else snprintf (portal_message,1024,"\nAs you walk on %s's portal, flowers comes\nfrom the ground around you.\nYou feel quiet.\n",op->name);
+         snprintf (portal_message,1024,"\nA shining door opens in the air in front of you,\nshowing you the path to another place.\n");
+    else snprintf (portal_message,1024,"\nAs you walk through %s's portal, flowers come out\nfrom the ground around you.\nYou feel awed.\n",op->name);
 
     /* Create a portal in front of player
      * dummy contain the portal and
@@ -1135,6 +1170,9 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	new_draw_info_format(NDI_UNIQUE, 0,op,"Something destroys your %s", name);
 	return 0;
     }
+    /* If this is a spellcasting wall, need to insert the spell object */
+    if (tmp->other_arch && tmp->other_arch->clone.type == SPELL)
+	insert_ob_in_ob(arch_to_object(tmp->other_arch), tmp);
 
     /*  This code causes the wall to extend some distance in
      * each direction, or until an obstruction is encountered. 
@@ -1153,6 +1191,7 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	 
 	x = tmp->x+i*freearr_x[dir2]; 
 	y = tmp->y+i*freearr_y[dir2];
+	m = tmp->map;
 
 	if(!(get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP | P_BLOCKED)) && !posblocked) {
 	    tmp2 = get_object();
@@ -1160,6 +1199,10 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	    tmp2->x = x;
 	    tmp2->y = y;
 	    insert_ob_in_map(tmp2,m,op,0);
+	    /* If this is a spellcasting wall, need to insert the spell object */
+	    if (tmp2->other_arch && tmp2->other_arch->clone.type == SPELL)
+		insert_ob_in_ob(arch_to_object(tmp2->other_arch), tmp2);
+
 	} else posblocked=1;
 
 	x = tmp->x-i*freearr_x[dir2];
@@ -1172,6 +1215,8 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	    tmp2->x = x;
 	    tmp2->y = y;
 	    insert_ob_in_map(tmp2,m,op,0);
+	    if (tmp2->other_arch && tmp2->other_arch->clone.type == SPELL)
+		insert_ob_in_ob(arch_to_object(tmp2->other_arch), tmp2);
 	} else negblocked=1;
     }
 
@@ -1305,7 +1350,7 @@ int cast_heal(object *op,object *caster, object *spell, int dir) {
 
     if (heal) {
 	if (tmp->stats.hp >= tmp->stats.maxhp) {
-	    new_draw_info(NDI_UNIQUE, 0,tmp, "Your are already fully healed.");
+	    new_draw_info(NDI_UNIQUE, 0,tmp, "You are already fully healed.");
 	}
 	else {
 	    /* See how many points we actually heal.  Instead of messages
@@ -1427,7 +1472,7 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir) {
 	}
     }
     if(force==NULL) {
-	force=get_archetype("force");
+	force=get_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1542,7 +1587,7 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
 	}
     }
     if(force==NULL) {
-	force=get_archetype("force");
+	force=get_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1648,7 +1693,7 @@ static void alchemy_object(object *obj, int *small_nuggets,
     else if (obj->type==MONEY || obj->type==GEM)
 	value /=3;
     else
-	value *= 0.9;
+	value = (value*9)/10;
 
     if ((obj->value>0) && rndm(0, 29)) {
 	int count;
@@ -2014,7 +2059,7 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
     if (QUERY_FLAG(spell, FLAG_KNOWN_CURSED) || QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL)) {
 	done_one = 0;
 	for (tmp = op->inv; tmp; tmp = tmp->below) {
-	    if (!QUERY_FLAG(tmp, FLAG_IDENTIFIED)) {
+	    if (!tmp->invisible && !QUERY_FLAG(tmp, FLAG_IDENTIFIED)) {
 		if (QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL) && 
 		    is_magical(tmp) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)) {
 			SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
@@ -2211,7 +2256,7 @@ int cast_consecrate(object *op, object *caster, object *spell) {
 
 	    if(tmp->level > caster_level(caster, spell)) {
 		new_draw_info_format(NDI_UNIQUE, 0,op,
-		    "You are not poweful enough to reconsecrate the %s", tmp->name);
+		    "You are not powerful enough to reconsecrate the %s", tmp->name);
 		return 0;
 	    } else {
 		/* If we got here, we are consecrating an altar */
