@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_util_c =
- *   "$Id: spell_util.c,v 1.86 2004/04/18 07:59:36 mwedel Exp $";
+ *   "$Id: spell_util.c,v 1.93 2005/07/15 08:02:00 ryo_saeba Exp $";
  */
 
 
@@ -244,7 +244,9 @@ int SP_level_spellpoint_cost(object *caster, object *spell, int flags)
 		 (float)(level-spell->level)/ (float)spell->stats.maxsp)));
 	}
 	else sp = spell->stats.sp;
+
 	sp *= PATH_SP_MULT(caster,spell);
+	if (!sp && spell->stats.sp) sp=1;
 
 	if (spell->stats.grace && spell->stats.maxgrace) {
 	    grace= (int) (spell->stats.grace * 
@@ -252,10 +254,14 @@ int SP_level_spellpoint_cost(object *caster, object *spell, int flags)
 		 (float)(level-spell->level)/ (float)spell->stats.maxgrace)));
 	}
 	else grace = spell->stats.grace;
+
 	grace *= PATH_SP_MULT(caster,spell);
+	if (spell->stats.grace && !grace) grace=1;
     } else {
 	sp = spell->stats.sp * PATH_SP_MULT(caster,spell);
+	if (spell->stats.sp && !sp) sp=1;
 	grace = spell->stats.grace * PATH_SP_MULT(caster,spell);
+	if (spell->stats.grace && !grace) grace=1;
     }
     if (flags == SPELL_HIGHEST)
 	return MAX(sp, grace);
@@ -323,7 +329,7 @@ int SP_level_range_adjust(object *caster, object *spob)
  * as an existing spell, we presume they know it.
  * returns 1 if they know the spell, 0 if they don't.
  */
-object *check_spell_known (object *op, char *name)
+object *check_spell_known (object *op, const char *name)
 {
     object *spop;
 
@@ -343,7 +349,7 @@ object *check_spell_known (object *op, char *name)
  * exact match, we also return NULL.
  */
 
-object *lookup_spell_by_name(object *op,char *spname) {
+object *lookup_spell_by_name(object *op,const char *spname) {
     object *spob1=NULL, *spob2=NULL, *spob;
     int nummatch=0;
 
@@ -503,6 +509,10 @@ int fire_arch_from_position (object *op, object *caster, sint16 x, sint16 y,
     if (mflags & P_OUT_OF_MAP) {
 	return 0;
     }
+    if (mflags & P_NO_PASS) {
+	new_draw_info(NDI_UNIQUE, 0, op, "You can't cast the spell on top of a wall!\n");
+	return 0;
+    }
 
     tmp=arch_to_object(spell->other_arch);
 
@@ -897,6 +907,36 @@ void spell_failure(object *op, int failure,int power, object *skill)
     }
 }
 
+int cast_party_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg)
+    {
+    int success;
+    player *pl;
+    object *spell;
+
+    if ( !spell_ob->other_arch )
+        {
+        LOG( llevError, "cast_party_spell: empty other arch\n" );
+        return 0;
+        }
+    spell = arch_to_object( spell_ob->other_arch );
+
+    /* Always cast spell on caster */
+    success = cast_spell( op, caster, dir, spell, stringarg );
+
+    if ( caster->contr->party_number == -1 )
+        {
+        remove_ob( spell );
+        return success;
+        }
+    for( pl=first_player; pl!=NULL; pl=pl->next )
+        if( ( pl->ob->contr->party_number == caster->contr->party_number ) && ( on_same_map( pl->ob, caster ) ) )
+            {
+            cast_spell( pl->ob, caster, pl->ob->facing, spell, stringarg );
+            }
+    remove_ob( spell );
+    return success;
+    }
+
 /* This is where the main dispatch when someone casts a spell.
  *
  * op is the creature that is owner of the object that is casting the spell -
@@ -912,7 +952,7 @@ void spell_failure(object *op, int failure,int power, object *skill)
  *    parameters to a spell (eg, item to create, information for marking runes,
  *    etc.
  * returns 1 on successful cast, or 0 on error.  These values should really
- * be swapped, so that 0 is sucessful, and non zero is failure, with a code
+ * be swapped, so that 0 is successful, and non zero is failure, with a code
  * of what it failed.
  *
  * Note that this function is really a dispatch routine that calls other
@@ -993,7 +1033,8 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 				     "%s grants your prayer, though you are unworthy.",godname);
 		}
 		else {
-		    prayer_failure(op,op->stats.grace,SP_level_spellpoint_cost(caster,spell_ob, SPELL_GRACE));
+		    prayer_failure(op,op->stats.grace,
+			   SP_level_spellpoint_cost(caster,spell_ob, SPELL_GRACE) - op->stats.grace);
 		    new_draw_info_format(NDI_UNIQUE, 0,op,"%s ignores your prayer.",godname);
 		    return 0;
 		}
@@ -1007,7 +1048,8 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 		if (settings.casting_time == TRUE) {
 		    op->casting_time = -1;
 		}
-		return(random_roll(1, SP_level_spellpoint_cost(caster,spell_ob, SPELL_GRACE), op, PREFER_LOW));
+		op->stats.grace -= random_roll(1, SP_level_spellpoint_cost(caster,spell_ob, SPELL_GRACE), op, PREFER_LOW);
+		return 0;
 	    } else if (spell_ob->stats.sp) {
 		int failure = random_roll(0, 199, op, PREFER_HIGH) -
 		    op->contr->encumbrance +op->level - spell_ob->level +35;
@@ -1019,7 +1061,8 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 			  SP_level_spellpoint_cost(caster,spell_ob, SPELL_MANA),
 			  skill);
 		    op->contr->shoottype = old_shoottype;
-		    return(random_roll(0, SP_level_spellpoint_cost(caster,spell_ob, SPELL_MANA), op, PREFER_LOW));
+		    op->stats.sp -= random_roll(0, SP_level_spellpoint_cost(caster,spell_ob, SPELL_MANA), op, PREFER_LOW);
+		    return 0;
 		}
 	    }
 	}
@@ -1342,6 +1385,9 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 	    success= cast_create_town_portal (op,caster,spell_ob, dir);
 	    break;
 
+    case SP_PARTY_SPELL:
+        success = cast_party_spell( op, caster, dir, spell_ob, stringarg );
+
 	default:
 	    LOG(llevError,"cast_spell: Unhandled spell subtype %d\n",
 		spell_ob->subtype);
