@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.61 2004/04/10 03:18:54 temitchell Exp $";
+ *    "$Id: time.c,v 1.71 2005/07/19 20:59:35 ryo_saeba Exp $";
  */
 
 /*
@@ -600,15 +600,17 @@ static void stop_arrow (object *op)
     {
         CFParm CFP;
         int k, l, m;
+        uint32 n;
         k = EVENT_STOP;
         l = SCRIPT_FIX_NOTHING;
         m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = NULL;
         CFP.Value[2] = op;
         CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
@@ -649,8 +651,19 @@ void move_arrow(object *op) {
 
     /* we need to stop thrown objects at some point. Like here. */ 
     if(op->type==THROWN_OBJ) {
-        if (op->inv == NULL)
+	/* If the object that the THROWN_OBJ encapsulates disappears,
+	 * we need to have this object go away also - otherwise, you get
+	 * left over remnants on the map.  Where this currently happens
+	 * is if the player throws a bomb - the bomb explodes on its own,
+	 * but this object sticks around.  We could handle the cleanup in the
+	 * bomb code, but there are potential other cases where that could happen,
+	 * and it is easy enough to clean it up here.
+	 */
+        if (op->inv == NULL) {
+	    remove_ob(op);
+	    free_object(op);
             return;
+	}
 	if(op->last_sp-- < 0) { 
 	    stop_arrow (op);
 	    return; 
@@ -842,6 +855,7 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
 void move_teleporter(object *op) {
     object *tmp, *head=op;
     event *evt;
+
     /* if this is a multipart teleporter, handle the other parts
      * The check for speed isn't strictly needed - basically, if
      * there is an old multipart teleporter in which the other parts
@@ -860,38 +874,37 @@ void move_teleporter(object *op) {
     if (!tmp) return;
 
     if(EXIT_PATH(head)) {
-      if(op->above->type==PLAYER)
-      {
-        /* GROS: Handle for plugin TRIGGER event */
-        if ((evt = find_event(op, EVENT_TRIGGER)) != NULL)
-        {
-          CFParm CFP;
-          CFParm* CFR;
-          int k, l, m;
-          int rtn_script = 0;
-          m = 0;
-          k = EVENT_TRIGGER;
-          l = SCRIPT_FIX_ALL;
-          CFP.Value[0] = &k;
-          CFP.Value[1] = op;
-          CFP.Value[2] = op->above;
-          CFP.Value[3] = NULL;
-          CFP.Value[4] = NULL;
-          CFP.Value[5] = &m;
-          CFP.Value[6] = &m;
-          CFP.Value[7] = &m;
-          CFP.Value[8] = &l;
-          CFP.Value[9] = evt->hook;
-          CFP.Value[10]= evt->options;
-          if (findPlugin(evt->plugin)>=0)
-          {
-            CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-            rtn_script = *(int *)(CFR->Value[0]);
-          }
-          if (rtn_script!=0) return;
-        }
-        enter_exit(op->above, head);
-      }
+	if(op->above->type==PLAYER) {
+	    /* GROS: Handle for plugin TRIGGER event */
+	    if ((evt = find_event(op, EVENT_TRIGGER)) != NULL) {
+		CFParm CFP;
+		CFParm* CFR;
+		int k, l, m;
+		uint32 n;
+		int rtn_script = 0;
+		m = 0;
+		n = 0;
+		k = EVENT_TRIGGER;
+		l = SCRIPT_FIX_ALL;
+		CFP.Value[0] = &k;
+		CFP.Value[1] = op;
+		CFP.Value[2] = op->above;
+		CFP.Value[3] = NULL;
+		CFP.Value[4] = NULL;
+		CFP.Value[5] = &n;
+		CFP.Value[6] = &m;
+		CFP.Value[7] = &m;
+		CFP.Value[8] = &l;
+		CFP.Value[9] = evt->hook;
+		CFP.Value[10]= evt->options;
+		if (findPlugin(evt->plugin)>=0) {
+		    CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
+		    rtn_script = *(int *)(CFR->Value[0]);
+		}
+		if (rtn_script!=0) return;
+	    }
+	    enter_exit(op->above, head);
+	}
 	else
 	    /* Currently only players can transfer maps */
 	    return;
@@ -904,13 +917,15 @@ void move_teleporter(object *op) {
 	    return;
 	}
 	/* GROS: Handle for plugin TRIGGER event */
-    if ((evt = find_event(op, EVENT_TRIGGER)) != NULL)
+	if ((evt = find_event(op, EVENT_TRIGGER)) != NULL)
 	{
 		CFParm CFP;
 		CFParm* CFR;
 		int k, l, m;
+		uint32 n;
 		int rtn_script = 0;
 		m = 0;
+		n = 0;
 		k = EVENT_TRIGGER;
 		l = SCRIPT_FIX_ALL;
 		CFP.Value[0] = &k;
@@ -918,7 +933,7 @@ void move_teleporter(object *op) {
 		CFP.Value[2] = op->above;
 		CFP.Value[3] = NULL;
 		CFP.Value[4] = NULL;
-		CFP.Value[5] = &m;
+		CFP.Value[5] = &n;
 		CFP.Value[6] = &m;
 		CFP.Value[7] = &m;
 		CFP.Value[8] = &l;
@@ -933,39 +948,38 @@ void move_teleporter(object *op) {
 	}
 	transfer_ob(tmp,EXIT_X(head),EXIT_Y(head),0,head);
     }
-    else
-    {
-		/* Random teleporter */
-		/* GROS: Handle for plugin TRIGGER event */
-        if ((evt = find_event(op, EVENT_TRIGGER)) != NULL)
-		{
-			CFParm CFP;
-			CFParm* CFR;
-			int k, l, m;
-			int rtn_script = 0;
-			m = 0;
-			k = EVENT_TRIGGER;
-			l = SCRIPT_FIX_ALL;
-			CFP.Value[0] = &k;
-			CFP.Value[1] = op;
-			CFP.Value[2] = op->above;
-			CFP.Value[3] = NULL;
-			CFP.Value[4] = NULL;
-			CFP.Value[5] = &m;
-			CFP.Value[6] = &m;
-			CFP.Value[7] = &m;
-			CFP.Value[8] = &l;
-			CFP.Value[9] = evt->hook;
-			CFP.Value[10]= evt->options;
-			if (findPlugin(evt->plugin)>=0)
-			{
-				CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-				rtn_script = *(int *)(CFR->Value[0]);
-			}
-			if (rtn_script!=0) return;
-		}
-		teleport(head, TELEPORTER, tmp);
+    else {
+	/* Random teleporter */
+	/* GROS: Handle for plugin TRIGGER event */
+        if ((evt = find_event(op, EVENT_TRIGGER)) != NULL) {
+	    CFParm CFP;
+	    CFParm* CFR;
+	    int k, l, m;
+	    uint32 n;
+	    int rtn_script = 0;
+	    m = 0;
+	    n = 0;
+	    k = EVENT_TRIGGER;
+	    l = SCRIPT_FIX_ALL;
+	    CFP.Value[0] = &k;
+	    CFP.Value[1] = op;
+	    CFP.Value[2] = op->above;
+	    CFP.Value[3] = NULL;
+	    CFP.Value[4] = NULL;
+	    CFP.Value[5] = &n;
+	    CFP.Value[6] = &m;
+	    CFP.Value[7] = &m;
+	    CFP.Value[8] = &l;
+	    CFP.Value[9] = evt->hook;
+	    CFP.Value[10]= evt->options;
+	    if (findPlugin(evt->plugin)>=0) {
+		CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
+		rtn_script = *(int *)(CFR->Value[0]);
+	    }
+	    if (rtn_script!=0) return;
 	}
+	teleport(head, TELEPORTER, tmp);
+    }
 }
 
 
@@ -992,9 +1006,11 @@ void move_player_changer(object *op) {
 	    CFParm CFP;
 	    CFParm* CFR;
 	    int k, l, m;
+	    uint32 n;
 	    int rtn_script = 0;
 
 	    m = 0;
+	    n = 0;
 	    k = EVENT_TRIGGER;
 	    l = SCRIPT_FIX_NOTHING;
 	    CFP.Value[0] = &k;
@@ -1002,7 +1018,7 @@ void move_player_changer(object *op) {
 	    CFP.Value[2] = op->above;
 	    CFP.Value[3] = NULL;
 	    CFP.Value[4] = NULL;
-	    CFP.Value[5] = &m;
+	    CFP.Value[5] = &n;
 	    CFP.Value[6] = &m;
 	    CFP.Value[7] = &m;
 	    CFP.Value[8] = &l;
@@ -1031,7 +1047,7 @@ void move_player_changer(object *op) {
 	    player->contr->bed_y = EXIT_Y(op);
 	}
 	else
-	    LOG(llevDebug, "WARNING: destination '%s' in player_changer must be an absolute path!",
+	    LOG(llevDebug, "WARNING: destination '%s' in player_changer must be an absolute path!\n",
 		EXIT_PATH(op));
 	
 	enter_exit(op->above,op);
@@ -1074,6 +1090,8 @@ void move_firewall(object *op) {
 void move_player_mover(object *op) {
     object *victim, *nextmover;
     int dir = op->stats.sp;
+    sint16 nx, ny;
+    mapstruct *m;
 
     /* Determine direction now for random movers so we do the right thing */
     if (!dir) dir=rndm(1, 8);
@@ -1088,8 +1106,16 @@ void move_player_mover(object *op) {
 		free_object(op);
 		return;
 	    }
+	    nx = op->x+freearr_x[dir];
+	    ny = op->y+freearr_y[dir];
+	    m = op->map;
+	    if (get_map_flags(m, &m, nx, ny, &nx, &ny) & P_OUT_OF_MAP) {
+		LOG(llevError,"move_player_mover: Trying to push player off the map! map=%s (%d, %d)\n",
+		    m->path, op->x, op->y);
+		return ;
+	    }
 
-	    for(nextmover=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]); nextmover !=NULL; nextmover=nextmover->above) {
+	    for(nextmover=get_map_ob(m,nx, ny); nextmover !=NULL; nextmover=nextmover->above) {
 		if(nextmover->type == PLAYERMOVER) 
 		    nextmover->speed_left=-.99;
 		if(QUERY_FLAG(nextmover,FLAG_ALIVE)) {
@@ -1155,29 +1181,63 @@ void move_duplicator(object *op) {
     }
 }
 
-/*  move_creator (by peterm) 
-  it has the creator object create it's other_arch right on top of it.
-  connected:  what will trigger it
-  hp:  how many times it may create before stopping
-  lifesave:  if set, it'll never disappear but will go on creating
-	everytime it's triggered
-  other_arch:  the object to create
+/* move_creator (by peterm) 
+ * it has the creator object create it's other_arch right on top of it.
+ * connected:  what will trigger it
+ * hp:  how many times it may create before stopping
+ * lifesave:  if set, it'll never disappear but will go on creating
+ *    everytime it's triggered
+ * other_arch:  the object to create
+ * Note this can create large objects, however, in that case, it
+ * has to make sure that there is in fact space for the object.
+ * It should really do this for small objects also, but there is
+ * more concern with large objects, most notably a part being placed
+ * outside of the map which would cause the server to crash
 */
 
 void move_creator(object *op) {
-  object *tmp;
-  op->stats.hp--;
-  if(op->stats.hp < 0 && !QUERY_FLAG(op,FLAG_LIFESAVE)) 
-	{ op->stats.hp=-1;return;}
-  tmp=arch_to_object(op->other_arch);
-  if(op->slaying) {
-	 if (tmp->name) free_string (tmp->name);
-	 if (tmp->title) free_string (tmp->title);
-	 tmp->name = add_string(op->slaying);
-	 tmp->title = add_string(op->slaying);
-  }
-  tmp->x=op->x;tmp->y=op->y;tmp->map=op->map;tmp->level=op->level;
-  insert_ob_in_map(tmp,op->map,op,0);
+    object *tmp,*head=NULL,*prev=NULL;
+    archetype *at=op->other_arch;
+
+    op->stats.hp--;
+    if(op->stats.hp < 0 && !QUERY_FLAG(op,FLAG_LIFESAVE)) {
+	op->stats.hp=-1;
+	return;
+    }
+    if (!at) {
+	LOG(llevError,"move_creator: Creator doesn't have other arch set: %s (%s, %d, %d)\n",
+	    op->name?op->name:"(null)", op->map->path, op->x, op->y);
+	return;
+    }
+    /* Make sure this multipart object fits */
+    if (at->more && arch_blocked(at, op->map, op->x, op->y))
+	return;
+
+    while(at!=NULL) {
+	tmp=arch_to_object(at);
+	tmp->x=op->x+at->clone.x;
+	tmp->y=op->y+at->clone.y;
+	tmp->map=op->map;
+	tmp->level=op->level;
+
+	if(head!=NULL)
+	    tmp->head=head,prev->more=tmp;
+
+	insert_ob_in_map(tmp,op->map,op,0);
+	if (QUERY_FLAG(tmp, FLAG_FREED)) return;
+
+	if(head==NULL)
+	    head=tmp;
+	prev=tmp;
+	at=at->more;
+
+	if(op->slaying) {
+	    if (tmp->name) free_string (tmp->name);
+	    if (tmp->title) free_string (tmp->title);
+	    tmp->name = add_string(op->slaying);
+	    tmp->title = add_string(op->slaying);
+	}
+    }
 }
 
 /* move_marker --peterm@soda.csua.berkeley.edu
@@ -1195,7 +1255,7 @@ void move_marker(object *op) {
   for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above) {
 	 
 
-    if(tmp->type == PLAYER) { /* we've got someone to MARK */
+    if(tmp->type == PLAYER && quest_marker_compatible( op, tmp ) ) { /* we've got someone to MARK */
 
 		/* remove an old force with a slaying field == op->name */
       for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
@@ -1214,7 +1274,7 @@ void move_marker(object *op) {
       /* if we didn't find our own MARK */
       if(tmp2==NULL) {
 	         
-		  object *force = get_archetype("force");
+		  object *force = get_archetype(FORCE_NAME);
 		  force->speed = 0;
 		  if(op->stats.food) {
 			 force->speed = 0.01;
@@ -1223,6 +1283,9 @@ void move_marker(object *op) {
 		  update_ob_speed (force);
 		  /* put in the lock code */
 		  force->slaying = add_string(op->slaying);
+          if ( op->lore )
+              force->lore = add_string( op->lore );
+          quest_clear_markers( op, tmp );
 		  insert_ob_in_ob(force,tmp);
 		  if(op->msg)
 		    new_draw_info(NDI_UNIQUE|NDI_NAVY,0,tmp,op->msg);
@@ -1285,15 +1348,17 @@ int process_object(object *op) {
     {
 	CFParm CFP;
 	int k, l, m;
+	uint32 n;
 	k = EVENT_TIME;
 	l = SCRIPT_FIX_NOTHING;
 	m = 0;
+	n = 0;
 	CFP.Value[0] = &k;
 	CFP.Value[1] = NULL;
 	CFP.Value[2] = op;
 	CFP.Value[3] = NULL;
 	CFP.Value[4] = NULL;
-	CFP.Value[5] = &m;
+	CFP.Value[5] = &n;
 	CFP.Value[6] = &m;
 	CFP.Value[7] = &m;
 	CFP.Value[8] = &l;
