@@ -40,6 +40,7 @@ void cftimer_process_event(object* ob)
 {
     CFParm CFP;
     int k, l, m;
+    uint32 n;
     event *evt;
 
     if ((evt = find_event(ob, EVENT_TIMER)) != NULL)
@@ -47,12 +48,13 @@ void cftimer_process_event(object* ob)
         k = EVENT_TIMER;
         l = SCRIPT_FIX_ALL;
         m = 0;
+        n = 0;
         CFP.Value[0] = &k;
         CFP.Value[1] = ob;
         CFP.Value[2] = NULL;
         CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &n;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
