@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_weather_c =
- *   "$Id: weather.c,v 1.35 2003/09/26 06:12:07 mwedel Exp $";
+ *   "$Id: weather.c,v 1.40 2005/07/17 11:00:05 ryo_saeba Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -936,7 +936,7 @@ void init_humid_elev()
 {
     int x, y, tx, ty, nx, ny, ax, ay, j;
     int spwtx, spwty;
-    char *mapname;
+    const char *mapname;
     long int elev;
     int water, space;
     mapstruct *m;
@@ -969,11 +969,12 @@ void init_humid_elev()
 		     space < spwtx*spwty); ax++,nx++) {
 		for (ny=0,ay=ty; (ny < spwty && ay < settings.worldmaptilesizey &&
 			 space < spwtx*spwty);
-		     ay++,ny++,space++) {
-		    if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
-			water++;
-		    elev += GET_MAP_OB(m, ax, ay)->elevation;
-		}
+		     ay++,ny++,space++)
+		    if(GET_MAP_OB(m, ax, ay)){
+		    	if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
+				water++;
+		    	elev += GET_MAP_OB(m, ax, ay)->elevation;
+		    }
 	    }
 	    delete_map(m);
 
@@ -990,11 +991,12 @@ void init_humid_elev()
 		     space < spwtx*spwty); ax++,nx++) {
 		for (ny=j,ay=MAX(0, ty - (spwty-1)); (ny < spwty && ay <= ty &&
 			 space < spwtx*spwty);
-		     space++,ay++,ny++) {
-		    if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
-			water++;
-		    elev += GET_MAP_OB(m, ax, ay)->elevation;
-		}
+		     space++,ay++,ny++)
+		    if(GET_MAP_OB(m, ax, ay)){
+		    	if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
+				water++;
+		    	elev += GET_MAP_OB(m, ax, ay)->elevation;
+			}
 	    }
 	    delete_map(m);
 
@@ -1010,11 +1012,12 @@ void init_humid_elev()
 		     space < spwtx*spwty); ax++,nx++) {
 		for (ny=0,ay=ty; (ny < spwty && ay < settings.worldmaptilesizey &&
 			 space < spwtx*spwty);
-		     ay++,ny++,space++) {
-		    if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
-			water++;
-		    elev += GET_MAP_OB(m, ax, ay)->elevation;
-		}
+		     ay++,ny++,space++)
+		    if(GET_MAP_OB(m, ax, ay)){
+		    	if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
+				water++;
+		    	elev += GET_MAP_OB(m, ax, ay)->elevation;
+			}
 	    }
 	    delete_map(m);
 
@@ -1030,11 +1033,12 @@ void init_humid_elev()
 		     space < spwtx*spwty); ax++,nx++) {
 		for (ny=0,ay=MAX(0, ty - (spwty-1)); (ny < spwty && ay <= ty &&
 			 space < spwtx*spwty);
-		     space++,ay++,ny++) {
-		    if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
-			water++;
-		    elev += GET_MAP_OB(m, ax, ay)->elevation;
-		}
+		     space++,ay++,ny++)
+		    if(GET_MAP_OB(m, ax, ay)){
+		    	if (QUERY_FLAG(GET_MAP_OB(m, ax, ay), FLAG_IS_WATER))
+				water++;
+		    	elev += GET_MAP_OB(m, ax, ay)->elevation;
+			}
 	    }
 	    delete_map(m);
 	    /* jesus thats confusing as all hell */
@@ -1078,7 +1082,7 @@ void read_temperaturemap()
     int x, y;
 
     sprintf(filename, "%s/temperaturemap", settings.localdir);
-    LOG(llevDebug, "Reading teperature data from %s...", filename);
+    LOG(llevDebug, "Reading temperature data from %s...", filename);
     if ((fp = fopen(filename, "r")) == NULL) {
 	LOG(llevError, "Cannot open %s for reading\n", filename);
 	init_temperature();
@@ -1342,7 +1346,7 @@ void perform_weather()
  * occur.
  */
 
-void weather_effect(char *filename)
+void weather_effect(const char *filename)
 {
     mapstruct *m;
     int wx, wy, x, y;
@@ -1358,23 +1362,23 @@ void weather_effect(char *filename)
     x = 0;
     y = 0;
     /* for now, just bail if it's not the worldmap */
-    if (worldmap_to_weathermap(x, y, &wx, &wy, /*filename*/ m) != 0)
+    if (worldmap_to_weathermap(x, y, &wx, &wy, m) != 0)
 	return;
 	/*First, calculate temperature*/
 	calculate_temperature(m, wx, wy);
     /* we change the world first, if needed */
     if (settings.dynamiclevel >= 5) {
-	change_the_world(m, wx, wy, filename);
+	change_the_world(m, wx, wy);
     }
     if (settings.dynamiclevel >= 2) {
-	let_it_snow(m, wx, wy, filename);
-	singing_in_the_rain(m, wx, wy, filename);
+	let_it_snow(m, wx, wy);
+	singing_in_the_rain(m, wx, wy);
 	}
     /*    if (settings.dynamiclevel >= 4) {
 	    feather_map(m, wx, wy, filename);
 	    }*/
     if (settings.dynamiclevel >= 3) {
-	plant_a_garden(m, wx, wy, filename);
+	plant_a_garden(m, wx, wy);
     }
 }
 
@@ -1405,7 +1409,7 @@ object *avoid_weather(int *av, mapstruct *m, int x, int y, int *gs, int grow)
 		    !(tmp->material & M_ICE || tmp->material & M_LIQUID))
 		    gotsnow++;
 	    for (i=0; growth_avoids[i].name != NULL; i++) {
-		//if (!strcmp(tmp->arch->name, growth_avoids[i].name)) {
+		/*if (!strcmp(tmp->arch->name, growth_avoids[i].name)) {*/
 		if (tmp->arch== growth_avoids[i].what) {
 		    avoid++;
 		    break;
@@ -1421,7 +1425,7 @@ object *avoid_weather(int *av, mapstruct *m, int x, int y, int *gs, int grow)
     } else {
 	for (tmp=GET_MAP_OB(m, x, y); tmp; tmp = tmp->above) {
 	    for (i=0; weather_avoids[i].name != NULL; i++) {
-		//if (!strcmp(tmp->arch->name, weather_avoids[i].name)) {
+		/*if (!strcmp(tmp->arch->name, weather_avoids[i].name)) {*/
 		if (tmp->arch == weather_avoids[i].what) {
 		    if (weather_avoids[i].snow == 1)
 			gotsnow++;
@@ -1458,7 +1462,7 @@ void calculate_temperature(mapstruct *m, int wx, int wy){
  * weather_effect()
  */
 
-void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
+void let_it_snow(mapstruct *m, int wx, int wy)
 {
     int x, y, i;
     int nx, ny, j, d;
@@ -1487,7 +1491,7 @@ void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
 		y = ny;
 	    }
 	    /* we use the unjittered coordinates */
-	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, /*filename*/ m);
+	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, m);
 	    ob = NULL;
 	    at = NULL;
 	    /* this will definately need tuning */
@@ -1495,7 +1499,7 @@ void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
 	    two = 0;
 	    gotsnow = 0;
 	    nodstk = 0;
-	    //temp = real_world_temperature(x, y, m);
+	    /*temp = real_world_temperature(x, y, m);*/
 		temp = weathermap[wx][wy].realtemp;
 	    sky = weathermap[wx][wy].sky;
 	    if (temp <= 0 && sky > SKY_OVERCAST && sky < SKY_FOG)
@@ -1619,11 +1623,11 @@ void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
 			remove_ob(tmp);
 			free_object(tmp);
 			tmp=GET_MAP_OB(m, x, y);
-				if (!strcmp(tmp->arch->name, "mountain")){
+				if (tmp &&(!strcmp(tmp->arch->name, "mountain"))){
 					at = find_archetype("mountain1_rivlets");}
-				else if (!strcmp(tmp->arch->name, "mountain2")){
+				else if ( tmp && (!strcmp(tmp->arch->name, "mountain2"))){
 					at = find_archetype("mountain2_rivlets");}
-				else if (!strcmp(tmp->arch->name, "mountain4")){
+				else if (tmp && (!strcmp(tmp->arch->name, "mountain4"))){
 					at = find_archetype("mountain2_rivlets");}
 				else {at = find_archetype("rain5");}
 			if (at != NULL) {
@@ -1647,13 +1651,13 @@ void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
 			avoid--;
 		}
 		tmp = GET_MAP_OB(m, x, y);
-		if (!strcasecmp(tmp->name, "sea"))
+		if (tmp && (!strcasecmp(tmp->name, "sea")))
 		    avoid++;
-		else if (!strcasecmp(tmp->name, "sea1"))
+		else if (tmp && (!strcasecmp(tmp->name, "sea1")))
 		    avoid++;
-		else if (!strcasecmp(tmp->name, "deep_sea"))
+		else if (tmp && (!strcasecmp(tmp->name, "deep_sea")))
 		    avoid++;
-		else if (!strcasecmp(tmp->name, "shallow_sea"))
+		else if (tmp && (!strcasecmp(tmp->name, "shallow_sea")))
 		    avoid++;
 		if (avoid > 0) {
 		    at = find_archetype("ice");
@@ -1676,7 +1680,7 @@ void let_it_snow(mapstruct *m, int wx, int wy, char *filename)
  * weather_effect()
  */
 
-void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename)
+void singing_in_the_rain(mapstruct *m, int wx, int wy)
 {
     int x, y, i;
     int nx, ny, d, j;
@@ -1705,14 +1709,14 @@ void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename)
 		y = ny;
 	    }
 	    /* we use the unjittered coordinates */
-	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, /*filename*/m);
+	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, m);
 	    ob = NULL;
 	    at = NULL;
 	    avoid = 0;
 	    two = 0;
 	    gotsnow = 0;
 	    nodstk = 0;
-	    //temp = real_world_temperature(x, y, m);
+	    /*temp = real_world_temperature(x, y, m);*/
 		temp = weathermap[wx][wy].realtemp;
 	    sky = weathermap[wx][wy].sky;
 	    /* it's probably allready snowing */
@@ -1721,11 +1725,11 @@ void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename)
 	    oldsnow = avoid_weather(&avoid, m, x, y, &gotsnow, 0);
 	    if (!avoid) {
 		tmp=GET_MAP_OB(m, x, y);
-		if (!strcmp(tmp->arch->name, "mountain")){
+		if (tmp && (!strcmp(tmp->arch->name, "mountain"))){
 			at = find_archetype("mountain1_rivlets"); break;}
-		else if (!strcmp(tmp->arch->name, "mountain2")){
+		else if (tmp && (!strcmp(tmp->arch->name, "mountain2"))){
 			at = find_archetype("mountain2_rivlets"); break;}
-		else if (!strcmp(tmp->arch->name, "mountain4")){
+		else if (tmp && (!strcmp(tmp->arch->name, "mountain4"))){
 			at = find_archetype("mountain2_rivlets"); break;}
 		if (sky == SKY_LIGHT_RAIN || sky == SKY_RAIN) {		
 			switch (rndm(0, SKY_HAIL-sky)) {
@@ -1832,7 +1836,7 @@ void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename)
 		}
 	    }
 	    /* Things evaporate fast in the heat */
-	    if (temp > 8 && sky < SKY_OVERCAST && rndm(temp, 60) > 50) {
+	    if (GET_MAP_OB(m, x, y) && temp > 8 && sky < SKY_OVERCAST && rndm(temp, 60) > 50) {
 		/* evaporate */
 		for (tmp=GET_MAP_OB(m, x, y)->above; tmp; tmp = tmp->above) {
 		    avoid = 0;
@@ -1896,7 +1900,7 @@ void singing_in_the_rain(mapstruct *m, int wx, int wy, char *filename)
  * weather_effect()
  */
 
-void plant_a_garden(mapstruct *m, int wx, int wy, char *filename)
+void plant_a_garden(mapstruct *m, int wx, int wy)
 {
     int x, y, i;
     int avoid, two, temp, sky, gotsnow, found, days;
@@ -1906,13 +1910,13 @@ void plant_a_garden(mapstruct *m, int wx, int wy, char *filename)
     days = todtick / HOURS_PER_DAY;
     for (x=0; x < settings.worldmaptilesizex; x++) {
 	for (y=0; y < settings.worldmaptilesizey; y++) {
-	    (void)worldmap_to_weathermap(x, y, &wx, &wy, /*filename*/m);
+	    (void)worldmap_to_weathermap(x, y, &wx, &wy, m);
 	    ob = NULL;
 	    at = NULL;
 	    avoid = 0;
 	    two = 0;
 	    gotsnow = 0;
-	    //temp = real_world_temperature(x, y, m);
+	    /*temp = real_world_temperature(x, y, m);*/
 		temp = weathermap[wx][wy].realtemp;
 	    sky = weathermap[wx][wy].sky;
 	    (void)avoid_weather(&avoid, m, x, y, &gotsnow, 1);
@@ -1965,7 +1969,8 @@ void plant_a_garden(mapstruct *m, int wx, int wy, char *filename)
 		    if (temp < weather_grow[i].tempmin ||
 			temp > weather_grow[i].tempmax)
 			continue;
-		    if (GET_MAP_OB(m, x, y)->elevation < weather_grow[i].elevmin ||
+		    if ((!GET_MAP_OB(m, x, y)) ||
+		    	GET_MAP_OB(m, x, y)->elevation < weather_grow[i].elevmin ||
 			GET_MAP_OB(m, x, y)->elevation > weather_grow[i].elevmax)
 			continue;
 		    /* we got this far.. must be a match */
@@ -1991,11 +1996,10 @@ void plant_a_garden(mapstruct *m, int wx, int wy, char *filename)
  * Process worldmap regrowth.  m is the map we are currently processing.
  * wx and wy are
  * the weathermap coordinates for the weathermap square we want to work on.
- * filename is the pathname for the current map.  This should be called from
- * weather_effect()
+ * This should be called from weather_effect()
  */
 
-void change_the_world(mapstruct *m, int wx, int wy, char *filename)
+void change_the_world(mapstruct *m, int wx, int wy)
 {
     int x, y, i;
     int nx, ny, j, d;
@@ -2024,14 +2028,14 @@ void change_the_world(mapstruct *m, int wx, int wy, char *filename)
 		y = ny;
 	    }
 	    /* we use the unjittered coordinates */
-	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, /*filename*/m);
+	    (void)worldmap_to_weathermap(nx, ny, &wx, &wy, m);
 	    ob = NULL;
 	    at = NULL;
 	    dat = NULL;
 	    avoid = 0;
 	    two = 0;
 	    gotsnow = 0;
-	    //temp = real_world_temperature(x, y, m);
+	    /*temp = real_world_temperature(x, y, m);*/
 		temp = weathermap[wx][wy].realtemp;
 	    sky = weathermap[wx][wy].sky;
 	    (void)avoid_weather(&avoid, m, x, y, &gotsnow, 1);
@@ -2039,7 +2043,8 @@ void change_the_world(mapstruct *m, int wx, int wy, char *filename)
 		for (i=0; weather_tile[i].herb != NULL; i++) {
 		    found=0;
 		    doublestack=NULL;
-		    for (tmp=GET_MAP_OB(m, x, y)->above; tmp; tmp = tmp->above) {
+		    if (GET_MAP_OB(m, x, y))
+		      for (tmp=GET_MAP_OB(m, x, y)->above; tmp; tmp = tmp->above) {
 			if (weather_tile[i].tile != NULL)
 			    if (strcmp(tmp->arch->name,
 				       weather_tile[i].tile) == 0) {
@@ -2080,18 +2085,19 @@ void change_the_world(mapstruct *m, int wx, int wy, char *filename)
 		    if (temp < weather_tile[i].tempmin ||
 			temp > weather_tile[i].tempmax)
 			continue;
-		    if (GET_MAP_OB(m, x, y)->elevation < weather_tile[i].elevmin ||
+		    if ( (!GET_MAP_OB(m, x, y)) ||
+		        GET_MAP_OB(m, x, y)->elevation < weather_tile[i].elevmin ||
 			GET_MAP_OB(m, x, y)->elevation > weather_tile[i].elevmax)
 			continue;
 		    /* we got this far.. must be a match */
-		    if (strcmp(GET_MAP_OB(m, x, y)->arch->name,
+		    if (GET_MAP_OB(m, x, y) && strcmp(GET_MAP_OB(m, x, y)->arch->name,
 			       weather_tile[i].herb) == 0)
 			break; /* no sense in doubling up */
 		    at = find_archetype(weather_tile[i].herb);
 		    break;
 		}
 		if (at != NULL) {
-		    if (weather_tile[i].tile != NULL &&
+		    if (weather_tile[i].tile != NULL && GET_MAP_OB(m, x, y) &&
 			strcmp(weather_tile[i].tile,
 			       GET_MAP_OB(m, x, y)->arch->name) != 0)
 			dat = find_archetype(weather_tile[i].tile);
@@ -2126,11 +2132,10 @@ void change_the_world(mapstruct *m, int wx, int wy, char *filename)
  * Reduce the blockiness of the maps. m is the map we are currently processing.
  * wx and wy are
  * the weathermap coordinates for the weathermap square we want to work on.
- * filename is the pathname for the current map.  This should be called from
- * weather_effect()
+ * This should be called from weather_effect()
  */
 
-void feather_map(mapstruct *m, int wx, int wy, char *filename)
+void feather_map(mapstruct *m, int wx, int wy)
 {
     int x, y, i, nx, ny, j;
     int avoid, two, gotsnow, nodstk;
@@ -2139,7 +2144,7 @@ void feather_map(mapstruct *m, int wx, int wy, char *filename)
 
     for (x=0; x < settings.worldmaptilesizex; x++) {
 	for (y=0; y < settings.worldmaptilesizey; y++) {
-	    (void)worldmap_to_weathermap(x, y, &wx, &wy, /*filename*/m);
+	    (void)worldmap_to_weathermap(x, y, &wx, &wy, m);
 	    ob = NULL;
 	    at = NULL;
 	    avoid = 0;
@@ -2226,7 +2231,6 @@ void feather_map(mapstruct *m, int wx, int wy, char *filename)
    map.  returns -1 if you give it something it can't figure out. 0 normally.
 */
 
-//int worldmap_to_weathermap(int x, int y, int *wx, int *wy, char *filename)
 int worldmap_to_weathermap(int x, int y, int *wx, int *wy, mapstruct* m)
 {
     int spwtx, spwty;
@@ -2274,11 +2278,11 @@ int worldmap_to_weathermap(int x, int y, int *wx, int *wy, mapstruct* m)
  * value is the name of the map that corner resides in.
  */
 
-char *weathermap_to_worldmap_corner(int wx, int wy, int *x, int *y, int dir)
+const char *weathermap_to_worldmap_corner(int wx, int wy, int *x, int *y, int dir)
 {
     int spwtx, spwty;
     int tx, ty, nx, ny;
-    char *mapname;
+	static char mapname[ MAX_BUF ];
     
     spwtx = (settings.worldmaptilesx * settings.worldmaptilesizex) / WEATHERMAPTILESX;
     spwty = (settings.worldmaptilesy * settings.worldmaptilesizey) / WEATHERMAPTILESY;
@@ -2299,12 +2303,11 @@ char *weathermap_to_worldmap_corner(int wx, int wy, int *x, int *y, int dir)
     
     nx = (tx / settings.worldmaptilesizex) + settings.worldmapstartx;
     ny = (ty / settings.worldmaptilesizey) + settings.worldmapstarty;
-    mapname = malloc(sizeof(char) * strlen("world/world_0000_0000"));
-    sprintf(mapname, "world/world_%d_%d", nx, ny);
-    
+    snprintf(mapname, MAX_BUF, "world/world_%d_%d", nx, ny);
+
     *x = tx%settings.worldmaptilesizex;
     *y = ty%settings.worldmaptilesizey;
-    return(mapname);
+    return mapname;
 }
 
 /*
@@ -2458,7 +2461,7 @@ int real_world_temperature(int x, int y, mapstruct *m)
 {
     int wx, wy, temp, eleva, elevb;
     object *op;
-	//printf ("real_world_temperature: worldmaptoweathermap : %s\n",m->path);
+	/*printf ("real_world_temperature: worldmaptoweathermap : %s\n",m->path);*/
     worldmap_to_weathermap(x, y, &wx, &wy, /*m->path*/m);
     temp = real_temperature(wx, wy);
     if (weathermap[wx][wy].avgelev < 0)
@@ -2970,8 +2973,8 @@ void write_weather_images()
 		pixels[3 * x + (1 * WEATHERMAPTILESX * 3 + BLUE)]  = speed;
 	    } else {
 		speed = (speed - realmaxwind) * realscalewind;
-//		if (speed < 100)
-//		    speed = 100;
+/*		if (speed < 100)*/
+/*		    speed = 100;*/
 		pixels[3 * x + (1 * WEATHERMAPTILESX * 3 + RED)]   = speed;
 		pixels[3 * x + (1 * WEATHERMAPTILESX * 3 + GREEN)] = 0;
 		pixels[3 * x + (1 * WEATHERMAPTILESX * 3 + BLUE)]  = 0;
