@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: image.c,v 1.5 2003/12/14 16:28:47 ryo_saeba Exp $";
+ *    "$Id: image.c,v 1.8 2005/07/14 09:18:57 akirschbaum Exp $";
  */
 
 /*
@@ -327,7 +327,7 @@ void esrv_send_face(NewSocket *ns,short face_num, int nocache)
     SockList sl;
     char fallback;
 
-    if (face_num < 0 || face_num >= nrofpixmaps) {
+    if (face_num <= 0 || face_num >= nrofpixmaps) {
 	LOG(llevError,"esrv_send_face (%d) out of bounds??\n",face_num);
 	return;
     }
@@ -373,7 +373,7 @@ void esrv_send_face(NewSocket *ns,short face_num, int nocache)
 	sl.len += facesets[fallback].faces[face_num].datalen;
 	Send_With_Handling(ns, &sl);
     }
-    ns->faces_sent[face_num] = 1;
+    ns->faces_sent[face_num] |= NS_FACESENT_FACE;
     free(sl.buf);
 }
 
@@ -438,7 +438,7 @@ void send_image_sums(NewSocket *ns, char *params)
 
     for (i=start; i<=stop; i++) {
 	SockList_AddShort(&sl, i);
-	ns->faces_sent[i] = 1;
+	ns->faces_sent[i] |= NS_FACESENT_FACE;
 
 	qq = get_face_fallback(ns->faceset, i);
 	SockList_AddInt(&sl, facesets[qq].faces[i].checksum);
@@ -456,7 +456,7 @@ void send_image_sums(NewSocket *ns, char *params)
      * support.
      */
     if (sl.len > MAXSOCKBUF) {
-	LOG(llevError,"send_image_send: buffer overrun, %s > %s\n", sl.len, MAXSOCKBUF);
+	LOG(llevError,"send_image_send: buffer overrun, %d > %d\n", sl.len, MAXSOCKBUF);
 	abort();
     }
     Send_With_Handling(ns, &sl);
