@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sock_info_c =
- *   "$Id: info.c,v 1.17 2004/01/18 10:36:10 ryo_saeba Exp $";
+ *   "$Id: info.c,v 1.24 2005/05/29 17:35:54 tchize Exp $";
  */
 
 /*
@@ -53,14 +53,31 @@ static void esrv_print_msg(NewSocket *ns,int color, const char *str)
     char buf[HUGE_BUF];
 
     if (ns->status == Ns_Old) {
-	sprintf(buf,"%s\n", str);
+	snprintf(buf,HUGE_BUF,"%s\n", str);
     } else {
-	sprintf(buf,"drawinfo %d %s", color, str);
+	snprintf(buf,HUGE_BUF, "drawinfo %d %s", color, str);
     }
-/*    LOG(llevDebug,"sending %s to socket, len=%d", buf, strlen(buf));*/
+/*    LOG(llevDebug,"sending %s to socket, len=%d\n", buf, strlen(buf));*/
     Write_String_To_Socket(ns, buf, strlen(buf));
 }
 
+/**
+ * Draws   an extended message on the client. 
+ * ns      the socket to send message to
+ * color   color informations (used mainly if client does not support message type)
+ * type,
+ * subtype type and subtype of text message
+ * intro   Intro message to send with main message if client does not support the message type
+ * message The main message
+ */
+static void esrv_print_ext_msg(NewSocket *ns,int color,uint8 type, uint8 subtype, const char *message)
+{
+    char buf[HUGE_BUF];
+    snprintf(buf,HUGE_BUF, "drawextinfo %d %hhu %hhu %s", color, type, subtype, message);
+	Write_String_To_Socket(ns, buf, strlen(buf));
+/*    LOG(llevDebug,"sending %s to socket, len=%d", buf, strlen(buf));*/
+
+}
 
 /**
  * Frontend for esrv_print_msg
@@ -100,7 +117,7 @@ void flush_output_element(object *pl, Output_Buf *outputs)
     char tbuf[MAX_BUF];
 
     if (outputs->buf==NULL) return;
-    sprintf(tbuf,"%d times %s", outputs->count, outputs->buf);
+    snprintf(tbuf,MAX_BUF, "%d times %s", outputs->count, outputs->buf);
     print_message(NDI_BLACK, pl, tbuf);
     free_string(outputs->buf);
     outputs->buf=NULL;
@@ -122,7 +139,7 @@ void flush_output_element(object *pl, Output_Buf *outputs)
  * and adds message to queue.
  */
 
-void check_output_buffers(object *pl, char *buf)
+void check_output_buffers(object *pl, const char *buf)
 {
     int i, oldest=0;
 
@@ -207,7 +224,7 @@ void new_draw_info(int flags,int pri, object *pl, const char *buf)
 
     if ((flags&NDI_COLOR_MASK)==NDI_BLACK && !(flags &NDI_UNIQUE)) {
 	/* following prints stuff out, as appropriate */
-	check_output_buffers(pl, (char*)buf);
+	check_output_buffers(pl, buf);
     }
     else {
 	print_message(flags&NDI_COLOR_MASK, pl, buf);
@@ -223,25 +240,76 @@ void new_draw_info(int flags,int pri, object *pl, const char *buf)
  * client/server bandwidth (client could keep track of various strings
  */
 
-void new_draw_info_format(int flags, int pri,object *pl, char *format, ...)
+void new_draw_info_format(int flags, int pri,object *pl, const char *format, ...)
 {
     char buf[HUGE_BUF];
 
     va_list ap;
     va_start(ap, format);
 
-    vsprintf(buf, format, ap);
+    vsnprintf(buf, HUGE_BUF, format, ap);
 
     va_end(ap);
 
     new_draw_info(flags, pri, pl, buf);
 }
 
+
+void draw_ext_info(
+        int flags, int pri, object *pl, uint8 type, 
+        uint8 subtype, const char* message, const char* oldmessage){
+            
+    if(!pl || (pl->type!=PLAYER) || (pl->contr==NULL))
+        return;
+        
+    if (pri>=pl->contr->listening) return;
+    if (!CLIENT_SUPPORT_READABLES(&pl->contr->socket,type)){
+        char *buf = (char*)malloc(strlen(oldmessage==NULL?message:oldmessage)+1);
+        if (buf==NULL)
+            LOG(llevError,"info::draw_ext_info -> Out of memory!");
+        else{
+            strcpy(buf,oldmessage==NULL?message:oldmessage);
+            strip_media_tag(buf);
+            new_draw_info(flags, pri, pl, buf);
+            free(buf);
+        }
+    }else{
+        esrv_print_ext_msg(&pl->contr->socket,flags&NDI_COLOR_MASK,type,subtype,message);
+    }
+}
+
+void draw_ext_info_format(
+        int flags, int pri, object *pl, uint8 type, 
+        uint8 subtype, const char* old_format, 
+        char* new_format, ...){
+            
+    char buf[HUGE_BUF];
+	if(!pl || (pl->type!=PLAYER) || (pl->contr==NULL))
+		return;
+		
+    if (pri>=pl->contr->listening) return;
+	if (!CLIENT_SUPPORT_READABLES(&pl->contr->socket,type)){
+        va_list ap;
+        LOG(llevDebug,"Non supported extension text type for client.\n");
+        va_start(ap, new_format);
+        vsnprintf(buf, HUGE_BUF, old_format, ap);
+        va_end(ap);
+        new_draw_info(flags, pri, pl, buf);
+		return;
+	}else{
+        va_list ap;
+        va_start(ap, new_format);
+        vsnprintf(buf, HUGE_BUF, new_format, ap);
+        va_end(ap);
+        strip_media_tag(buf);
+        esrv_print_ext_msg(&pl->contr->socket,flags&NDI_COLOR_MASK,type,subtype,buf);
+    }
+}
 /**
  * Writes to everyone on the map *except* op.  This is useful for emotions.
  */
 
-void new_info_map_except(int color, mapstruct *map, object *op, char *str) {
+void new_info_map_except(int color, mapstruct *map, object *op, const char *str) {
     player *pl;
 
     for(pl = first_player; pl != NULL; pl = pl->next)
@@ -255,7 +323,7 @@ void new_info_map_except(int color, mapstruct *map, object *op, char *str) {
  */
 
 void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
-			  char *str) {
+			  const char *str) {
     player *pl;
 
     for(pl = first_player; pl != NULL; pl = pl->next)
@@ -269,7 +337,7 @@ void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
  * Writes to everyone on the specified map
  */
 
-void new_info_map(int color, mapstruct *map, char *str) {
+void new_info_map(int color, mapstruct *map, const char *str) {
     player *pl;
 
     for(pl = first_player; pl != NULL; pl = pl->next)
@@ -364,13 +432,13 @@ void rangetostring(object *pl,char *obuf)
 /**
  * Sets player title.
  */
-void set_title(object *pl,char *buf)
+void set_title(object *pl, char *buf)
 {
     /* Eneq(@csd.uu.se): Let players define their own titles. */
     if (pl->contr->own_title[0]=='\0')
 	sprintf(buf,"Player: %s the %s",pl->name,pl->contr->title);
     else
-	sprintf(buf,"Player: %s the %s",pl->name,pl->contr->own_title);
+	sprintf(buf,"Player: %s %s",pl->name,pl->contr->own_title);
 }
 
 
@@ -523,7 +591,7 @@ void draw_magic_map(object *pl)
     }
 
     sl.buf=malloc(MAXSOCKBUF);
-    sprintf((char*)sl.buf,"magicmap %d %d %d %d ", (xmax-xmin+1), (ymax-ymin+1),
+    snprintf((char*)sl.buf, MAXSOCKBUF, "magicmap %d %d %d %d ", (xmax-xmin+1), (ymax-ymin+1),
 	    MAGIC_MAP_HALF - xmin, MAGIC_MAP_HALF - ymin);
     sl.len=strlen((char*)sl.buf);
     
@@ -552,10 +620,10 @@ void Log_Kill(const char *Who,
     char buf[MAX_BUF];
 
     if (With!=NULL) {
-	sprintf(buf,"%s\t%s\t%d\t%s\t%d\n",Who,What,WhatType,With,WithType);
+	snprintf(buf, MAX_BUF, "%s\t%s\t%d\t%s\t%d\n",Who,What,WhatType,With,WithType);
     }
     else {
-	sprintf(buf,"%s\t%s\t%d\n",Who,What,WhatType);
+	snprintf(buf,MAX_BUF, "%s\t%s\t%d\n",Who,What,WhatType);
     }
     len=strlen(buf);
     for(i=1; i<socket_info.allocated_sockets; i++) {
