@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: init.c,v 1.33 2003/12/18 20:40:44 ryo_saeba Exp $";
+ *    "$Id: init.c,v 1.37 2005/07/22 20:41:25 ryo_saeba Exp $";
  */
 
 /*
@@ -122,6 +122,8 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->EMI_smooth=0; 
     ns->look_position = 0;
     ns->update_look = 0;
+    ns->has_readable_type = 0;
+    ns->supported_readables = 0;
 
     /* we should really do some checking here - if total clients overflows
      * we need to do something more intelligent, because client id's will start
@@ -143,7 +145,7 @@ void InitConnection(NewSocket *ns, uint32 from)
      * this face.  Face 0 is sent to the client to say clear
      * face information.
      */
-    ns->faces_sent[0] = 1;
+    ns->faces_sent[0] = NS_FACESENT_FACE;
 
     ns->outputbuffer.start=0;
     ns->outputbuffer.len=0;
@@ -236,8 +238,10 @@ void init_ericserver()
  * of them needs extra valus passed.
  */
 #if defined(__osf__) || defined(hpux) || defined(sgi) || defined(NeXT) || \
-        defined(__sun__) || defined(linux) || defined(SVR4) || defined(__FreeBSD__) || \
-	defined(__OpenBSD__) || defined(WIN32) /* ---win32 add this here */
+        defined(__sun__) || defined(linux) || defined(SVR4) || \
+        defined(__FreeBSD__) || defined(__OpenBSD__) || \
+        defined(WIN32) /* ---win32 add this here */  || \
+        defined(__GNU__) /* HURD */
     {
 #ifdef WIN32
     char tmp = 1;
