@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_item_c =
- *    "$Id: item.c,v 1.28 2004/02/11 08:09:29 mwedel Exp $";
+ *    "$Id: item.c,v 1.32 2005/07/14 09:18:57 akirschbaum Exp $";
  */
 
 /*
@@ -54,9 +54,6 @@
  *
  ******************************************************************************/
 
-/** This is more or less stolen from the query_weight function. */
-#define WEIGHT(op) (op->nrof?op->weight:op->weight+op->carrying)
-
 /**
  * Adds string to socklist.
  *
@@ -65,7 +62,7 @@
  * single byte in length.  If the data is longer than that byte, it is
  * truncated approprately.
  */
-inline void add_stringlen_to_sockbuf(char *buf, SockList *sl)
+inline void add_stringlen_to_sockbuf(const char *buf, SockList *sl)
 {
     int len;
 
@@ -171,7 +168,7 @@ void esrv_draw_look(object *pl)
 
     SockList_AddInt(&sl, 0);
 
-    if (!pl->contr->socket.faces_sent[empty_face->number])
+    if (!(pl->contr->socket.faces_sent[empty_face->number]&NS_FACESENT_FACE))
 	esrv_send_face(&pl->contr->socket, empty_face->number,0);
 
     if (pl->contr->socket.look_position) {
@@ -218,7 +215,7 @@ void esrv_draw_look(object *pl)
 	    flags = query_flags (tmp);
 	    if (QUERY_FLAG(tmp, FLAG_NO_PICK))
 		flags |=  F_NOPICK;
-	    if (!pl->contr->socket.faces_sent[tmp->face->number])
+	    if (!(pl->contr->socket.faces_sent[tmp->face->number] & NS_FACESENT_FACE))
 		esrv_send_face(&pl->contr->socket, tmp->face->number,0);
 
 	    if (QUERY_FLAG(tmp,FLAG_ANIMATE) && 
@@ -310,7 +307,7 @@ void esrv_send_inventory(object *pl, object *op)
 	    flags = query_flags (tmp);
 	    if (QUERY_FLAG(tmp, FLAG_NO_PICK))
 		flags |=  F_NOPICK;
-	    if (!pl->contr->socket.faces_sent[tmp->face->number])
+	    if (!(pl->contr->socket.faces_sent[tmp->face->number] & NS_FACESENT_FACE))
 		esrv_send_face(&pl->contr->socket, tmp->face->number,0);
 	    if (QUERY_FLAG(tmp,FLAG_ANIMATE) && 
 			   !pl->contr->socket.anims_sent[tmp->animation_id])
@@ -417,11 +414,16 @@ void esrv_update_item(int flags, object *pl, object *op)
     if (flags & UPD_FLAGS)
 	SockList_AddInt(&sl, query_flags(op));
 
-    if (flags & UPD_WEIGHT)
-	SockList_AddInt(&sl, WEIGHT(op));
+    if (flags & UPD_WEIGHT) {
+	sint32 weight = WEIGHT(op);
+	SockList_AddInt(&sl, weight);
+	if (pl == op) {
+	    op->contr->last_weight = weight;
+	}
+    }
 
     if (flags & UPD_FACE) {
-	if (!pl->contr->socket.faces_sent[op->face->number])
+	if (!(pl->contr->socket.faces_sent[op->face->number] & NS_FACESENT_FACE))
 	    esrv_send_face(&pl->contr->socket, op->face->number,0);
 	SockList_AddInt(&sl, op->face->number);
     }
@@ -502,7 +504,7 @@ void esrv_send_item(object *pl, object*op)
 
     SockList_AddInt(&sl, op->env? op->env->count:0);
 
-    if (!pl->contr->socket.faces_sent[op->face->number])
+    if (!(pl->contr->socket.faces_sent[op->face->number] & NS_FACESENT_FACE))
 	esrv_send_face(&pl->contr->socket, op->face->number,0);
     if (op->env && QUERY_FLAG(op,FLAG_ANIMATE) &&
 	   !pl->contr->socket.anims_sent[op->animation_id])
@@ -616,7 +618,7 @@ void ExamineCmd(char *buf, int len,player *pl)
     object *op = esrv_get_ob_from_count(pl->ob, tag);
 
     if (!op) {
-	LOG(llevDebug, "Player '%s' tried examine the unknown object (%d)\n",
+	LOG(llevDebug, "Player '%s' tried examine the unknown object (%ld)\n",
 	    pl->ob->name, tag);
 	return;
     }
