@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c,v 1.26 2003/12/02 18:51:44 ryo_saeba Exp $";
+ *    "$Id: loop.c,v 1.29 2005/06/23 05:18:38 mwedel Exp $";
  */
 
 /*
@@ -127,8 +127,9 @@ static struct NsCmdMapping nscommands[] = {
     { "setup",		SetUp},
     { "version",	VersionCmd },
     { "toggleextendedinfos", ToggleExtendedInfos}, /*Added: tchize*/
+    { "toggleextendedtext", ToggleExtendedText},   /*Added: tchize*/
     { "asksmooth", AskSmooth},   /*Added: tchize (smoothing technologies)*/ 
-    { NULL, NULL}	/* terminator */
+    { NULL, NULL}	/* terminator (I, II & III)*/
 };
 
 /**
@@ -450,24 +451,38 @@ static void block_until_new_connection()
 	cycles++;
 	if (cycles%2 == 0)
 	    tick_the_clock();
-	if (cycles == 7) {
-	    metaserver_update();
-	    cycles=1;
-	}
+
 	FD_ZERO(&readfs);
 	FD_SET((uint32)init_sockets[0].fd, &readfs);
+
+	/* If fastclock is set, we need to seriously slow down the updates
+	 * to the metaserver as well as watchdog.  Do same for flush_old_maps() -
+	 * that is time sensitive, so there is no good reason to call it 2000 times
+	 * a second.
+	 */
 	if (settings.fastclock > 0) {
+#ifdef WATCHDOG
+	    if (cycles % 120000 == 0) {
+		watchdog();
+		flush_old_maps();
+	    }
+#endif
+	    if (cycles == 720000) {
+		metaserver_update();
+		cycles=1;
+	    }
 	    Timeout.tv_sec=0;
 	    Timeout.tv_usec=50;
 	} else {
 	    Timeout.tv_sec=60;
 	    Timeout.tv_usec=0;
+	    if (cycles == 7) {
+		metaserver_update();
+		cycles=1;
+	    }
+	    flush_old_maps();
 	}
-#ifdef WATCHDOG
-	watchdog();
-#endif
-	flush_old_maps();
-	}
+    }
     while (select(socket_info.max_filedescriptor, &readfs, NULL, NULL, &Timeout)==0);
 
     reset_sleep(); /* Or the game would go too fast */
@@ -658,6 +673,11 @@ void doeric_server()
 		 * sending them whenever they change, and probably just as useful
 		 */
 		esrv_update_stats(pl);
+		if (pl->last_weight != (uint32)-1 && pl->last_weight != WEIGHT(pl->ob)) {
+		    esrv_update_item(UPD_WEIGHT, pl->ob, pl->ob);
+		    if(pl->last_weight != WEIGHT(pl->ob))
+			LOG(llevError, "esrv_update_item(UPD_WEIGHT) did not set player weight: is %lu, should be %lu\n", (unsigned long)pl->last_weight, WEIGHT(pl->ob));
+		}
 		if (pl->ob->map && pl->ob->map->in_memory==MAP_IN_MEMORY)
 		    draw_client_map(pl->ob);
 		if (pl->socket.update_look) esrv_draw_look(pl->ob);
