@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: metaserver.pl.in,v 1.1 2001/02/15 09:05:35 tanner Exp $
+# $Id: metaserver.pl.in,v 1.3 2005/07/08 23:48:59 mwedel Exp $
 
 # Copyright 2000 by Mark Wedel.
 # This script follows the same license as crossfire (GPL).
@@ -18,15 +18,29 @@ $HTML_FILE="/var/apache/htdocs/metaserver.html";
 $CACHE_FILE="/var/tmp/meta_xfire.cache";
 
 # We remove a server after $REMOVE_SERVER number of seconds of no updates.
-# 86400 is 1 day - maybe a bit long, but at current time I would like to get
-# a more accurate of number of servers out there.  Also, I can see an
-# established server being down for a while, but I think it is still nice
-# to have that in the listing.
-$REMOVE_SERVER=86400;
+# 600 is 10 minutes - if we haven't gotten an update that fast, the server
+# is almost certainly gone/not available.  This reduces congestion when
+# someone on a dhcp connection keeps running a server and it fills
+# up a bunch of the slots.
+$REMOVE_SERVER=600;
 
 # UPDATE_SYNC determines how often we update the HTML_FILE and CACHE_FILE.
 $UPDATE_SYNC=300;
 
+# IP_INTERVAL is how often (in seconds) same IP can request metaserver
+# info.  This is to prevent DOS attacks.
+
+$IP_INTERVAL=5;
+
+# For gathering some anonymous statistics. You probably want to use
+# MRTG/RRDTOOL for generating statistics from the file.
+# -- Heikki Hokkanen, 2005-03-26
+my $STATS_FILE="/var/tmp/meta_xfire.stats";
+my $stats_updatehits = 0; # COUNTER
+my $stats_requesthits = 0; # COUNTER
+my $stats_totalplayers = 0; # GAUGE
+my $stats_totalservers = 0; # GAUGE
+
 socket(SOCKET, PF_INET, SOCK_DGRAM, getprotobyname("udp")) || 
 	die("$0: can not open udp socket: $OS_ERROR\n");
 bind(SOCKET, sockaddr_in(13326, INADDR_ANY)) ||
@@ -71,6 +85,7 @@ while (1) {
 	    ($name, $rest) = split /\|/, $data;
 	    if ($name ne "put.your.hostname.here") {
 		$data{$host} = "$host|$cur_time|$data";
+		$stats_updatehits++;
 	    }
 	}
 	if (vec($rout, fileno(SOCKET1),1)) {
@@ -80,15 +95,23 @@ while (1) {
 	    # ourselves up to someone intentionally designing something
 	    # that causes these to block, and then have us fork a huge
 	    # number of process potentially filling up our proc table.
-	    if (accept NEWSOCKET, SOCKET1) {
-		foreach $i (keys %data) {
-		    # Report idle time to client, and not last update
-		    # as we received in seconds since epoch.
-		    ($ip, $time, $rest) = split /\|/, $data{$i}, 3;
-		    $newtime = $cur_time - $time;
-		    print NEWSOCKET "$ip|$newtime|$rest\n";
+	    if ($ipaddr=accept NEWSOCKET, SOCKET1) {
+		($port, $ip ) = sockaddr_in( $ipaddr );
+		$dq = join('.',unpack('C4', $ip));
+		if ($ip_times{$dq} > time) {
+		    close(NEWSOCKET);
+ 		} else {
+		    $ip_times{$dq} = time + $IP_INTERVAL;
+		    foreach $i (keys %data) {
+		        # Report idle time to client, and not last update
+		    	# as we received in seconds since epoch.
+			($ip, $time, $rest) = split /\|/, $data{$i}, 3;
+		    	$newtime = $cur_time - $time;
+		    	print NEWSOCKET "$ip|$newtime|$rest\n";
+		    }
+		    close(NEWSOCKET);
+		    $stats_requesthits++;
 		}
-		close(NEWSOCKET);
 	    }
 	}
     }
@@ -110,7 +133,10 @@ while (1) {
 </tr>
 ';
 
+	$stats_totalplayers = 0;
+	$stats_totalservers = 0;
 	foreach $i (keys %data) {
+		$stats_totalservers++;
 	    ($ip, $time, @rest) = split /\|/, $data{$i};
 	    if ($time+$REMOVE_SERVER<$cur_time) { 
 		delete $data{$i}; 
@@ -120,6 +146,7 @@ while (1) {
 		$gmtime = gmtime($time);
 		print HTML "<tr><td>$i</td><td>$gmtime</td><td>$elapsed</td>";
 		print HTML "<td>$rest[0]</td><td>$rest[1]</td><td>$rest[2]</td><td>$rest[3]</td></tr>\n";
+		$stats_totalplayers += int($rest[2]);
 	    }
 	}
 	$gmtime = gmtime($cur_time);
@@ -134,5 +161,10 @@ All times are in GMT.<p>
 <font size=-2>Last Updated: $gmtime<font size=+2><p>";
 	close(HTML);
 	close(CACHE);
+	
+	open(STATS,">$STATS_FILE");
+	print STATS "$stats_updatehits:$stats_requesthits:$stats_totalservers:$stats_totalplayers\n";
+	close(STATS);
+	
     }
 }
